@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -133,6 +133,12 @@ Object::Object (const ant::Object &d)
   //  .. nothing else ..
 }
 
+Object::~Object ()
+{
+  //  .. nothing yet ..
+}
+
+
 Object &
 Object::operator= (const ant::Object &d)
 {
@@ -331,7 +337,7 @@ Object::seg_p2 (size_t seg_index, const db::DPoint &p)
 void
 Object::p1 (const db::DPoint &p)
 {
-  if (! p1 ().equal (p)) {
+  if (m_points.size () < 1 || ! p1 ().equal (p)) {
     if (m_points.size () < 1) {
       m_points.push_back (p);
     } else {
@@ -348,7 +354,7 @@ Object::p1 (const db::DPoint &p)
 void
 Object::p2 (const db::DPoint &p)
 {
-  if (! p2 ().equal (p)) {
+  if (m_points.size () < 2 || ! p2 ().equal (p)) {
     if (m_points.size () < 2) {
       if (m_points.empty ()) {
         m_points.push_back (db::DPoint ());
