@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -137,6 +137,11 @@ public:
    */
   Object &operator= (const ant::Object &d);
 
+  /**
+   *  @brief Destructor
+   */
+  ~Object ();
+
   /**
    *  @brief Less operator
    */
