@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -47,6 +47,7 @@ public:
   virtual void finish (bool);
 
   void keep_for_healing (const db::Polygon &poly);
+  void keep_for_healing (const db::Box &box);
 
 private:
   size_t *mp_count;
@@ -78,6 +79,15 @@ public:
     }
   }
 
+  void operator() (const db::Box &box)
+  {
+    if (m_healing && ! box.inside (mp_tile->enlarged (db::Vector (-1, -1)))) {
+      mp_receiver->keep_for_healing (box);
+    } else {
+      m_count += 1;
+    }
+  }
+
   size_t count () const
   {
     return m_count;
@@ -110,6 +120,12 @@ HealingCountingReceiver::keep_for_healing (const db::Polygon &poly)
   m_for_healing.insert (poly);
 }
 
+void
+HealingCountingReceiver::keep_for_healing (const db::Box &box)
+{
+  m_for_healing.insert (box);
+}
+
 void
 HealingCountingReceiver::finish (bool)
 {
@@ -132,7 +148,9 @@ public:
   void finish (bool /*success*/);
 
   void keep_for_healing (const db::Polygon &poly);
+  void keep_for_healing (const db::Box &box);
   void output (const db::Polygon &poly);
+  void output (const db::Box &poly);
 
 private:
   db::Layout *mp_layout;
@@ -167,6 +185,23 @@ public:
     }
   }
 
+  void operator() (const db::Box &box)
+  {
+    if (m_healing && ! box.inside (mp_tile->enlarged (db::Vector (-1, -1)))) {
+      if (mp_trans->is_complex ()) {
+        mp_receiver->keep_for_healing (*mp_trans * db::Polygon (box));
+      } else {
+        mp_receiver->keep_for_healing (*mp_trans * box);
+      }
+    } else {
+      if (mp_trans->is_complex ()) {
+        mp_receiver->output (*mp_trans * db::Polygon (box));
+      } else {
+        mp_receiver->output (*mp_trans * box);
+      }
+    }
+  }
+
 private:
   const db::Box *mp_tile;
   bool m_healing;
@@ -211,12 +246,24 @@ HealingTileLayoutOutputReceiver::keep_for_healing (const db::Polygon &poly)
   m_for_healing.insert (poly);
 }
 
+void
+HealingTileLayoutOutputReceiver::keep_for_healing (const db::Box &box)
+{
+  m_for_healing.insert (box);
+}
+
 void
 HealingTileLayoutOutputReceiver::output (const db::Polygon &poly)
 {
   mp_cell->shapes (m_layer).insert (poly);
 }
 
+void
+HealingTileLayoutOutputReceiver::output (const db::Box &box)
+{
+  mp_cell->shapes (m_layer).insert (box);
+}
+
 // ---------------------------------------------------------------------
 
 struct ResultDescriptor
@@ -707,6 +754,7 @@ bool run_deep_xor (const XORData &xor_data)
 {
   db::DeepShapeStore dss;
   dss.set_threads (xor_data.threads);
+  dss.set_wants_all_cells (true);  //  saves time for less cell mapping operations
 
   double dbu = std::min (xor_data.layout_a->dbu (), xor_data.layout_b->dbu ());
 
