@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -92,7 +92,8 @@ Cell::box_type Cell::ms_empty_box = Cell::box_type ();
 
 Cell::Cell (cell_index_type ci, db::Layout &l) 
   : db::Object (l.manager ()), 
-    m_cell_index (ci), mp_layout (&l), m_instances (this), m_prop_id (0), m_hier_levels (0), m_bbox_needs_update (false), m_ghost_cell (false), 
+    m_cell_index (ci), mp_layout (&l), m_instances (this), m_prop_id (0), m_hier_levels (0),
+    m_bbox_needs_update (false), m_ghost_cell (false),
     mp_last (0), mp_next (0)
 {
   //  .. nothing yet 
@@ -187,6 +188,17 @@ Cell::clear (unsigned int index)
   }
 }
 
+void
+Cell::clear (unsigned int index, unsigned int types)
+{
+  shapes_map::iterator s = m_shapes_map.find(index);
+  if (s != m_shapes_map.end() && ! s->second.empty ()) {
+    mp_layout->invalidate_bboxes (index);  //  HINT: must come before the change is done!
+    s->second.clear (types);
+    m_bbox_needs_update = true;
+  }
+}
+
 Cell::shapes_type &
 Cell::shapes (unsigned int index) 
 {
@@ -344,6 +356,20 @@ Cell::copy (unsigned int src, unsigned int dest)
   }
 }
 
+void
+Cell::copy (unsigned int src, unsigned int dest, unsigned int types)
+{
+  if (src != dest) {
+    shapes (dest).insert (shapes (src), types);
+  } else {
+    //  When duplicating the layer, first create a copy to avoid problems with non-stable containers
+    //  Hint: using the assignment and not the copy ctor does not copy the db::Manager association.
+    db::Shapes shape_copy;
+    shape_copy.insert (shapes (src), types);
+    shapes (dest).insert (shape_copy);
+  }
+}
+
 void
 Cell::move (unsigned int src, unsigned int dest)
 {
@@ -353,6 +379,15 @@ Cell::move (unsigned int src, unsigned int dest)
   }
 }
 
+void
+Cell::move (unsigned int src, unsigned int dest, unsigned int types)
+{
+  if (src != dest) {
+    copy (src, dest, types);
+    clear (src, types);
+  }
+}
+
 void
 Cell::swap (unsigned int i1, unsigned int i2)
 {
@@ -666,7 +701,7 @@ Cell::clear_parent_insts (size_t sz)
 void 
 Cell::sort_child_insts ()
 {
-  m_instances.sort_child_insts ();
+  m_instances.sort_child_insts (false);
 }
 
 std::pair<bool, db::pcell_id_type> 
@@ -710,9 +745,9 @@ Cell::change_pcell_parameters (const instance_type &ref, const std::vector<tl::V
 }
 
 void 
-Cell::sort_inst_tree ()
+Cell::sort_inst_tree (bool force)
 {
-  m_instances.sort_inst_tree (mp_layout);
+  m_instances.sort_inst_tree (mp_layout, force);
 
   //  update the number of hierarchy levels
   m_hier_levels = count_hier_levels ();
