@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -176,6 +176,7 @@ DeepEdges::~DeepEdges ()
 DeepEdges::DeepEdges (const DeepEdges &other)
   : MutableEdges (other), DeepShapeCollectionDelegateBase (other),
     m_merged_edges_valid (other.m_merged_edges_valid),
+    m_merged_edges_boc_hash (other.m_merged_edges_boc_hash),
     m_is_merged (other.m_is_merged)
 {
   if (m_merged_edges_valid) {
@@ -192,6 +193,7 @@ DeepEdges::operator= (const DeepEdges &other)
     DeepShapeCollectionDelegateBase::operator= (other);
 
     m_merged_edges_valid = other.m_merged_edges_valid;
+    m_merged_edges_boc_hash = other.m_merged_edges_boc_hash;
     m_is_merged = other.m_is_merged;
     if (m_merged_edges_valid) {
       m_merged_edges = other.m_merged_edges.copy ();
@@ -205,6 +207,7 @@ DeepEdges::operator= (const DeepEdges &other)
 void DeepEdges::init ()
 {
   m_merged_edges_valid = false;
+  m_merged_edges_boc_hash = 0;
   m_merged_edges = db::DeepLayer ();
   m_is_merged = false;
 }
@@ -244,22 +247,21 @@ static void transform_deep_layer (db::DeepLayer &deep_layer, const Trans &t)
 
     //  Plain move
 
+    db::Layout &layout = deep_layer.layout ();
+
     //  build cell variants for different orientations
     db::OrientationReducer same_orientation;
 
     db::VariantsCollectorBase vars (&same_orientation);
-    vars.collect (deep_layer.layout (), deep_layer.initial_cell ());
-    deep_layer.separate_variants (vars);
+    vars.collect (&layout, deep_layer.initial_cell ().cell_index ());
+    vars.separate_variants ();
 
     //  process the variants
-    db::Layout &layout = deep_layer.layout ();
 
     for (db::Layout::iterator c = layout.begin (); c != layout.end (); ++c) {
 
-      const std::map<db::ICplxTrans, size_t> &v = vars.variants (c->cell_index ());
-      tl_assert (v.size () == size_t (1));
-
-      db::Trans tr (v.begin ()->first.inverted () * t.disp ());
+      const db::ICplxTrans &tv = vars.single_variant_transformation (c->cell_index ());
+      db::ICplxTrans tr (tv.inverted () * t.disp ());
 
       db::Shapes &shapes = c->shapes (deep_layer.layer ());
       db::Shapes new_shapes (layout.manager (), c.operator-> (), layout.is_editable ());
@@ -459,6 +461,7 @@ void DeepEdges::apply_property_translator (const db::PropertiesTranslator &pt)
   DeepShapeCollectionDelegateBase::apply_property_translator (pt);
 
   m_merged_edges_valid = false;
+  m_merged_edges_boc_hash = 0;
   m_merged_edges = db::DeepLayer ();
 }
 
@@ -638,10 +641,16 @@ DeepEdges::merged_deep_layer () const
   }
 }
 
+bool
+DeepEdges::merged_edges_available () const
+{
+  return m_is_merged || (m_merged_edges_valid && m_merged_edges_boc_hash == deep_layer ().breakout_cells_hash ());
+}
+
 void
 DeepEdges::ensure_merged_edges_valid () const
 {
-  if (! m_merged_edges_valid) {
+  if (! m_merged_edges_valid || (! m_is_merged && m_merged_edges_boc_hash != deep_layer ().breakout_cells_hash ())) {
 
     if (m_is_merged) {
 
@@ -660,7 +669,7 @@ DeepEdges::ensure_merged_edges_valid () const
       db::Connectivity conn;
       conn.connect (deep_layer ());
       hc.set_base_verbosity (base_verbosity() + 10);
-      hc.build (layout, deep_layer ().initial_cell (), conn);
+      hc.build (layout, deep_layer ().initial_cell (), conn, 0, deep_layer ().breakout_cells ());
 
       //  collect the clusters and merge them into larger edges
       //  NOTE: using the ClusterMerger we merge bottom-up forming bigger and bigger polygons. This is
@@ -684,6 +693,7 @@ DeepEdges::ensure_merged_edges_valid () const
     }
 
     m_merged_edges_valid = true;
+    m_merged_edges_boc_hash = deep_layer ().breakout_cells_hash ();
 
   }
 }
@@ -693,6 +703,7 @@ DeepEdges::set_is_merged (bool f)
 {
   m_is_merged = f;
   m_merged_edges_valid = false;
+  m_merged_edges_boc_hash = 0;
   m_merged_edges = db::DeepLayer ();
 }
 
@@ -739,8 +750,8 @@ DeepEdges::length_type DeepEdges::length (const db::Box &box) const
     const db::DeepLayer &edges = merged_deep_layer ();
 
     db::MagnificationReducer red;
-    db::cell_variants_collector<db::MagnificationReducer> vars (red);
-    vars.collect (edges.layout (), edges.initial_cell ());
+    db::cell_variants_statistics<db::MagnificationReducer> vars (red);
+    vars.collect (&edges.layout (), edges.initial_cell ().cell_index ());
 
     DeepEdges::length_type l = 0;
 
@@ -751,7 +762,7 @@ DeepEdges::length_type DeepEdges::length (const db::Box &box) const
         lc += s->edge ().length ();
       }
       const std::map<db::ICplxTrans, size_t> &vv = vars.variants (*c);
-      for (std::map<db::ICplxTrans, size_t>::const_iterator v = vv.begin (); v != vv.end (); ++v) {
+      for (auto v = vv.begin (); v != vv.end (); ++v) {
         double mag = v->first.mag ();
         l += v->second * lc * mag;
       }
@@ -812,21 +823,21 @@ DeepEdges *
 DeepEdges::apply_filter (const EdgeFilterBase &filter) const
 {
   const db::DeepLayer &edges = filter.requires_raw_input () ? deep_layer () : merged_deep_layer ();
+  db::Layout &layout = const_cast<db::Layout &> (edges.layout ());
 
   std::unique_ptr<VariantsCollectorBase> vars;
   if (filter.vars ()) {
 
     vars.reset (new db::VariantsCollectorBase (filter.vars ()));
 
-    vars->collect (edges.layout (), edges.initial_cell ());
+    vars->collect (&layout, edges.initial_cell ().cell_index ());
 
     if (filter.wants_variants ()) {
-      const_cast<db::DeepLayer &> (edges).separate_variants (*vars);
+      vars->separate_variants ();
     }
 
   }
 
-  db::Layout &layout = const_cast<db::Layout &> (edges.layout ());
   std::map<db::cell_index_type, std::map<db::ICplxTrans, db::Shapes> > to_commit;
 
   std::unique_ptr<db::DeepEdges> res (new db::DeepEdges (edges.derived ()));
@@ -836,20 +847,18 @@ DeepEdges::apply_filter (const EdgeFilterBase &filter) const
 
     if (vars.get ()) {
 
-      const std::map<db::ICplxTrans, size_t> &vv = vars->variants (c->cell_index ());
-      for (std::map<db::ICplxTrans, size_t>::const_iterator v = vv.begin (); v != vv.end (); ++v) {
+      const std::set<db::ICplxTrans> &vv = vars->variants (c->cell_index ());
+      for (auto v = vv.begin (); v != vv.end (); ++v) {
 
         db::Shapes *st;
         if (vv.size () == 1) {
           st = & c->shapes (res->deep_layer ().layer ());
         } else {
-          st = & to_commit [c->cell_index ()] [v->first];
+          st = & to_commit [c->cell_index ()] [*v];
         }
 
-        const db::ICplxTrans &tr = v->first;
-
         for (db::Shapes::shape_iterator si = s.begin (db::ShapeIterator::Edges); ! si.at_end (); ++si) {
-          if (filter.selected (si->edge ().transformed (tr))) {
+          if (filter.selected (si->edge ().transformed (*v))) {
             st->insert (*si);
           }
         }
@@ -871,7 +880,7 @@ DeepEdges::apply_filter (const EdgeFilterBase &filter) const
   }
 
   if (! to_commit.empty () && vars.get ()) {
-    res->deep_layer ().commit_shapes (*vars, to_commit);
+    vars->commit_shapes (res->deep_layer ().layer (), to_commit);
   }
 
   if (! filter.requires_raw_input ()) {
@@ -912,7 +921,7 @@ DeepEdges::and_or_not_with (const DeepEdges *other, EdgeBoolOp op) const
 
   db::EdgeBoolAndOrNotLocalOperation local_op (op);
 
-  db::local_processor<db::Edge, db::Edge, db::Edge> proc (const_cast<db::Layout *> (&deep_layer ().layout ()), const_cast<db::Cell *> (&deep_layer ().initial_cell ()), &other->deep_layer ().layout (), &other->deep_layer ().initial_cell ());
+  db::local_processor<db::Edge, db::Edge, db::Edge> proc (const_cast<db::Layout *> (&deep_layer ().layout ()), const_cast<db::Cell *> (&deep_layer ().initial_cell ()), &other->deep_layer ().layout (), &other->deep_layer ().initial_cell (), deep_layer ().breakout_cells (), other->deep_layer ().breakout_cells ());
   proc.set_base_verbosity (base_verbosity ());
   proc.set_threads (deep_layer ().store ()->threads ());
   proc.set_area_ratio (deep_layer ().store ()->max_area_ratio ());
@@ -939,7 +948,7 @@ DeepEdges::edge_region_op (const DeepRegion *other, EdgePolygonOp::mode_t mode,
 
   db::EdgeToPolygonLocalOperation op (mode, include_borders);
 
-  db::local_processor<db::Edge, db::PolygonRef, db::Edge> proc (const_cast<db::Layout *> (&deep_layer ().layout ()), const_cast<db::Cell *> (&deep_layer ().initial_cell ()), &other->deep_layer ().layout (), &other->deep_layer ().initial_cell ());
+  db::local_processor<db::Edge, db::PolygonRef, db::Edge> proc (const_cast<db::Layout *> (&deep_layer ().layout ()), const_cast<db::Cell *> (&deep_layer ().initial_cell ()), &other->deep_layer ().layout (), &other->deep_layer ().initial_cell (), deep_layer ().breakout_cells (), other->deep_layer ().breakout_cells ());
   proc.set_base_verbosity (base_verbosity ());
   proc.set_threads (deep_layer ().store ()->threads ());
   proc.set_area_ratio (deep_layer ().store ()->max_area_ratio ());
@@ -1256,13 +1265,12 @@ RegionDelegate *DeepEdges::extended (coord_type ext_b, coord_type ext_e, coord_t
   std::unique_ptr<db::DeepRegion> res (new db::DeepRegion (edges.derived ()));
 
   db::Layout &layout = const_cast<db::Layout &> (edges.layout ());
-  db::Cell &top_cell = const_cast<db::Cell &> (edges.initial_cell ());
 
   //  TODO: there is a special case when we'd need a MagnificationAndOrientationReducer:
   //  dots formally don't have an orientation, hence the interpretation is x and y.
   db::MagnificationReducer red;
   db::cell_variants_collector<db::MagnificationReducer> vars (red);
-  vars.collect (edges.layout (), edges.initial_cell ());
+  vars.collect (&layout, edges.initial_cell ().cell_index ());
 
   std::map<db::cell_index_type, std::map<db::ICplxTrans, db::Shapes> > to_commit;
 
@@ -1274,19 +1282,19 @@ RegionDelegate *DeepEdges::extended (coord_type ext_b, coord_type ext_e, coord_t
     db::Connectivity conn (db::Connectivity::EdgesConnectByPoints);
     conn.connect (edges);
     hc.set_base_verbosity (base_verbosity () + 10);
-    hc.build (layout, edges.initial_cell (), conn);
+    hc.build (layout, edges.initial_cell (), conn, 0, edges.breakout_cells ());
 
     //  TODO: iterate only over the called cells?
     for (db::Layout::iterator c = layout.begin (); c != layout.end (); ++c) {
 
-      const std::map<db::ICplxTrans, size_t> &vv = vars.variants (c->cell_index ());
-      for (std::map<db::ICplxTrans, size_t>::const_iterator v = vv.begin (); v != vv.end (); ++v) {
+      const std::set<db::ICplxTrans> &vv = vars.variants (c->cell_index ());
+      for (auto v = vv.begin (); v != vv.end (); ++v) {
 
         db::Shapes *out;
         if (vv.size () == 1) {
           out = & c->shapes (res->deep_layer ().layer ());
         } else {
-          out = & to_commit [c->cell_index ()][v->first];
+          out = & to_commit [c->cell_index ()][*v];
         }
 
         const db::connected_clusters<db::Edge> &cc = hc.clusters_per_cell (c->cell_index ());
@@ -1295,12 +1303,12 @@ RegionDelegate *DeepEdges::extended (coord_type ext_b, coord_type ext_e, coord_t
           if (cc.is_root (*cl)) {
 
             PolygonRefToShapesGenerator prgen (&layout, out);
-            polygon_transformation_filter<db::ICplxTrans> ptrans (&prgen, v->first.inverted ());
+            polygon_transformation_filter<db::ICplxTrans> ptrans (&prgen, v->inverted ());
             JoinEdgesCluster jec (&ptrans, ext_b, ext_e, ext_o, ext_i);
 
             std::list<db::Edge> heap;
             for (db::recursive_cluster_shape_iterator<db::Edge> rcsi (hc, edges.layer (), c->cell_index (), *cl); ! rcsi.at_end (); ++rcsi) {
-              heap.push_back (rcsi->transformed (v->first * rcsi.trans ()));
+              heap.push_back (rcsi->transformed (*v * rcsi.trans ()));
               jec.add (&heap.back (), 0);
             }
 
@@ -1318,20 +1326,20 @@ RegionDelegate *DeepEdges::extended (coord_type ext_b, coord_type ext_e, coord_t
 
     for (db::Layout::iterator c = layout.begin (); c != layout.end (); ++c) {
 
-      const std::map<db::ICplxTrans, size_t> &vv = vars.variants (c->cell_index ());
-      for (std::map<db::ICplxTrans, size_t>::const_iterator v = vv.begin (); v != vv.end (); ++v) {
+      const std::set<db::ICplxTrans> &vv = vars.variants (c->cell_index ());
+      for (auto v = vv.begin (); v != vv.end (); ++v) {
 
         db::Shapes *out;
         if (vv.size () == 1) {
           out = & c->shapes (res->deep_layer ().layer ());
         } else {
-          out = & to_commit [c->cell_index ()][v->first];
+          out = & to_commit [c->cell_index ()][*v];
         }
 
         PolygonRefToShapesGenerator prgen (&layout, out);
         for (db::Shapes::shape_iterator si = c->shapes (edges.layer ()).begin (db::ShapeIterator::Edges); ! si.at_end (); ++si) {
           prgen.set_prop_id (si->prop_id ());
-          prgen.put (extended_edge (si->edge ().transformed (v->first), ext_b, ext_e, ext_o, ext_i).transformed (v->first.inverted ()));
+          prgen.put (extended_edge (si->edge ().transformed (*v), ext_b, ext_e, ext_o, ext_i).transformed (v->inverted ()));
         }
 
       }
@@ -1341,7 +1349,7 @@ RegionDelegate *DeepEdges::extended (coord_type ext_b, coord_type ext_e, coord_t
   }
 
   //  propagate results from variants
-  vars.commit_shapes (layout, top_cell, res->deep_layer ().layer (), to_commit);
+  vars.commit_shapes (res->deep_layer ().layer (), to_commit);
 
   return res.release ();
 }
@@ -1367,7 +1375,7 @@ public:
     return 1;
   }
 
-  virtual void do_compute_local (db::Layout * /*layout*/, const shape_interactions<db::Edge, db::Edge> &interactions, std::vector<std::unordered_set<db::Edge> > &results, size_t /*max_vertex_count*/, double /*area_ratio*/) const
+  virtual void do_compute_local (db::Layout * /*layout*/, db::Cell * /*cell*/, const shape_interactions<db::Edge, db::Edge> &interactions, std::vector<std::unordered_set<db::Edge> > &results, const db::LocalProcessorBase * /*proc*/) const
   {
     tl_assert (results.size () == (m_output_mode == Both ? 2 : 1));
 
@@ -1460,7 +1468,7 @@ public:
     return 1;
   }
 
-  virtual void do_compute_local (db::Layout * /*layout*/, const shape_interactions<db::Edge, db::Edge> &interactions, std::vector<std::unordered_set<db::Edge> > &results, size_t /*max_vertex_count*/, double /*area_ratio*/) const
+  virtual void do_compute_local (db::Layout * /*layout*/, db::Cell * /*cell*/, const shape_interactions<db::Edge, db::Edge> &interactions, std::vector<std::unordered_set<db::Edge> > &results, const db::LocalProcessorBase * /*proc*/) const
   {
     tl_assert (results.size () == 1);
     std::unordered_set<db::Edge> &result = results.front ();
@@ -1517,7 +1525,7 @@ public:
     return 1;
   }
 
-  virtual void do_compute_local (db::Layout * /*layout*/, const shape_interactions<db::Edge, db::PolygonRef> &interactions, std::vector<std::unordered_set<db::Edge> > &results, size_t /*max_vertex_count*/, double /*area_ratio*/) const
+  virtual void do_compute_local (db::Layout * /*layout*/, db::Cell * /*cell*/, const shape_interactions<db::Edge, db::PolygonRef> &interactions, std::vector<std::unordered_set<db::Edge> > &results, const db::LocalProcessorBase * /*proc*/) const
   {
     tl_assert (results.size () == size_t (m_output_mode == Both ? 2 : 1));
 
@@ -1657,7 +1665,7 @@ public:
     return 1;
   }
 
-  virtual void do_compute_local (db::Layout *layout, const shape_interactions<db::Edge, db::PolygonRef> &interactions, std::vector<std::unordered_set<db::PolygonRef> > &results, size_t /*max_vertex_count*/, double /*area_ratio*/) const
+  virtual void do_compute_local (db::Layout *layout, db::Cell * /*cell*/, const shape_interactions<db::Edge, db::PolygonRef> &interactions, std::vector<std::unordered_set<db::PolygonRef> > &results, const db::LocalProcessorBase * /*proc*/) const
   {
     tl_assert (results.size () == 1);
     std::unordered_set<db::PolygonRef> &result = results.front ();
@@ -1717,7 +1725,7 @@ DeepEdges::selected_interacting_generic (const Region &other, EdgeInteractionMod
 
   db::Edge2PolygonInteractingLocalOperation op (mode, inverse ? db::Edge2PolygonInteractingLocalOperation::Inverse : db::Edge2PolygonInteractingLocalOperation::Normal);
 
-  db::local_processor<db::Edge, db::PolygonRef, db::Edge> proc (const_cast<db::Layout *> (&edges.layout ()), const_cast<db::Cell *> (&edges.initial_cell ()), &other_deep->deep_layer ().layout (), &other_deep->deep_layer ().initial_cell ());
+  db::local_processor<db::Edge, db::PolygonRef, db::Edge> proc (const_cast<db::Layout *> (&edges.layout ()), const_cast<db::Cell *> (&edges.initial_cell ()), &other_deep->deep_layer ().layout (), &other_deep->deep_layer ().initial_cell (), edges.breakout_cells (), other_deep->deep_layer ().breakout_cells ());
   proc.set_base_verbosity (base_verbosity ());
   proc.set_threads (edges.store ()->threads ());
 
@@ -1749,7 +1757,7 @@ DeepEdges::selected_interacting_pair_generic (const Region &other, EdgeInteracti
 
   db::Edge2PolygonInteractingLocalOperation op (mode, db::Edge2PolygonInteractingLocalOperation::Both);
 
-  db::local_processor<db::Edge, db::PolygonRef, db::Edge> proc (const_cast<db::Layout *> (&edges.layout ()), const_cast<db::Cell *> (&edges.initial_cell ()), &other_deep->deep_layer ().layout (), &other_deep->deep_layer ().initial_cell ());
+  db::local_processor<db::Edge, db::PolygonRef, db::Edge> proc (const_cast<db::Layout *> (&edges.layout ()), const_cast<db::Cell *> (&edges.initial_cell ()), &other_deep->deep_layer ().layout (), &other_deep->deep_layer ().initial_cell (), edges.breakout_cells (), other_deep->deep_layer ().breakout_cells ());
   proc.set_base_verbosity (base_verbosity ());
   proc.set_threads (edges.store ()->threads ());
 
@@ -1775,7 +1783,7 @@ DeepEdges::selected_interacting_generic (const Edges &other, EdgeInteractionMode
 
   db::Edge2EdgeInteractingLocalOperation op (mode, inverse ? db::Edge2EdgeInteractingLocalOperation::Inverse : db::Edge2EdgeInteractingLocalOperation::Normal);
 
-  db::local_processor<db::Edge, db::Edge, db::Edge> proc (const_cast<db::Layout *> (&edges.layout ()), const_cast<db::Cell *> (&edges.initial_cell ()), &other_deep->deep_layer ().layout (), &other_deep->deep_layer ().initial_cell ());
+  db::local_processor<db::Edge, db::Edge, db::Edge> proc (const_cast<db::Layout *> (&edges.layout ()), const_cast<db::Cell *> (&edges.initial_cell ()), &other_deep->deep_layer ().layout (), &other_deep->deep_layer ().initial_cell (), edges.breakout_cells (), other_deep->deep_layer ().breakout_cells ());
   proc.set_base_verbosity (base_verbosity ());
   proc.set_threads (edges.store ()->threads ());
 
@@ -1807,7 +1815,7 @@ DeepEdges::selected_interacting_pair_generic (const Edges &other, EdgeInteractio
 
   db::Edge2EdgeInteractingLocalOperation op (mode, db::Edge2EdgeInteractingLocalOperation::Both);
 
-  db::local_processor<db::Edge, db::Edge, db::Edge> proc (const_cast<db::Layout *> (&edges.layout ()), const_cast<db::Cell *> (&edges.initial_cell ()), &other_deep->deep_layer ().layout (), &other_deep->deep_layer ().initial_cell ());
+  db::local_processor<db::Edge, db::Edge, db::Edge> proc (const_cast<db::Layout *> (&edges.layout ()), const_cast<db::Cell *> (&edges.initial_cell ()), &other_deep->deep_layer ().layout (), &other_deep->deep_layer ().initial_cell (), edges.breakout_cells (), other_deep->deep_layer ().breakout_cells ());
   proc.set_base_verbosity (base_verbosity ());
   proc.set_threads (edges.store ()->threads ());
 
@@ -1833,7 +1841,7 @@ RegionDelegate *DeepEdges::pull_generic (const Region &other) const
 
   db::Edge2PolygonPullLocalOperation op;
 
-  db::local_processor<db::Edge, db::PolygonRef, db::PolygonRef> proc (const_cast<db::Layout *> (&edges.layout ()), const_cast<db::Cell *> (&edges.initial_cell ()), &other_polygons.layout (), &other_polygons.initial_cell ());
+  db::local_processor<db::Edge, db::PolygonRef, db::PolygonRef> proc (const_cast<db::Layout *> (&edges.layout ()), const_cast<db::Cell *> (&edges.initial_cell ()), &other_polygons.layout (), &other_polygons.initial_cell (), edges.breakout_cells (), other_polygons.breakout_cells ());
   proc.set_base_verbosity (base_verbosity ());
   proc.set_threads (edges.store ()->threads ());
 
@@ -1859,7 +1867,7 @@ EdgesDelegate *DeepEdges::pull_generic (const Edges &other) const
 
   db::Edge2EdgePullLocalOperation op;
 
-  db::local_processor<db::Edge, db::Edge, db::Edge> proc (const_cast<db::Layout *> (&edges.layout ()), const_cast<db::Cell *> (&edges.initial_cell ()), &other_edges.layout (), &other_edges.initial_cell ());
+  db::local_processor<db::Edge, db::Edge, db::Edge> proc (const_cast<db::Layout *> (&edges.layout ()), const_cast<db::Cell *> (&edges.initial_cell ()), &other_edges.layout (), &other_edges.initial_cell (), edges.breakout_cells (), other_edges.breakout_cells ());
   proc.set_base_verbosity (base_verbosity ());
   proc.set_threads (edges.store ()->threads ());
 
@@ -1888,7 +1896,7 @@ EdgesDelegate *DeepEdges::in (const Edges &other, bool invert) const
 
   db::ContainedEdgesLocalOperation op (invert ? Negative : Positive);
 
-  db::local_processor<db::Edge, db::Edge, db::Edge> proc (const_cast<db::Layout *> (&edges.layout ()), const_cast<db::Cell *> (&edges.initial_cell ()), &other_deep->deep_layer ().layout (), &other_deep->deep_layer ().initial_cell ());
+  db::local_processor<db::Edge, db::Edge, db::Edge> proc (const_cast<db::Layout *> (&edges.layout ()), const_cast<db::Cell *> (&edges.initial_cell ()), &other_deep->deep_layer ().layout (), &other_deep->deep_layer ().initial_cell (), edges.breakout_cells (), other_deep->deep_layer ().breakout_cells ());
   proc.set_base_verbosity (base_verbosity ());
   proc.set_threads (edges.store ()->threads ());
 
@@ -1919,7 +1927,7 @@ std::pair<EdgesDelegate *, EdgesDelegate *> DeepEdges::in_and_out (const Edges &
 
   db::ContainedEdgesLocalOperation op (PositiveAndNegative);
 
-  db::local_processor<db::Edge, db::Edge, db::Edge> proc (const_cast<db::Layout *> (&edges.layout ()), const_cast<db::Cell *> (&edges.initial_cell ()), &other_deep->deep_layer ().layout (), &other_deep->deep_layer ().initial_cell ());
+  db::local_processor<db::Edge, db::Edge, db::Edge> proc (const_cast<db::Layout *> (&edges.layout ()), const_cast<db::Cell *> (&edges.initial_cell ()), &other_deep->deep_layer ().layout (), &other_deep->deep_layer ().initial_cell (), edges.breakout_cells (), other_deep->deep_layer ().breakout_cells ());
   proc.set_base_verbosity (base_verbosity ());
   proc.set_threads (edges.store ()->threads ());
 
@@ -1941,12 +1949,16 @@ public:
     //  .. nothing yet ..
   }
 
-  virtual void do_compute_local (db::Layout * /*layout*/, const shape_interactions<db::Edge, db::Edge> &interactions, std::vector<std::unordered_set<db::EdgePair> > &results, size_t /*max_vertex_count*/, double /*area_ratio*/) const
+  virtual void do_compute_local (db::Layout * /*layout*/, db::Cell *cell, const shape_interactions<db::Edge, db::Edge> &interactions, std::vector<std::unordered_set<db::EdgePair> > &results, const db::LocalProcessorBase *proc) const
   {
     tl_assert (results.size () == 1);
     std::unordered_set<db::EdgePair> &result = results.front ();
 
-    edge2edge_check_for_edges<std::unordered_set<db::EdgePair> > edge_check (m_check, result, m_has_other);
+    //  implement check in local coordinates in the presence of magnification variants
+    EdgeRelationFilter check = m_check;
+    check.set_distance (proc->dist_for_cell (cell, check.distance ()));
+
+    edge2edge_check_for_edges<std::unordered_set<db::EdgePair> > edge_check (check, result, m_has_other);
 
     db::box_scanner<db::Edge, size_t> scanner;
     std::set<db::Edge> others;
@@ -1989,7 +2001,7 @@ public:
 
     }
 
-    scanner.process (edge_check, m_check.distance (), db::box_convert<db::Edge> ());
+    scanner.process (edge_check, check.distance (), db::box_convert<db::Edge> ());
   }
 
   virtual db::Coord dist () const
@@ -2008,9 +2020,15 @@ public:
     return tl::to_string (tr ("Generic DRC check"));
   }
 
+  virtual const db::TransformationReducer *vars () const
+  {
+    return &m_vars;
+  }
+
 private:
   EdgeRelationFilter m_check;
   bool m_has_other;
+  db::MagnificationReducer m_vars;
 };
 
 }
@@ -2042,7 +2060,9 @@ DeepEdges::run_check (db::edge_relation_type rel, const Edges *other, db::Coord
   db::local_processor<db::Edge, db::Edge, db::EdgePair> proc (const_cast<db::Layout *> (&edges.layout ()),
                                                               const_cast<db::Cell *> (&edges.initial_cell ()),
                                                               other_deep ? &other_deep->deep_layer ().layout () : const_cast<db::Layout *> (&edges.layout ()),
-                                                              other_deep ? &other_deep->deep_layer ().initial_cell () : const_cast<db::Cell *> (&edges.initial_cell ()));
+                                                              other_deep ? &other_deep->deep_layer ().initial_cell () : const_cast<db::Cell *> (&edges.initial_cell ()),
+                                                              edges.breakout_cells (),
+                                                              other_deep ? other_deep->deep_layer ().breakout_cells () : 0);
 
   proc.set_base_verbosity (base_verbosity ());
   proc.set_threads (edges.store ()->threads ());
