@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -179,6 +179,7 @@ DeepRegion::~DeepRegion ()
 DeepRegion::DeepRegion (const DeepRegion &other)
   : MutableRegion (other), DeepShapeCollectionDelegateBase (other),
     m_merged_polygons_valid (other.m_merged_polygons_valid),
+    m_merged_polygons_boc_hash (other.m_merged_polygons_boc_hash),
     m_is_merged (other.m_is_merged)
 {
   if (m_merged_polygons_valid) {
@@ -195,6 +196,7 @@ DeepRegion::operator= (const DeepRegion &other)
     DeepShapeCollectionDelegateBase::operator= (other);
 
     m_merged_polygons_valid = other.m_merged_polygons_valid;
+    m_merged_polygons_boc_hash = other.m_merged_polygons_boc_hash;
     m_is_merged = other.m_is_merged;
     if (m_merged_polygons_valid) {
       m_merged_polygons = other.m_merged_polygons.copy ();
@@ -208,6 +210,7 @@ DeepRegion::operator= (const DeepRegion &other)
 void DeepRegion::init ()
 {
   m_merged_polygons_valid = false;
+  m_merged_polygons_boc_hash = 0;
   m_merged_polygons = db::DeepLayer ();
   m_is_merged = false;
 }
@@ -252,22 +255,20 @@ static void transform_deep_layer (db::DeepLayer &deep_layer, const Trans &t)
 
     //  Plain move
 
-    //  build cell variants for different orientations
-    db::OrientationReducer same_orientation;
+    //  build cell variants for different orientations and magnifications
+    db::MagnificationAndOrientationReducer same_orientation;
 
     db::VariantsCollectorBase vars (&same_orientation);
-    vars.collect (deep_layer.layout (), deep_layer.initial_cell ());
-    deep_layer.separate_variants (vars);
+    vars.collect (&deep_layer.layout (), deep_layer.initial_cell ().cell_index ());
+    vars.separate_variants ();
 
     //  process the variants
     db::Layout &layout = deep_layer.layout ();
 
     for (db::Layout::iterator c = layout.begin (); c != layout.end (); ++c) {
 
-      const std::map<db::ICplxTrans, size_t> &v = vars.variants (c->cell_index ());
-      tl_assert (v.size () == size_t (1));
-
-      db::Trans tr (v.begin ()->first.inverted () * t.disp ());
+      const db::ICplxTrans &tv = vars.single_variant_transformation (c->cell_index ());
+      db::ICplxTrans tr (tv.inverted () * t.disp ());
 
       db::Shapes &shapes = c->shapes (deep_layer.layer ());
       db::Shapes new_shapes (layout.manager (), c.operator-> (), layout.is_editable ());
@@ -486,6 +487,7 @@ void DeepRegion::apply_property_translator (const db::PropertiesTranslator &pt)
   DeepShapeCollectionDelegateBase::apply_property_translator (pt);
 
   m_merged_polygons_valid = false;
+  m_merged_polygons_boc_hash = 0;
   m_merged_polygons = db::DeepLayer ();
 }
 
@@ -685,13 +687,13 @@ DeepRegion::merged_deep_layer () const
 bool
 DeepRegion::merged_polygons_available () const
 {
-  return m_is_merged || m_merged_polygons_valid;
+  return m_is_merged || (m_merged_polygons_valid && m_merged_polygons_boc_hash == deep_layer ().breakout_cells_hash ());
 }
 
 void
 DeepRegion::ensure_merged_polygons_valid () const
 {
-  if (! m_merged_polygons_valid) {
+  if (! m_merged_polygons_valid || (! m_is_merged && m_merged_polygons_boc_hash != deep_layer ().breakout_cells_hash ())) {
 
     if (m_is_merged) {
 
@@ -710,7 +712,7 @@ DeepRegion::ensure_merged_polygons_valid () const
       db::Connectivity conn;
       conn.connect (deep_layer ());
       hc.set_base_verbosity (base_verbosity () + 10);
-      hc.build (layout, deep_layer ().initial_cell (), conn, 0, 0, true /*separate_attributes*/);
+      hc.build (layout, deep_layer ().initial_cell (), conn, 0, deep_layer ().breakout_cells (), true /*separate_attributes*/);
 
       //  collect the clusters and merge them into big polygons
       //  NOTE: using the ClusterMerger we merge bottom-up forming bigger and bigger polygons. This is
@@ -734,6 +736,7 @@ DeepRegion::ensure_merged_polygons_valid () const
     }
 
     m_merged_polygons_valid = true;
+    m_merged_polygons_boc_hash = deep_layer ().breakout_cells_hash ();
 
   }
 }
@@ -743,6 +746,7 @@ DeepRegion::set_is_merged (bool f)
 {
   m_is_merged = f;
   m_merged_polygons_valid = false;
+  m_merged_polygons_boc_hash = 0;
   m_merged_polygons = db::DeepLayer ();
 }
 
@@ -962,10 +966,31 @@ DeepRegion::xor_with (const Region &other, db::PropertyConstraint property_const
 
     //  Implement XOR as (A-B)+(B-A) - only this implementation
     //  is compatible with the local processor scheme
-    DeepLayer n1 (and_or_not_with (other_deep, false, property_constraint));
-    DeepLayer n2 (other_deep->and_or_not_with (this, false, property_constraint));
 
+    //  Prepare a version of "other_deep" that is mapped into the hierarchy space
+    //  of "this"
+    std::unique_ptr<DeepRegion> other_deep_mapped;
+    if (&other_deep->deep_layer ().layout () == &deep_layer ().layout ()) {
+      //  shallow copy for reconfiguration (progress etc.)
+      other_deep_mapped.reset (new DeepRegion (other_deep->deep_layer ()));
+    } else {
+      //  deep copy with mapped hierarchy
+      other_deep_mapped.reset (new DeepRegion (deep_layer ().derived ()));
+      other_deep_mapped->deep_layer ().add_from (other_deep->deep_layer ());
+    }
+
+    other_deep_mapped->set_strict_handling (strict_handling ());
+    other_deep_mapped->set_base_verbosity (base_verbosity ());
+    if (report_progress ()) {
+      other_deep_mapped->enable_progress (progress_desc () + tl::to_string (tr (" - reverse part")));
+    } else {
+      other_deep_mapped->disable_progress ();
+    }
+
+    DeepLayer n1 (and_or_not_with (other_deep_mapped.get (), false, property_constraint));
+    DeepLayer n2 (other_deep_mapped->and_or_not_with (this, false, property_constraint));
     n1.add_from (n2);
+
     return new DeepRegion (n1);
 
   }
@@ -1090,8 +1115,8 @@ DeepRegion::area (const db::Box &box) const
 
     const db::DeepLayer &polygons = merged_deep_layer ();
 
-    db::cell_variants_collector<db::MagnificationReducer> vars;
-    vars.collect (polygons.layout (), polygons.initial_cell ());
+    db::cell_variants_statistics<db::MagnificationReducer> vars;
+    vars.collect (&polygons.layout (), polygons.initial_cell ().cell_index ());
 
     DeepRegion::area_type a = 0;
 
@@ -1102,7 +1127,7 @@ DeepRegion::area (const db::Box &box) const
         ac += s->area ();
       }
       const std::map<db::ICplxTrans, size_t> &vv = vars.variants (*c);
-      for (std::map<db::ICplxTrans, size_t>::const_iterator v = vv.begin (); v != vv.end (); ++v) {
+      for (auto v = vv.begin (); v != vv.end (); ++v) {
         double mag = v->first.mag ();
         a += v->second * ac * mag * mag;
       }
@@ -1127,8 +1152,8 @@ DeepRegion::perimeter (const db::Box &box) const
 
     const db::DeepLayer &polygons = merged_deep_layer ();
 
-    db::cell_variants_collector<db::MagnificationReducer> vars;
-    vars.collect (polygons.layout (), polygons.initial_cell ());
+    db::cell_variants_statistics<db::MagnificationReducer> vars;
+    vars.collect (&polygons.layout (), polygons.initial_cell ().cell_index ());
 
     DeepRegion::perimeter_type p = 0;
 
@@ -1139,7 +1164,7 @@ DeepRegion::perimeter (const db::Box &box) const
         pc += s->perimeter ();
       }
       const std::map<db::ICplxTrans, size_t> &vv = vars.variants (*c);
-      for (std::map<db::ICplxTrans, size_t>::const_iterator v = vv.begin (); v != vv.end (); ++v) {
+      for (auto v = vv.begin (); v != vv.end (); ++v) {
         double mag = v->first.mag ();
         p += v->second * pc * mag;
       }
@@ -1189,7 +1214,7 @@ DeepRegion::grid_check (db::Coord gx, db::Coord gy) const
   db::Layout &layout = const_cast<db::Layout &> (polygons.layout ());
 
   db::cell_variants_collector<db::GridReducer> vars (gx);
-  vars.collect (layout, polygons.initial_cell ());
+  vars.collect (&layout, polygons.initial_cell ().cell_index ());
 
   std::map<db::cell_index_type, std::map<db::ICplxTrans, db::Shapes> > to_commit;
   std::unique_ptr<db::DeepEdgePairs> res (new db::DeepEdgePairs (polygons.derived ()));
@@ -1198,14 +1223,14 @@ DeepRegion::grid_check (db::Coord gx, db::Coord gy) const
 
     const db::Shapes &shapes = c->shapes (polygons.layer ());
 
-    const std::map<db::ICplxTrans, size_t> &vv = vars.variants (c->cell_index ());
-    for (std::map<db::ICplxTrans, size_t>::const_iterator v = vv.begin (); v != vv.end (); ++v) {
+    const std::set<db::ICplxTrans> &vv = vars.variants (c->cell_index ());
+    for (auto v = vv.begin (); v != vv.end (); ++v) {
 
       db::Shapes *markers;
       if (vv.size () == 1) {
         markers = & c->shapes (res->deep_layer ().layer ());
       } else {
-        markers = & to_commit [c->cell_index ()] [v->first];
+        markers = & to_commit [c->cell_index ()] [*v];
       }
 
       for (db::Shapes::shape_iterator si = shapes.begin (db::ShapeIterator::All); ! si.at_end (); ++si) {
@@ -1213,7 +1238,7 @@ DeepRegion::grid_check (db::Coord gx, db::Coord gy) const
         db::Polygon poly;
         si->polygon (poly);
 
-        AsIfFlatRegion::produce_markers_for_grid_check (poly, v->first, gx, gy, *markers);
+        AsIfFlatRegion::produce_markers_for_grid_check (poly, *v, gx, gy, *markers);
 
       }
 
@@ -1222,7 +1247,7 @@ DeepRegion::grid_check (db::Coord gx, db::Coord gy) const
   }
 
   //  propagate the markers with a similar algorithm used for producing the variants
-  res->deep_layer ().commit_shapes (vars, to_commit);
+  vars.commit_shapes (res->deep_layer ().layer (), to_commit);
 
   return res.release ();
 }
@@ -1276,23 +1301,19 @@ DeepRegion::snapped (db::Coord gx, db::Coord gy)
   }
 
   const db::DeepLayer &polygons = merged_deep_layer ();
+  db::Layout &layout = const_cast<db::Layout &> (polygons.layout ());
 
   db::cell_variants_collector<db::GridReducer> vars (gx);
 
-  vars.collect (polygons.layout (), polygons.initial_cell ());
+  vars.collect (&layout, polygons.initial_cell ().cell_index ());
+  vars.separate_variants ();
 
-  //  NOTE: m_merged_polygons is mutable, so why is the const_cast needed?
-  const_cast<db::DeepLayer &> (polygons).separate_variants (vars);
-
-  db::Layout &layout = const_cast<db::Layout &> (polygons.layout ());
   std::vector<db::Point> heap;
 
   std::unique_ptr<db::DeepRegion> res (new db::DeepRegion (polygons.derived ()));
   for (db::Layout::iterator c = layout.begin (); c != layout.end (); ++c) {
 
-    const std::map<db::ICplxTrans, size_t> &v = vars.variants (c->cell_index ());
-    tl_assert (v.size () == size_t (1));
-    const db::ICplxTrans &tr = v.begin ()->first;
+    const db::ICplxTrans &tr = vars.single_variant_transformation (c->cell_index ());
     db::ICplxTrans trinv = tr.inverted ();
 
     const db::Shapes &s = c->shapes (polygons.layer ());
@@ -1346,27 +1367,22 @@ DeepRegion::edges (const EdgeFilterBase *filter) const
     db::PropertyMapper pm (res->properties_repository (), &polygons.layout ().properties_repository ());
 
     std::unique_ptr<VariantsCollectorBase> vars;
+    db::Layout &layout = const_cast<db::Layout &> (polygons.layout ());
 
     if (filter && filter->vars ()) {
 
       vars.reset (new db::VariantsCollectorBase (filter->vars ()));
 
-      vars->collect (polygons.layout (), polygons.initial_cell ());
-
-      //  NOTE: m_merged_polygons is mutable, so why is the const_cast needed?
-      const_cast<db::DeepLayer &> (polygons).separate_variants (*vars);
+      vars->collect (&layout, polygons.initial_cell ().cell_index ());
+      vars->separate_variants ();
 
     }
 
-    db::Layout &layout = const_cast<db::Layout &> (polygons.layout ());
-
     for (db::Layout::iterator c = layout.begin (); c != layout.end (); ++c) {
 
       db::ICplxTrans tr;
       if (vars.get ()) {
-        const std::map<db::ICplxTrans, size_t> &v = vars->variants (c->cell_index ());
-        tl_assert (v.size () == size_t (1));
-        tr = v.begin ()->first;
+        tr = vars->single_variant_transformation (c->cell_index ());
       }
 
       const db::Shapes &s = c->shapes (polygons.layer ());
@@ -1461,21 +1477,21 @@ DeepRegion *
 DeepRegion::apply_filter (const PolygonFilterBase &filter) const
 {
   const db::DeepLayer &polygons = filter.requires_raw_input () ? deep_layer () : merged_deep_layer ();
+  db::Layout &layout = const_cast<db::Layout &> (polygons.layout ());
 
   std::unique_ptr<VariantsCollectorBase> vars;
   if (filter.vars ()) {
 
     vars.reset (new db::VariantsCollectorBase (filter.vars ()));
 
-    vars->collect (polygons.layout (), polygons.initial_cell ());
+    vars->collect (&layout, polygons.initial_cell ().cell_index ());
 
     if (filter.wants_variants ()) {
-      const_cast<db::DeepLayer &> (polygons).separate_variants (*vars);
+      vars->separate_variants ();
     }
 
   }
 
-  db::Layout &layout = const_cast<db::Layout &> (polygons.layout ());
   std::map<db::cell_index_type, std::map<db::ICplxTrans, db::Shapes> > to_commit;
 
   std::unique_ptr<db::DeepRegion> res (new db::DeepRegion (polygons.derived ()));
@@ -1485,22 +1501,20 @@ DeepRegion::apply_filter (const PolygonFilterBase &filter) const
 
     if (vars.get ()) {
 
-      const std::map<db::ICplxTrans, size_t> &vv = vars->variants (c->cell_index ());
-      for (std::map<db::ICplxTrans, size_t>::const_iterator v = vv.begin (); v != vv.end (); ++v) {
+      const std::set<db::ICplxTrans> &vv = vars->variants (c->cell_index ());
+      for (auto v = vv.begin (); v != vv.end (); ++v) {
 
         db::Shapes *st;
         if (vv.size () == 1) {
           st = & c->shapes (res->deep_layer ().layer ());
         } else {
-          st = & to_commit [c->cell_index ()] [v->first];
+          st = & to_commit [c->cell_index ()] [*v];
         }
 
-        const db::ICplxTrans &tr = v->first;
-
         for (db::Shapes::shape_iterator si = s.begin (db::ShapeIterator::All); ! si.at_end (); ++si) {
           db::Polygon poly;
           si->polygon (poly);
-          if (filter.selected (poly.transformed (tr))) {
+          if (filter.selected (poly.transformed (*v))) {
             st->insert (*si);
           }
         }
@@ -1524,7 +1538,7 @@ DeepRegion::apply_filter (const PolygonFilterBase &filter) const
   }
 
   if (! to_commit.empty () && vars.get ()) {
-    res->deep_layer ().commit_shapes (*vars, to_commit);
+    vars->commit_shapes (res->deep_layer ().layer (), to_commit);
   }
 
   if (! filter.requires_raw_input ()) {
@@ -1634,17 +1648,14 @@ DeepRegion::sized (coord_type d, unsigned int mode) const
   db::Layout &layout = const_cast<db::Layout &> (polygons.layout ());
 
   db::cell_variants_collector<db::MagnificationReducer> vars;
-  vars.collect (polygons.layout (), polygons.initial_cell ());
-
-  //  NOTE: m_merged_polygons is mutable, so why is the const_cast needed?
-  const_cast<db::DeepLayer &> (polygons).separate_variants (vars);
+  vars.collect (&layout, polygons.initial_cell ().cell_index ());
+  vars.separate_variants ();
 
   std::unique_ptr<db::DeepRegion> res (new db::DeepRegion (polygons.derived ()));
   for (db::Layout::iterator c = layout.begin (); c != layout.end (); ++c) {
 
-    const std::map<db::ICplxTrans, size_t> &v = vars.variants (c->cell_index ());
-    tl_assert (v.size () == size_t (1));
-    double mag = v.begin ()->first.mag ();
+    const db::ICplxTrans &tr = vars.single_variant_transformation (c->cell_index ());
+    double mag = tr.mag ();
     db::Coord d_with_mag = db::coord_traits<db::Coord>::rounded (d / mag);
 
     const db::Shapes &s = c->shapes (polygons.layer ());
@@ -1690,18 +1701,15 @@ DeepRegion::sized (coord_type dx, coord_type dy, unsigned int mode) const
   db::Layout &layout = const_cast<db::Layout &> (polygons.layout ());
 
   db::cell_variants_collector<db::XYAnisotropyAndMagnificationReducer> vars;
-  vars.collect (polygons.layout (), polygons.initial_cell ());
-
-  //  NOTE: m_merged_polygons is mutable, so why is the const_cast needed?
-  const_cast<db::DeepLayer &> (polygons).separate_variants (vars);
+  vars.collect (&layout, polygons.initial_cell ().cell_index ());
+  vars.separate_variants ();
 
   std::unique_ptr<db::DeepRegion> res (new db::DeepRegion (polygons.derived ()));
   for (db::Layout::iterator c = layout.begin (); c != layout.end (); ++c) {
 
-    const std::map<db::ICplxTrans, size_t> &v = vars.variants (c->cell_index ());
-    tl_assert (v.size () == size_t (1));
-    double mag = v.begin ()->first.mag ();
-    double angle = v.begin ()->first.angle ();
+    const db::ICplxTrans &tr = vars.single_variant_transformation (c->cell_index ());
+    double mag = tr.mag ();
+    double angle = tr.angle ();
 
     db::Coord dx_with_mag = db::coord_traits<db::Coord>::rounded (dx / mag);
     db::Coord dy_with_mag = db::coord_traits<db::Coord>::rounded (dy / mag);
@@ -1778,7 +1786,7 @@ Output *region_cop_impl (DeepRegion *region, db::CompoundRegionOperationNode &no
   }
 
   compound_local_operation<db::PolygonRef, db::PolygonRef, TR> op (&node);
-  proc.run (&op, polygons.layer (), other_layers, res->deep_layer ().layer ());
+  proc.run (&op, polygons.layer (), other_layers, res->deep_layer ().layer (), true /*make_variants*/);
 
   return res.release ();
 }
@@ -1831,7 +1839,7 @@ Output *region_cop_with_properties_impl (DeepRegion *region, db::CompoundRegionO
   }
 
   compound_local_operation_with_properties<db::PolygonRef, db::PolygonRef, TR> op (&node, prop_constraint, res->properties_repository (), subject_pr, intruder_prs);
-  proc.run (&op, polygons.layer (), other_layers, res->deep_layer ().layer ());
+  proc.run (&op, polygons.layer (), other_layers, res->deep_layer ().layer (), true /*make_variants*/);
 
   return res.release ();
 }
@@ -1895,6 +1903,7 @@ DeepRegion::run_check (db::edge_relation_type rel, bool different_polygons, cons
 
   //  force different polygons in the different properties case to skip intra-polygon checks
   if (pc_always_different (options.prop_constraint)) {
+    //  TODO: this forces merged primaries, so maybe that is not a good optimization?
     different_polygons = true;
   }
 
@@ -1986,19 +1995,26 @@ DeepRegion::run_single_polygon_check (db::edge_relation_type rel, db::Coord d, c
   }
 
   const db::DeepLayer &polygons = merged_deep_layer ();
-
-  EdgeRelationFilter check (rel, d, options.metrics);
-  check.set_include_zero (false);
-  check.set_whole_edges (options.whole_edges);
-  check.set_ignore_angle (options.ignore_angle);
-  check.set_min_projection (options.min_projection);
-  check.set_max_projection (options.max_projection);
-
   db::Layout &layout = const_cast<db::Layout &> (polygons.layout ());
 
+  db::cell_variants_collector<db::MagnificationReducer> vars;
+  vars.collect (&layout, polygons.initial_cell ().cell_index ());
+  vars.separate_variants ();
+
   std::unique_ptr<db::DeepEdgePairs> res (new db::DeepEdgePairs (polygons.derived ()));
   for (db::Layout::iterator c = layout.begin (); c != layout.end (); ++c) {
 
+    const db::ICplxTrans &tr = vars.single_variant_transformation (c->cell_index ());
+    double mag = tr.mag ();
+    db::Coord d_with_mag = db::coord_traits<db::Coord>::rounded (d / mag);
+
+    EdgeRelationFilter check (rel, d_with_mag, options.metrics);
+    check.set_include_zero (false);
+    check.set_whole_edges (options.whole_edges);
+    check.set_ignore_angle (options.ignore_angle);
+    check.set_min_projection (options.min_projection);
+    check.set_max_projection (options.max_projection);
+
     const db::Shapes &shapes = c->shapes (polygons.layer ());
     db::Shapes &result = c->shapes (res->deep_layer ().layer ());
 
