@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -228,6 +228,14 @@ struct DB_PUBLIC EdgeRelationFilter
     return m_max_projection;
   }
 
+  /**
+   *  @brief sets the check distance
+   */
+  void set_distance (distance_type d)
+  {
+    m_d = d;
+  }
+
   /**
    *  @brief Gets the check distance
    */
@@ -236,6 +244,14 @@ struct DB_PUBLIC EdgeRelationFilter
     return m_d;
   }
 
+  /**
+   *  @brief Sets the relation
+   */
+  void set_relation (edge_relation_type r)
+  {
+    m_r = r;
+  }
+
   /**
    *  @brief Gets the relation
    */
