@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -218,7 +218,7 @@ merge_layouts (db::Layout &target,
   std::map<db::cell_index_type, db::cell_index_type> new_cell_mapping;
   for (std::set<db::cell_index_type>::const_iterator c = all_cells_to_copy.begin (); c != all_cells_to_copy.end (); ++c) {
     if (cell_mapping.find (*c) == cell_mapping.end ()) {
-      new_cell_mapping.insert (std::make_pair (*c, target.add_cell (source.cell_name (*c))));
+      new_cell_mapping.insert (std::make_pair (*c, target.add_cell (source, *c)));
     }
   }
 
@@ -234,7 +234,7 @@ merge_layouts (db::Layout &target,
   //  provide the property mapper
   db::PropertyMapper pm (&target, &source);
 
-  tl::RelativeProgress progress (tl::to_string (tr ("Merge cells")), all_cells_to_copy.size (), 1);
+  tl::RelativeProgress progress (tl::to_string (tr ("Merge layouts")), all_cells_to_copy.size (), 1);
 
   //  actually to the mapping
   for (std::set<db::cell_index_type>::const_iterator c = all_cells_to_copy.begin (); c != all_cells_to_copy.end (); ++c) {
@@ -346,7 +346,7 @@ copy_or_move_shapes (db::Layout &target,
   //  provide the property mapper
   db::PropertyMapper pm (&target, &source);
 
-  tl::RelativeProgress progress (tl::to_string (tr ("Merge cells")), all_cells_to_copy.size () * layer_mapping.size (), 1);
+  tl::RelativeProgress progress (tl::to_string (tr ("Copy shapes")), all_cells_to_copy.size () * layer_mapping.size (), 1);
 
   //  and copy
   for (std::set<db::cell_index_type>::const_iterator c = all_cells_to_copy.begin (); c != all_cells_to_copy.end (); ++c) {
@@ -521,8 +521,8 @@ scale_and_snap (db::Layout &layout, db::Cell &cell, db::Coord g, db::Coord m, db
 
   {
     tl::SelfTimer timer1 (tl::verbosity () >= 41, tl::to_string (tr ("scale_and_snap: variant formation")));
-    vars.collect (layout, cell);
-    vars.separate_variants (layout, cell);
+    vars.collect (&layout, cell.cell_index ());
+    vars.separate_variants ();
   }
 
   std::set<db::cell_index_type> called_cells;
@@ -543,9 +543,7 @@ scale_and_snap (db::Layout &layout, db::Cell &cell, db::Coord g, db::Coord m, db
       continue;
     }
 
-    const std::map<db::ICplxTrans, size_t> &v = vars.variants (c->cell_index ());
-    tl_assert (v.size () == size_t (1));
-    db::ICplxTrans tr = v.begin ()->first;
+    db::ICplxTrans tr = vars.single_variant_transformation (c->cell_index ());
 
     //  NOTE: tr_disp is already multiplied with mag, so it can be an integer
     db::Vector tr_disp = tr.disp ();
