@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -1052,7 +1052,7 @@ NetlistCompareCore::derive_node_identities_from_ambiguity_group (const NodeRange
       if (ambiguous) {
         if (logger) {
           if (with_log) {
-            logger->log_entry (db::NetlistCompareLogger::Warning,
+            logger->log_entry (db::Warning,
                                tl::sprintf (tl::to_string (tr ("Matching nets %s from an ambiguous group of nets")), nets2string (p->first->net (), p->second->net ())));
           }
           logger->match_ambiguous_nets (p->first->net (), p->second->net ());
@@ -1116,7 +1116,7 @@ NetlistCompareCore::derive_node_identities_from_ambiguity_group (const NodeRange
         }
 
         if (logger && with_log && was_ambiguous) {
-          logger->log_entry (db::NetlistCompareLogger::Info,
+          logger->log_entry (db::Info,
                              tl::sprintf (tl::to_string (tr ("Matching nets %s following an ambiguous match")), nets2string (n->net (), n_other->net ())));
         }
 
@@ -1397,7 +1397,7 @@ analyze_nodes_for_close_matches (const std::multimap<size_t, const NetGraphNode
         double fuzz_factor = double (fuzz) / ne;
         if (fuzz_factor < max_fuzz_factor) {
           std::string msg = tl::to_string (tr ("Net %s from netlist approximately matches net %s from reference netlist (fuzziness %d nodes)"));
-          logger->log_entry (db::NetlistCompareLogger::Info, tl::sprintf (msg,
+          logger->log_entry (db::Info, tl::sprintf (msg,
                                   i->second->net ()->expanded_name (),
                                   j->second->net ()->expanded_name (),
                                   int (fuzz)));
@@ -1417,7 +1417,7 @@ analyze_nodes_for_close_matches (const std::multimap<size_t, const NetGraphNode
         size_t fuzz = distance3 (*i->second, *j->second, *k->second, g2);
         double fuzz_factor = double (fuzz) / i->first;
         if (fuzz_factor < max_fuzz_factor) {
-          logger->log_entry (db::NetlistCompareLogger::Info, tl::sprintf (msg,
+          logger->log_entry (db::Info, tl::sprintf (msg,
                                   (layout2ref ? i : j)->second->net ()->expanded_name (),
                                   (layout2ref ? j : k)->second->net ()->expanded_name (),
                                   (layout2ref ? k : i)->second->net ()->expanded_name (),
@@ -1493,7 +1493,7 @@ NetlistCompareCore::analyze_failed_matches () const
   }
 
   for (auto i = singular1.begin (); i != singular1.end (); ++i) {
-    logger->log_entry (db::NetlistCompareLogger::Error, tl::sprintf (tl::to_string (tr ("Net %s is not matching any net from reference netlist")), (*i)->net ()->expanded_name ()));
+    logger->log_entry (db::Error, tl::sprintf (tl::to_string (tr ("Net %s is not matching any net from reference netlist")), (*i)->net ()->expanded_name ()));
   }
 
   //  attempt some analysis for close matches (including shorts / opens)
@@ -1552,7 +1552,7 @@ NetlistCompareCore::derive_node_identities_from_node_set (std::vector<NodeEdgePa
 
   if (max_depth != std::numeric_limits<size_t>::max() && depth > max_depth) {
     if (with_log) {
-      logger->log_entry (db::NetlistCompareLogger::Warning, tl::sprintf (tl::to_string (tr ("Maximum depth exhausted (max depth is %d)")), int (max_depth)));
+      logger->log_entry (db::Warning, tl::sprintf (tl::to_string (tr ("Maximum depth exhausted (max depth is %d)")), int (max_depth)));
     }
     if (db::NetlistCompareGlobalOptions::options ()->debug_netcompare) {
       tl::info << indent_s << "max. depth exhausted (" << depth << ">" << max_depth << ")";
@@ -1670,7 +1670,7 @@ NetlistCompareCore::derive_node_identities_from_node_set (std::vector<NodeEdgePa
     } else if (max_n_branch != std::numeric_limits<size_t>::max () && double (std::max (nr->num1, nr->num2)) * double (n_branch) > double (max_n_branch)) {
 
       if (with_log) {
-        logger->log_entry (db::NetlistCompareLogger::Warning, tl::sprintf (tl::to_string (tr ("Maximum complexity exhausted (max complexity is %s, needs at least %s)")), tl::to_string (max_n_branch), tl::to_string (std::max (nr->num1, nr->num2) * n_branch)));
+        logger->log_entry (db::Warning, tl::sprintf (tl::to_string (tr ("Maximum complexity exhausted (max complexity is %s, needs at least %s)")), tl::to_string (max_n_branch), tl::to_string (std::max (nr->num1, nr->num2) * n_branch)));
       }
       if (db::NetlistCompareGlobalOptions::options ()->debug_netcompare) {
         tl::info << indent_s << "max. complexity exhausted (" << std::max (nr->num1, nr->num2) << "*" << n_branch << ">" << max_n_branch << ") - mismatch.";
