@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -105,15 +105,6 @@ public:
     std::string msg;
   };
 
-  struct LogEntryData
-  {
-    LogEntryData (Severity s, const std::string &m) : severity (s), msg (m) { }
-    LogEntryData () : severity (NoSeverity) { }
-
-    Severity severity;
-    std::string msg;
-  };
-
   struct PerCircuitData
   {
     PerCircuitData () : status (None) { }
