@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -32,45 +32,6 @@
 namespace db
 {
 
-// ----------------------------------------------------------------------------------------
-//  NetlistDeviceExtractorError implementation
-
-NetlistDeviceExtractorError::NetlistDeviceExtractorError ()
-{
-  //  .. nothing yet ..
-}
-
-NetlistDeviceExtractorError::NetlistDeviceExtractorError (const std::string &cell_name, const std::string &msg)
-  : m_cell_name (cell_name), m_message (msg)
-{
-  //  .. nothing yet ..
-}
-
-std::string NetlistDeviceExtractorError::to_string () const
-{
-  std::string res;
-
-  if (! m_category_name.empty ()) {
-    if (m_category_description.empty ()) {
-      res += "[" + m_category_name + "] ";
-    } else {
-      res += "[" + m_category_description + "] ";
-    }
-  }
-
-  res += m_message;
-
-  if (! m_cell_name.empty ()) {
-    res += tl::to_string (tr (", in cell: ")) + m_cell_name;
-  }
-
-  if (! m_geometry.box ().empty ()) {
-    res += tl::to_string (tr (", shape: ")) + m_geometry.to_string ();
-  }
-
-  return res;
-}
-
 // ----------------------------------------------------------------------------------------
 //  NetlistDeviceExtractor implementation
 
@@ -586,43 +547,89 @@ std::string NetlistDeviceExtractor::cell_name () const
 
 void NetlistDeviceExtractor::error (const std::string &msg)
 {
-  m_errors.push_back (db::NetlistDeviceExtractorError (cell_name (), msg));
+  m_log_entries.push_back (db::LogEntryData (db::Error, cell_name (), msg));
+  m_log_entries.back ().set_category_name ("device-extract");
 
   if (tl::verbosity () >= 20) {
-    tl::error << m_errors.back ().to_string ();
+    tl::error << m_log_entries.back ().to_string ();
   }
 }
 
 void NetlistDeviceExtractor::error (const std::string &msg, const db::DPolygon &poly)
 {
-  m_errors.push_back (db::NetlistDeviceExtractorError (cell_name (), msg));
-  m_errors.back ().set_geometry (poly);
+  m_log_entries.push_back (db::LogEntryData (db::Error, cell_name (), msg));
+  m_log_entries.back ().set_geometry (poly);
+  m_log_entries.back ().set_category_name ("device-extract");
 
   if (tl::verbosity () >= 20) {
-    tl::error << m_errors.back ().to_string ();
+    tl::error << m_log_entries.back ().to_string ();
   }
 }
 
 void NetlistDeviceExtractor::error (const std::string &category_name, const std::string &category_description, const std::string &msg)
 {
-  m_errors.push_back (db::NetlistDeviceExtractorError (cell_name (), msg));
-  m_errors.back ().set_category_name (category_name);
-  m_errors.back ().set_category_description (category_description);
+  m_log_entries.push_back (db::LogEntryData (db::Error, cell_name (), msg));
+  m_log_entries.back ().set_category_name (category_name);
+  m_log_entries.back ().set_category_description (category_description);
 
   if (tl::verbosity () >= 20) {
-    tl::error << m_errors.back ().to_string ();
+    tl::error << m_log_entries.back ().to_string ();
   }
 }
 
 void NetlistDeviceExtractor::error (const std::string &category_name, const std::string &category_description, const std::string &msg, const db::DPolygon &poly)
 {
-  m_errors.push_back (db::NetlistDeviceExtractorError (cell_name (), msg));
-  m_errors.back ().set_category_name (category_name);
-  m_errors.back ().set_category_description (category_description);
-  m_errors.back ().set_geometry (poly);
+  m_log_entries.push_back (db::LogEntryData (db::Error, cell_name (), msg));
+  m_log_entries.back ().set_category_name (category_name);
+  m_log_entries.back ().set_category_description (category_description);
+  m_log_entries.back ().set_geometry (poly);
 
   if (tl::verbosity () >= 20) {
-    tl::error << m_errors.back ().to_string ();
+    tl::error << m_log_entries.back ().to_string ();
+  }
+}
+
+void NetlistDeviceExtractor::warn (const std::string &msg)
+{
+  m_log_entries.push_back (db::LogEntryData (db::Warning, cell_name (), msg));
+  m_log_entries.back ().set_category_name ("device-extract");
+
+  if (tl::verbosity () >= 20) {
+    tl::warn << m_log_entries.back ().to_string ();
+  }
+}
+
+void NetlistDeviceExtractor::warn (const std::string &msg, const db::DPolygon &poly)
+{
+  m_log_entries.push_back (db::LogEntryData (db::Warning, cell_name (), msg));
+  m_log_entries.back ().set_geometry (poly);
+  m_log_entries.back ().set_category_name ("device-extract");
+
+  if (tl::verbosity () >= 20) {
+    tl::warn << m_log_entries.back ().to_string ();
+  }
+}
+
+void NetlistDeviceExtractor::warn (const std::string &category_name, const std::string &category_description, const std::string &msg)
+{
+  m_log_entries.push_back (db::LogEntryData (db::Warning, cell_name (), msg));
+  m_log_entries.back ().set_category_name (category_name);
+  m_log_entries.back ().set_category_description (category_description);
+
+  if (tl::verbosity () >= 20) {
+    tl::warn << m_log_entries.back ().to_string ();
+  }
+}
+
+void NetlistDeviceExtractor::warn (const std::string &category_name, const std::string &category_description, const std::string &msg, const db::DPolygon &poly)
+{
+  m_log_entries.push_back (db::LogEntryData (db::Warning, cell_name (), msg));
+  m_log_entries.back ().set_category_name (category_name);
+  m_log_entries.back ().set_category_description (category_description);
+  m_log_entries.back ().set_geometry (poly);
+
+  if (tl::verbosity () >= 20) {
+    tl::warn << m_log_entries.back ().to_string ();
   }
 }
 
