@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -277,7 +277,7 @@ void path<C>::create_shifted_points (C start, C end, C width, bool forward, Iter
     Iter ppp = pp;
     ++ppp;
 
-    //  Compute the unit vector of the line and it's normal (times width)
+    //  Compute the unit vector of the line and its normal (times width)
 
     db::DVector ed (*pp - *p);
     ed *= 1.0 / ed.double_length ();
@@ -383,7 +383,7 @@ void path<C>::create_shifted_points (C start, C end, C width, bool forward, Iter
 
           //  Segments are too short - the won't intersect: In this case we create a loop of three 
           //  points which define the area in self-overlapping way but confined to the path within
-          //  the limits of it's width. 
+          //  the limits of its width.
           //  HINT: the execution of this code is a pretty strong evidence for the existence to loops 
           //  in the contour delivered. A proof however is missing ..
           *pts++ = *pp + vector<C> (nd);
@@ -579,8 +579,8 @@ Path round_path_corners (const Path &path, int rad, int n)
   return Path (round_path_corners (db::DPath (path), double (rad), n, 0.5));
 }
 
-template class path<Coord>;
-template class path<DCoord>;
+template class DB_PUBLIC path<Coord>;
+template class DB_PUBLIC path<DCoord>;
 
 //  explicit instantiations
 template DB_PUBLIC void path<Coord>::create_shifted_points (Coord, Coord, Coord, bool, path<Coord>::pointlist_type::iterator, path<Coord>::pointlist_type::iterator, int, box_inserter<path<Coord>::box_type>) const;
