@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -456,6 +456,8 @@ static bool _cut_polygon_internal (const PolygonType &input, const Edge &line, c
         }
       }
 
+      hull->sort_holes ();
+
       right_of_line->put (*hull);
 
     }
@@ -859,6 +861,8 @@ smooth (const db::Polygon &polygon, db::Coord d, bool keep_hv)
       }
     }
 
+    new_poly.sort_holes ();
+
   }
 
   return new_poly;
@@ -1351,6 +1355,8 @@ do_extract_rad (const db::polygon<C> &polygon, double &rinner, double &router, u
 
     }
 
+    new_polygon->sort_holes ();
+
   } else {
 
     if (! do_extract_rad_from_contour (polygon.begin_hull (), polygon.end_hull (), rinner, router, n, (std::vector<db::point<C> > *) 0, false)) {
@@ -1542,6 +1548,8 @@ do_compute_rounded (const db::polygon<C> &polygon, double rinner, double router,
     new_poly.insert_hole (new_pts.begin (), new_pts.end (), false /*don't compress*/);
   }
 
+  new_poly.sort_holes ();
+
   return new_poly;
 }
 
@@ -1685,6 +1693,20 @@ AreaMap::bbox () const
 // -------------------------------------------------------------------------
 //  Implementation of rasterize
 
+static bool edge_is_partially_left_of (const db::Edge &e, const db::Edge &e_original, db::Coord x)
+{
+  Coord xmin = db::edge_xmin (e);
+  if (xmin < x) {
+    return true;
+  } else if (xmin == x && e_original.dx () != 0) {
+    //  the skew edge is cut partially rendering a straight vertical line (due to rounding)
+    //  which we will count as "left of"
+    return true;
+  } else {
+    return false;
+  }
+}
+
 bool
 rasterize (const db::Polygon &polygon, db::AreaMap &am)
 {
@@ -1801,7 +1823,7 @@ rasterize (const db::Polygon &polygon, db::AreaMap &am)
         for (std::vector <db::Edge>::iterator e = cc; e != ff; ++e) {
 
           std::pair<bool, db::Edge> ec = e->clipped (left);
-          if (ec.first && db::edge_xmin (ec.second) < x) {
+          if (ec.first && edge_is_partially_left_of (ec.second, *e, x)) {
             a += area_type (ec.second.dy ()) * area_type (px);
           }
 
@@ -1818,7 +1840,7 @@ rasterize (const db::Polygon &polygon, db::AreaMap &am)
         for (std::vector <db::Edge>::iterator e = cc; e != ff; ++e) {
 
           std::pair<bool, db::Edge> ec = e->clipped (cell);
-          if (ec.first && db::edge_xmin (ec.second) < xx) {
+          if (ec.first && edge_is_partially_left_of (ec.second, *e, xx)) {
             aa += (area_type (ec.second.dy ()) * area_type (2 * xx - (ec.second.p2 ().x () + ec.second.p1 ().x ()))) / 2;
             a += area_type (ec.second.dy ()) * area_type (px);
           }
@@ -1832,7 +1854,7 @@ rasterize (const db::Polygon &polygon, db::AreaMap &am)
         for (std::vector <db::Edge>::iterator e = cc; e != ff; ++e) {
 
           std::pair<bool, db::Edge> ec = e->clipped (cell);
-          if (ec.first && db::edge_xmin (ec.second) < xx) {
+          if (ec.first && edge_is_partially_left_of (ec.second, *e, xx)) {
             aa += (area_type (ec.second.dy ()) * area_type (2 * xx - (ec.second.p2 ().x () + ec.second.p1 ().x ()))) / 2;
           }
 
@@ -1843,7 +1865,7 @@ rasterize (const db::Polygon &polygon, db::AreaMap &am)
         for (std::vector <db::Edge>::iterator e = cc; e != fff; ++e) {
 
           std::pair<bool, db::Edge> wide_ec = e->clipped (wide_cell);
-          if (wide_ec.first && db::edge_xmin (wide_ec.second) < x + dx) {
+          if (wide_ec.first && edge_is_partially_left_of (wide_ec.second, *e, x + dx)) {
             a += area_type (wide_ec.second.dy ()) * area_type (px);
           }
 
@@ -2143,6 +2165,10 @@ ms_extraction (db::EdgeProcessor &ep, bool resolve_holes)
 static db::Polygon 
 do_minkowski_sum (const db::Polygon &a, const db::Edge &b, bool resolve_holes)
 {
+  if (a.begin_hull () == a.end_hull ()) {
+    return db::Polygon ();
+  }
+
   db::EdgeProcessor ep;
   db::ms_production (a, b.p1 (), b.p2 (), ep);
   return db::ms_extraction (ep, resolve_holes);
@@ -2161,7 +2187,9 @@ minkowski_sum (const db::Polygon &a, const db::Edge &b, bool rh)
 static db::Polygon 
 do_minkowski_sum (const db::Polygon &a, const db::Polygon &b, bool resolve_holes)
 {
-  tl_assert (a.begin_hull () != a.end_hull ());
+  if (a.begin_hull () == a.end_hull () || b.begin_hull () == b.end_hull ()) {
+    return db::Polygon ();
+  }
 
   db::Vector p0 = *a.begin_hull () - db::Point ();
 
@@ -2280,9 +2308,11 @@ static void decompose_convex_helper (int depth, PreferredOrientation po, const d
 
   db::Box bbox = sp.box ();
   db::coord_traits<db::Coord>::area_type atot = 0;
+  db::coord_traits<db::Coord>::distance_type min_edge = std::numeric_limits<db::coord_traits<db::Coord>::distance_type>::max ();
   for (size_t i = 0; i < n; ++i) {
     db::Edge ep (sp.hull ()[(i + n - 1) % n], sp.hull ()[i]);
     atot += db::vprod (ep.p2 () - db::Point (), ep.p1 () - db::Point ());
+    min_edge = std::min (min_edge, ep.length ());
   }
 
   std::set<db::Point> skipped;
@@ -2429,9 +2459,14 @@ static void decompose_convex_helper (int depth, PreferredOrientation po, const d
               int cr = 0;
               if (x.second == efc.p1 ()) {
                 if (db::vprod (efc, efp) < 0) {
-                  cr = 2;   //  cut terminates at another concave corner
+                  cr = 3;   //  cut terminates at another concave corner
                 } else {
-                  cr = 1;   //  cut terminates at a convex corner
+                  cr = 2;   //  cut terminates at a convex corner
+                }
+              } else {
+                db::coord_traits<db::Coord>::distance_type el = std::min (x.second.distance (efc.p1 ()), x.second.distance (efc.p2 ()));
+                if (el >= min_edge) {
+                  cr = 1;   //  does not induce shorter edge than we have so far
                 }
               }
 
@@ -2862,6 +2897,8 @@ snapped_polygon (const db::Polygon &poly, db::Coord gx, db::Coord gy, std::vecto
 
   }
 
+  pnew.sort_holes ();
+
   return pnew;
 }
 
@@ -2901,6 +2938,8 @@ scaled_and_snapped_polygon (const db::Polygon &poly, db::Coord gx, db::Coord mx,
 
   }
 
+  pnew.sort_holes ();
+
   return pnew;
 }
 
