@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -913,6 +913,11 @@ public:
    */
   virtual ~RecursiveShapeReceiver () { }
 
+  /**
+   *  @brief Returns true, if the receivers wants the full hierarchy and not just non-empty cells
+   */
+  virtual bool wants_all_cells () const { return false; }
+
   /**
    *  @brief Called once when the iterator begins pushing
    */
