@@ -13,6 +13,125 @@ module DRC
     end
   end
 
+  class OutputChannel
+    def initialize(engine)
+      @engine = engine
+    end
+    def is_rdb?
+      false
+    end
+    def cellname=(cellname)
+      # reimplement
+    end
+    def cell
+      # reimplement
+    end
+    def write
+      # reimplement
+    end
+    def finish(final)
+      # reimplement
+    end
+    def write
+      # reimplement
+    end
+    def layout
+      nil
+    end
+    def rdb
+      nil
+    end
+  end
+
+  class LayoutOutputChannel < OutputChannel
+
+    def initialize(engine, layout, cell, file_name)
+      super(engine)
+      @layout = layout
+      @cell = cell
+      @file_name = file_name
+    end
+
+    def cellname=(cellname)
+      @cell = @layout.cell(cellname) || @layout.create_cell(cellname)
+    end
+
+    def is_rdb?
+      false
+    end
+
+    def finish(final, view)
+      write
+    end
+
+    def write
+      if @file_name
+        opt = RBA::SaveLayoutOptions::new
+        gzip = opt.set_format_from_filename(@file_name)
+        @engine.info("Writing layout file: #{@file_name} ..")
+        @layout.write(@file_name, gzip, opt)
+      end
+    end
+
+    def layout
+      @layout
+    end
+
+    def cell
+      @cell
+    end
+
+  end
+
+  class RDBOutputChannel < OutputChannel
+
+    def initialize(engine, rdb, rdb_index, cellname, file_name)
+      super(engine)
+      @rdb = rdb
+      @rdb_index = rdb_index
+      @cell = cellname && rdb.create_cell(cellname)
+      @file_name = file_name
+    end
+
+    def cellname=(cellname)
+      @cell = nil
+      @rdb.each_cell do |c|
+        if c.name == cellname
+          @cell = c
+        end
+      end
+      @cell ||= @rdb.create_cell(cellname)
+    end
+
+    def is_rdb?
+      true
+    end
+
+    def finish(final, view)
+      write
+      if final && view
+        view.show_rdb(@rdb_index, view.active_cellview_index)
+      end
+    end
+
+    def write
+      if @file_name
+        rdb_file = @engine._make_path(@file_name)
+        @engine.info("Writing report database: #{rdb_file} ..")
+        @rdb.save(rdb_file)
+      end
+    end
+
+    def rdb
+      @rdb
+    end
+
+    def cell
+      @cell
+    end
+
+  end
+
   # The DRC engine
   
   # %DRC%
@@ -40,11 +159,9 @@ module DRC
       @def_source = nil
       @dbu_read = false
       use_dbu(@def_layout && @def_layout.dbu)
-      @output_layout = nil
-      @output_rdb = nil
-      @output_rdb_file = nil
-      @output_rdb_cell = nil
       @show_l2ndb = nil
+      @def_output = nil
+      @other_outputs = []
       @output_l2ndb_file = nil
       @target_netlist_file = nil
       @target_netlist_format = nil
@@ -70,6 +187,9 @@ module DRC
       dss._destroy
 
       @verbose = false
+      @profile = false
+      @profile_n = 0
+      @profile_info = {}
 
       @in_context = nil
 
@@ -711,6 +831,26 @@ module DRC
       end
     end
 
+    # %DRC%
+    # @name profile
+    # @brief Profiles the script and provides a runtime + memory statistics
+    # @synopsis profile
+    # @synopsis profile(n)
+    # Turns profiling on or off (default). In profiling mode, the 
+    # system will collect statistics about rules executed, their execution time
+    # and memory information. The argument specifies how many operations to 
+    # print at the end of the run. Without an argument, all operations are
+    # printed. Passing "false" for the argument will disable profiling. This is the
+    # default.
+
+    def profile(n = 0)
+      if !n.is_a?(1.class) && n != nil && n != false
+        raise("Argument to 'profile' must be either an integer number or nil")
+      end
+      @profile = !!n
+      @profile_n = [n || 0, 0].max
+    end
+
     # %DRC%
     # @name verbose?
     # @brief Returns true, if verbose mode is enabled
@@ -1340,46 +1480,42 @@ module DRC
 
       self._context("report") do
 
-        @output_rdb_file = filename
+        # finish what we got so far
+        _finish(false)
 
-        name = filename && File::basename(filename)
-        name ||= "DRC"
-        
-        @output_rdb_index = nil
-
-        view = RBA::LayoutView::current
-        if view
-          if self._rdb_index
-            @output_rdb = RBA::ReportDatabase::new("")   # reuse existing name
-            @output_rdb_index = view.replace_rdb(self._rdb_index, @output_rdb)
-          else
-            @output_rdb = RBA::ReportDatabase::new(name)
-            @output_rdb_index = view.add_rdb(@output_rdb)
-          end
-        else
-          @output_rdb = RBA::ReportDatabase::new(name)
-        end
-        
-        @output_layout = nil
-        @output_cell = nil
-        @output_layout_file = nil
-
-        cn = nil
-        cn ||= @def_cell && @def_cell.name
-        cn ||= source && source.cell_name
-        cn ||= cellname
-
-        cn || raise("No cell name specified - either the source was not specified before 'report' or there is no default source. In the latter case, specify a cell name as the third parameter")
-
-        @output_rdb_cell = @output_rdb.create_cell(cn)
-        @output_rdb.generator = self._generator
-        @output_rdb.top_cell_name = cn
-        @output_rdb.description = description
+        @def_output = nil
+        @def_output = _make_report(description, filename, cellname)
 
       end
       
     end
 
+    # %DRC%
+    # @name new_report
+    # @brief Creates a new report database object for use in "output"
+    # @synopsis new_report(description [, filename [, cellname ] ])
+    # 
+    # This method creates an independent report object. This object
+    # can be used in "output" to send a layer to a different report than
+    # the default report or target.
+    #
+    # Arguments are the same than for \global#report.
+    #
+    # See \Layer#output for details about this feature.
+
+    def new_report(description, filename = nil, cellname = nil)
+
+      output = nil
+
+      self._context("new_report") do
+        output = _make_report(description, filename, cellname)
+        @other_outputs << output
+      end
+
+      output
+      
+    end
+
     # %DRC%
     # @name report_netlist
     # @brief Specifies an extracted netlist report for output
@@ -1456,25 +1592,13 @@ module DRC
         # finish what we got so far
         _flush
         
-        if @output_rdb
-        
-          cell = nil
-          @output_rdb.each_cell do |c|
-            if c.name == cellname
-              cell = c
-            end
+        if ! @def_output
+          if @def_layout
+            # establish a new default output from the default layout on this occasion
+            @def_output = LayoutOutputChannel::new(self, @def_layout, @def_layout.cell(cellname.to_s), nil)
           end
-          
-          cell ||= @output_rdb.create_cell(cellname)
-          @output_rdb_cell = cell
-          
         else
-        
-          @output_layout ||= @def_layout
-          if @output_layout
-            @output_cell = @output_layout.cell(cellname.to_s) || @output_layout.create_cell(cellname.to_s)
-          end
-          
+          @def_output.cellname = cellname.to_s
         end
 
       end
@@ -1512,51 +1636,40 @@ module DRC
     
         # finish what we got so far
         _finish(false)
-            
-        if arg.is_a?(String)
-        
-          if arg =~ /^@(\d+|\+)/
-            view = RBA::LayoutView::current
-            view || raise("No view open")
-            if $1 == "+"
-              n = view.create_layout(true)
-              cellname ||= (@def_cell ? @def_cell.name : "TOP")
-            else
-              n = $1.to_i - 1
-            end
-            (n >= 0 && view.cellviews > n) || raise("Invalid layout index @#{n + 1}")
-            cv = view.cellview(n)
-            cv.is_valid? || raise("Invalid layout @#{n + 1}")
-            @output_layout = cv.layout
-            @output_cell = cellname ? (@output_layout.cell(cellname.to_s) || @output_layout.create_cell(cellname.to_s)) : cv.cell
-            cv.cell = @output_cell
-            @output_layout_file = nil
-          else
-            @output_layout = RBA::Layout::new
-            @output_cell = cellname && @output_layout.create_cell(cellname.to_s)
-            @output_layout_file = arg
-          end
-          
-        elsif arg.is_a?(RBA::Layout)
-        
-          @output_layout = arg
-          @output_cell = cellname && (@output_layout.cell(cellname.to_s) || @output_layout.create_cell(cellname.to_s))
-          @output_layout_file = nil
-          
-        elsif arg.is_a?(RBA::Cell)
-        
-          @output_layout = arg.layout
-          @output_cell = arg
-          @output_layout_file = nil
 
-        else
-          raise("Invalid argument '" + arg.inspect + "'")
-        end
-    
+        @def_output = nil
+        @def_output = _make_target(arg, cellname)
+
       end
 
     end
-    
+            
+    # %DRC%
+    # @name new_target
+    # @brief Creates a new layout target object for use in "output"
+    # @synopsis new_target(what [, cellname])
+    # 
+    # This method creates an independent target object. This object
+    # can be used in "output" to send a layer to a different layout file than
+    # the default report or target.
+    #
+    # Arguments are the same than for \global#target.
+    #
+    # See \Layer#output for details about this feature.
+
+    def new_target(arg, cellname = nil)
+
+      output = nil
+
+      self._context("new_target") do
+        output = _make_target(arg, cellname)
+        @other_outputs << output
+      end
+
+      output
+      
+    end
+
     # %DRC%
     # @name box 
     # @brief Creates a box object
@@ -2164,6 +2277,18 @@ CODE
     # @synopsis device_scaling(factor)
     # See \Netter#device_scaling for a description of that function.
  
+    # %DRC%
+    # @name top_level
+    # @brief Specifies that the circuit is a chip top level circuit
+    # @synopsis top_level(flag)
+    # See \Netter#top_level for a description of that function.
+ 
+    # %DRC%
+    # @name ignore_extraction_errors
+    # @brief Specifies whether to ignore extraction errors
+    # @synopsis ignore_extraction_errors(value)
+    # See \Netter#ignore_extraction_errors for a description of that function.
+ 
     # %DRC%
     # @name extract_devices
     # @brief Extracts devices for a given device extractor and device layer selection
@@ -2186,6 +2311,8 @@ CODE
       connect_implicit
       connect_explicit
       device_scaling
+      top_level
+      ignore_extraction_errors
       extract_devices
       l2n_data
       netlist
@@ -2204,7 +2331,11 @@ CODE
         c !~ /drc.lym:/ && c !~ /_drc_\w+\.rb:/ && c !~ /\(eval\)/
       end
       if cc =~ /(.*)\s*:\s*(\d+)\s*:\s*in.*$/
-        return File::basename($1) + ":" + $2
+        path = $1
+        line = $2.to_i
+        real_path = RBA::Macro::real_path(path, line)
+        real_line = RBA::Macro::real_line(path, line)
+        return File::basename(real_path) + ":" + real_line.to_s
       else
         return cc
       end
@@ -2267,13 +2398,16 @@ CODE
       end
       
       t = RBA::Timer::new
+
       t.start
       self._process_events
-      if @force_gc || Time::now - @time > 0.5
+      if @force_gc || Time::now - @time > 0.5 || @profile
         GC.start    # force a garbage collection before the operation to free unused memory
         @time = Time::now
       end
+      mem_before = RBA::Timer::memory_size
       res = yield
+      mem_after = RBA::Timer::memory_size
       t.stop
 
       begin
@@ -2283,15 +2417,25 @@ CODE
           # Report result statistics
           _result_info(res, 1)
 
-          mem = RBA::Timer::memory_size
-          if mem > 0
-            info("Elapsed: #{'%.3f'%(t.sys+t.user)}s  Memory: #{'%.2f'%(mem/(1024*1024))}M", 1)
+          if mem_after > 0
+            info("Elapsed: #{'%.3f'%(t.sys+t.user)}s  Memory: #{'%.2f'%(mem_after/(1024*1024))}M", 1)
           else
             info("Elapsed: #{'%.3f'%(t.sys+t.user)}s", 1)
           end
 
         end
 
+        if @profile
+
+          # calls, sys time (in sec), user time (in sec), memory added (in bytes)
+          info = (@profile_info[desc] ||= [ 0, 0.0, 0.0, 0 ])
+          info[0] += 1
+          info[1] += t.sys
+          info[2] += t.user
+          info[3] += mem_after - mem_before
+
+        end
+
       ensure
 
         # disable progress again
@@ -2477,8 +2621,8 @@ CODE
     end
 
     def _output_layout
-      if @output_layout 
-        output = @output_layout
+      if @def_output && @def_output.layout
+        output = @def_output.layout
       else
         output = @def_layout
         output || raise("No output layout specified")
@@ -2487,19 +2631,79 @@ CODE
     end
     
     def _output_cell
-      if @output_layout 
-        if @output_cell
-          output_cell = @output_cell
+      if @def_output && @def_output.layout 
+        if @def_output.cell
+          output_cell = @def_output.cell
         elsif @def_cell
-          output_cell = @output_layout.cell(@def_cell.name) || @output_layout.create_cell(@def_cell.name)
+          output_layout = @def_output.layout
+          output_cell = output_layout.cell(@def_cell.name) || output_layout.create_cell(@def_cell.name)
         end
         output_cell || raise("No output cell specified (see 'target' instruction)")
       else
-        output_cell = @output_cell || @def_cell
+        output_cell = @def_cell
         output_cell || raise("No output cell specified")
       end
       output_cell
     end
+
+    def _dump_profile
+
+      if @profile_info.empty?
+        return
+      end
+
+      desc_title = "Operation"
+      calls_title = "# calls"
+      time_title = "Time (s)"
+      memory_title = "Memory (k)"
+      titles = [ desc_title, calls_title, time_title, memory_title ]
+
+      max_len_desc  = [ @profile_info.keys.collect { |s| s.size }.max, desc_title.size ].max
+      max_len_calls = [ @profile_info.values.collect { |v| v[0].to_s.size }.max, calls_title.size ].max
+      max_len_time  = [ @profile_info.values.collect { |v| ("%.3f" % (v[1] + v[2])).to_s.size }.max, time_title.size ].max
+      max_len_mem   = [ @profile_info.values.collect { |v| v[3].to_s.size }.max, memory_title.size ].max
+
+      fmt = "%-" + max_len_desc.to_s + "s  " +
+            "%-" + max_len_calls.to_s + "d  " +
+            "%-" + max_len_time.to_s + ".3f  " + 
+            "%-" + max_len_mem.to_s + ".0f" 
+
+      fmt_title = "%-" + max_len_desc.to_s + "s  " +
+                  "%-" + max_len_calls.to_s + "s  " +
+                  "%-" + max_len_time.to_s + "s  " + 
+                  "%-" + max_len_mem.to_s + "s" 
+
+      pi = @profile_info.keys.collect { |s| [s] + @profile_info[s] }.collect do |desc,calls,sys_time,user_time,memory|
+        [ desc, calls, sys_time + user_time, memory.to_f / 1024.0 ]
+      end
+
+      self.log("")
+      self.log("Operations by execution time\n")
+      self.log(fmt_title % titles)
+      n = 1
+      pi.sort { |a,b| b[2] <=> a[2] }.each do |info|
+        self.log(fmt % info)
+        n += 1
+        if @profile_n > 0 && n > @profile_n
+          self.log("... (%d entries skipped)" % (pi.size - @profile_n))
+          break
+        end
+      end
+
+      self.log("")
+      self.log("Operations by memory adder\n")
+      self.log(fmt_title % titles)
+      n = 1
+      pi.sort { |a,b| b[3] <=> a[3] }.each do |info|
+        self.log(fmt % info)
+        n += 1
+        if @profile_n > 0 && n > @profile_n
+          self.log("... (%d entries skipped)" % (pi.size - @profile_n))
+          break
+        end
+      end
+
+    end
     
     def _start(job_description)
     
@@ -2528,39 +2732,32 @@ CODE
       begin
 
         _flush    
-        
+
         view = RBA::LayoutView::current
 
-        # save the report database if requested
-        if @output_rdb_file && final
-          rdb_file = _make_path(@output_rdb_file)
-          info("Writing report database: #{rdb_file} ..")
-          @output_rdb.save(rdb_file)
+        @def_output && @def_output.finish(final, view)
+
+        if final
+          @other_outputs.each do |output|
+            output.finish(final, view)
+          end
         end
-        if @output_rdb && final && view
-          view.show_rdb(@output_rdb_index, view.active_cellview_index)
-        end
-      
-        # save the output file if requested
-        if @output_layout && @output_layout_file
-          opt = RBA::SaveLayoutOptions::new
-          gzip = opt.set_format_from_filename(@output_layout_file)
-          info("Writing layout file: #{@output_layout_file} ..")
-          @output_layout.write(@output_layout_file, gzip, opt)
+
+        # dump the profile information
+        if @profile && final
+          _dump_profile
         end
         
         # create the new layers as visual layers if necessary
         if view
         
-          output = @output_layout || @def_layout
+          output = ( @def_output && @def_output.layout ) || @def_layout
           cv_index = nil
           view.cellviews.times do |cvi|
             view.cellview(cvi).layout == output && cv_index = cvi
           end
           if cv_index
           
-            view = RBA::LayoutView::current
-            
             # clear selection
             view.cancel 
       
@@ -2634,13 +2831,10 @@ CODE
       ensure
 
         @output_layers = []
-        @output_layout = nil
-        @output_layout_file = nil
-        @output_cell = nil
-        @output_rdb_file = nil
-        @output_rdb_cell = nil
-        @output_rdb = nil
-        @output_rdb_index = nil
+        @def_output = nil
+        if final
+          @other_outputs = []
+        end
         @show_l2ndb = nil
         @output_l2ndb_file = nil
 
@@ -2732,7 +2926,7 @@ CODE
 
     def _make_path(file)
       # resolves the file path relative to the source's path
-      sp = self.source.path
+      sp = (@def_source || @def_layout) && self.source.path
       if sp
         if File.respond_to?(:absolute_path)
           return File::absolute_path(file, File::dirname(sp))
@@ -2933,21 +3127,36 @@ CODE
     
     def _output(data, *args)
 
-      if @output_rdb
+      channel = args.find { |a| a.is_a?(OutputChannel) }
+      if ! channel
+        if ! @def_output
+          @def_output = LayoutOutputChannel::new(self, self._output_layout, self._output_cell, nil)
+        end
+        channel = @def_output
+      end
+
+      args = args.select { |a| !a.is_a?(OutputChannel) }
+
+      if channel.rdb
         
         if args.size < 1
           raise("Invalid number of arguments - category name and optional description expected")
         end
 
-        cat = @output_rdb.create_category(args[0].to_s)
+        output_rdb = channel.rdb
+        output_cell = channel.cell
+
+        cat = output_rdb.create_category(args[0].to_s)
         args[1] && cat.description = args[1]
 
-        cat.scan_collection(@output_rdb_cell, RBA::CplxTrans::new(self.dbu), data)
+        cat.scan_collection(output_cell, RBA::CplxTrans::new(self.dbu), data)
       
-      else 
+      end
 
-        output = self._output_layout
-        output_cell = self._output_cell
+      if channel.layout
+
+        output = channel.layout
+        output_cell = channel.cell
 
         info = nil
         if args.size == 1
@@ -2977,14 +3186,16 @@ CODE
 
         begin
 
-          if !@used_output_layers[li]
-            @output_layers.push(li)
-            # Note: to avoid issues with output onto the input layer, we
-            # output to a temp layer and later swap both. The simple implementation
-            # did a clear here and the effect of that was that the data potentially
-            # got invalidated.
-            tmp = output.insert_layer(RBA::LayerInfo::new)
-            @used_output_layers[li] = true
+          if channel == @def_output
+            if !@used_output_layers[li]
+              @output_layers.push(li)
+              # Note: to avoid issues with output onto the input layer, we
+              # output to a temp layer and later swap both. The simple implementation
+              # did a clear here and the effect of that was that the data potentially
+              # got invalidated.
+              tmp = output.insert_layer(RBA::LayerInfo::new)
+              @used_output_layers[li] = true
+            end
           end
 
           # insert the data into the output layer
@@ -3020,7 +3231,111 @@ CODE
       @layout_sources[name] = src
       src
     end
+    
+    def _make_report(description, filename, cellname)
 
+      output_rdb_file = filename
+
+      name = filename && File::basename(filename)
+      name ||= "DRC"
+      
+      output_rdb_index = nil
+
+      view = RBA::LayoutView::current
+      if view
+        if self._rdb_index
+          output_rdb = RBA::ReportDatabase::new("")   # reuse existing name
+          output_rdb_index = view.replace_rdb(self._rdb_index, output_rdb)
+        else
+          output_rdb = RBA::ReportDatabase::new(name)
+          output_rdb_index = view.add_rdb(output_rdb)
+        end
+      else
+        output_rdb = RBA::ReportDatabase::new(name)
+      end
+      
+      cn = cellname && cellname.to_s
+      cn ||= @def_cell && @def_cell.name
+      cn ||= @def_source && @def_source.cell_obj && @def_source.cell_obj.name
+
+      cn || raise("No cell name specified - either the source was not specified before 'report' or there is no default source. In the latter case, specify a cell name as the third parameter")
+
+      output_rdb.generator = self._generator
+      output_rdb.top_cell_name = cn
+      output_rdb.description = description
+
+      RDBOutputChannel::new(self, output_rdb, output_rdb_index, cn, output_rdb_file)
+      
+    end
+
+    def _make_target(arg, cellname = nil)
+
+      if arg.is_a?(String)
+      
+        if arg =~ /^@(\d+|\+)/
+
+          view = RBA::LayoutView::current
+          view || raise("No view open")
+          if $1 == "+"
+            n = view.create_layout(true)
+            cellname ||= (@def_cell ? @def_cell.name : "TOP")
+          else
+            n = $1.to_i - 1
+          end
+
+          (n >= 0 && view.cellviews > n) || raise("Invalid layout index @#{n + 1}")
+
+          cv = view.cellview(n)
+          cv.is_valid? || raise("Invalid layout @#{n + 1}")
+
+          output_layout = cv.layout
+          output_cell = cellname ? (output_layout.cell(cellname.to_s) || output_layout.create_cell(cellname.to_s)) : cv.cell
+          cv.cell = output_cell
+          output_layout_file = nil
+
+        else
+
+          cn = cellname && cellname.to_s
+          cn ||= @def_cell && @def_cell.name
+          cn ||= @def_source && @def_source.cell_obj && @def_source.cell_obj.name
+
+          cn || raise("No cell name specified - either the source was not specified before 'report' or there is no default source. In the latter case, specify a cell name as the second parameter")
+
+          output_layout = RBA::Layout::new
+          output_cell = output_layout.create_cell(cn)
+          output_layout_file = arg
+
+        end
+        
+      elsif arg.is_a?(RBA::Layout)
+      
+        output_layout = arg
+
+        output_cell = cellname && (output_layout.cell(cellname.to_s) || output_layout.create_cell(cellname.to_s))
+        if ! output_cell
+          begin
+            output_cell = output_layout.top_cell
+          rescue 
+            raise("No cell name specified and the layout does not have a unique top cell - specify the name of the top cell to write the output to")
+          end
+        end
+
+        output_layout_file = nil
+        
+      elsif arg.is_a?(RBA::Cell)
+      
+        output_layout = arg.layout
+        output_cell = arg
+        output_layout_file = nil
+
+      else
+        raise("Invalid argument '" + arg.inspect + "'")
+      end
+    
+      output = LayoutOutputChannel::new(self, output_layout, output_cell, output_layout_file)
+
+    end
+    
   end
  
 end
