@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -731,6 +731,146 @@ TEST(14_SwitchingTargets)
   db::compare_layouts (_this, layout2, au2, db::NoNormalization);
 }
 
+TEST(14b_SideTargetsAndReports)
+{
+  std::string rs = tl::testdata ();
+  rs += "/drc/drcSimpleTests_14b.drc";
+
+  std::string input = tl::testdata ();
+  input += "/drc/drcSimpleTests_14b.gds";
+
+  std::string au = tl::testdata ();
+  au += "/drc/drcSimpleTests_au14b.gds";
+
+  std::string au2 = tl::testdata ();
+  au2 += "/drc/drcSimpleTests_au14b_2.gds";
+
+  std::string au_report = tl::testdata ();
+  au_report += "/drc/drcSimpleTests_au14b.lyrdb";
+
+  std::string au_report2 = tl::testdata ();
+  au_report2 += "/drc/drcSimpleTests_au14b_2.lyrdb";
+
+  std::string output = this->tmp_file ("tmp.gds");
+  std::string output2 = this->tmp_file ("tmp2.gds");
+  std::string report = this->tmp_file ("tmp.lydrc");
+  std::string report2 = this->tmp_file ("tmp2.lydrc");
+
+  {
+    //  Set some variables
+    lym::Macro config;
+    config.set_text (tl::sprintf (
+        "$drc_force_gc = true\n"
+        "$drc_test_source = '%s'\n"
+        "$drc_test_target = '%s'\n"
+        "$drc_test_target2 = '%s'\n"
+        "$drc_test_report = '%s'\n"
+        "$drc_test_report2 = '%s'\n"
+      , input, output, output2, report, report2)
+    );
+    config.set_interpreter (lym::Macro::Ruby);
+    EXPECT_EQ (config.run (), 0);
+  }
+
+  lym::Macro drc;
+  drc.load_from (rs);
+  EXPECT_EQ (drc.run (), 0);
+
+  db::Layout layout;
+
+  {
+    tl::InputStream stream (output);
+    db::Reader reader (stream);
+    reader.read (layout);
+  }
+
+  db::compare_layouts (_this, layout, au, db::NoNormalization);
+
+  db::Layout layout2;
+
+  {
+    tl::InputStream stream (output2);
+    db::Reader reader (stream);
+    reader.read (layout2);
+  }
+
+  db::compare_layouts (_this, layout2, au2, db::NoNormalization);
+
+  compare_text_files (report, au_report);
+  compare_text_files (report2, au_report2);
+}
+
+TEST(14c_OnlySpecialInputsAndReports)
+{
+  std::string rs = tl::testdata ();
+  rs += "/drc/drcSimpleTests_14c.drc";
+
+  //  apart from that it's a variant of 14b ...
+
+  std::string input = tl::testdata ();
+  input += "/drc/drcSimpleTests_14b.gds";
+
+  std::string au = tl::testdata ();
+  au += "/drc/drcSimpleTests_au14b.gds";
+
+  std::string au2 = tl::testdata ();
+  au2 += "/drc/drcSimpleTests_au14b_2.gds";
+
+  std::string au_report = tl::testdata ();
+  au_report += "/drc/drcSimpleTests_au14b.lyrdb";
+
+  std::string au_report2 = tl::testdata ();
+  au_report2 += "/drc/drcSimpleTests_au14b_2.lyrdb";
+
+  std::string output = this->tmp_file ("tmp.gds");
+  std::string output2 = this->tmp_file ("tmp2.gds");
+  std::string report = this->tmp_file ("tmp.lydrc");
+  std::string report2 = this->tmp_file ("tmp2.lydrc");
+
+  {
+    //  Set some variables
+    lym::Macro config;
+    config.set_text (tl::sprintf (
+        "$drc_force_gc = true\n"
+        "$drc_test_source = '%s'\n"
+        "$drc_test_target = '%s'\n"
+        "$drc_test_target2 = '%s'\n"
+        "$drc_test_report = '%s'\n"
+        "$drc_test_report2 = '%s'\n"
+      , input, output, output2, report, report2)
+    );
+    config.set_interpreter (lym::Macro::Ruby);
+    EXPECT_EQ (config.run (), 0);
+  }
+
+  lym::Macro drc;
+  drc.load_from (rs);
+  EXPECT_EQ (drc.run (), 0);
+
+  db::Layout layout;
+
+  {
+    tl::InputStream stream (output);
+    db::Reader reader (stream);
+    reader.read (layout);
+  }
+
+  db::compare_layouts (_this, layout, au, db::NoNormalization);
+
+  db::Layout layout2;
+
+  {
+    tl::InputStream stream (output2);
+    db::Reader reader (stream);
+    reader.read (layout2);
+  }
+
+  db::compare_layouts (_this, layout2, au2, db::NoNormalization);
+
+  compare_text_files (report, au_report);
+  compare_text_files (report2, au_report2);
+}
+
 TEST(15_issue548)
 {
   std::string rs = tl::testdata ();
@@ -1302,6 +1442,16 @@ TEST(47_fillWithOverlappingBoxesTiled)
   run_test (_this, "47", false);
 }
 
+TEST(47b_fillWithUsingOutput)
+{
+  run_test (_this, "47b", false);
+}
+
+TEST(47bd_fillWithUsingOutputDeep)
+{
+  run_test (_this, "47b", true);
+}
+
 TEST(48_drcWithFragments)
 {
   run_test (_this, "48", false);
@@ -1397,6 +1547,16 @@ TEST(60d_issue1216)
   run_test (_this, "60", true);
 }
 
+TEST(61_issue1485)
+{
+  run_test (_this, "61", false);
+}
+
+TEST(61d_issue1485)
+{
+  run_test (_this, "61", true);
+}
+
 TEST(70_props)
 {
   run_test (_this, "70", false);
@@ -1406,3 +1566,54 @@ TEST(70d_props)
 {
   run_test (_this, "70", true);
 }
+
+TEST(80_deep_with_mag_width)
+{
+  run_test (_this, "80", true);
+}
+
+TEST(81_deep_with_mag_space)
+{
+  run_test (_this, "81", true);
+}
+
+TEST(82_deep_with_mag_cop_width)
+{
+  run_test (_this, "82", true);
+}
+
+TEST(83_deep_with_mag_cop_space)
+{
+  run_test (_this, "83", true);
+}
+
+TEST(84_deep_with_mag_edge_width)
+{
+  run_test (_this, "84", true);
+}
+
+TEST(85_deep_with_mag_edge_space)
+{
+  run_test (_this, "85", true);
+}
+
+TEST(86_deep_with_mag_size)
+{
+  run_test (_this, "86", true);
+}
+
+TEST(87_deep_with_mag_size_aniso)
+{
+  run_test (_this, "87", true);
+}
+
+TEST(88_deep_with_mag_cop_size)
+{
+  run_test (_this, "88", true);
+}
+
+TEST(89_deep_with_mag_cop_size_aniso)
+{
+  run_test (_this, "89", true);
+}
+
