@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -266,6 +266,26 @@ public:
    */
   virtual bool has_transient_selection ();
 
+  /**
+   *  @brief Gets the selection bounding box
+   */
+  virtual db::DBox selection_bbox ();
+
+  /**
+   *  @brief Start a "move" operation
+   */
+  virtual bool begin_move (MoveMode sel, const db::DPoint &p, lay::angle_constraint_type ac);
+
+  /**
+   *  @brief Continue a "move" operation
+   */
+  virtual void move (const db::DPoint &p, lay::angle_constraint_type ac);
+
+  /**
+   *  @brief Terminate a "move" operation
+   */
+  virtual void end_move (const db::DPoint &p, lay::angle_constraint_type ac);
+
   /**
    *  @brief Implement the "select" method at least to clear the selection
    */
@@ -321,6 +341,7 @@ private:
   lay::angle_constraint_type m_connect_ac, m_move_ac, m_alt_ac;
   db::DVector m_edit_grid;
   bool m_snap_to_objects;
+  bool m_snap_objects_to_grid;
   db::DVector m_global_grid;
   bool m_top_level_sel;
 
@@ -353,6 +374,9 @@ private:
   db::DPoint snap (const db::DPoint &p) const;
   db::DVector snap (const db::DVector &p) const;
   lay::PointSnapToObjectResult snap2 (const db::DPoint &p) const;
+  void update_vector_snapped_point (const db::DPoint &pt, db::DVector &vr, bool &result_set) const;
+  db::DVector snap_marker_to_grid (const db::DVector &v, bool &snapped) const;
+  db::DVector snap_move(const db::DVector &p) const;
 
   void enter_edge (const EdgeWithIndex &e, size_t &nmarker, partial_objects::const_iterator sel, const std::map <PointWithIndex, db::Point> &new_points, const std::map <EdgeWithIndex, db::Edge> &new_edges, const db::ICplxTrans &gt, const std::vector<db::DCplxTrans> &tv, bool transient);
   void enter_vertices (size_t &nmarker, partial_objects::const_iterator sel, const std::map <PointWithIndex, db::Point> &new_points, const std::map <EdgeWithIndex, db::Edge> &new_edges, const db::ICplxTrans &gt, const std::vector<db::DCplxTrans> &tv, bool transient);
@@ -363,7 +387,9 @@ private:
   void resize_markers (size_t n, bool transient);
   void resize_inst_markers (size_t n, bool transient);
   bool is_single_point_selection () const;
+  bool is_single_edge_selection () const;
   db::DPoint single_selected_point () const;
+  db::DEdge single_selected_edge () const;
   bool handle_guiding_shape_changes ();
   void transform_selection (const db::DTrans &move_trans);
 };
