@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -131,6 +131,22 @@ private:
   db::Coord m_l, m_b, m_r, m_t;
 };
 
+/**
+ *  @brief A point change applicator
+ */
+class PointDimensionsChangeApplicator
+  : public ChangeApplicator
+{
+public:
+  PointDimensionsChangeApplicator (const db::Point &point, const db::Point &org_point);
+
+  bool supports_relative_mode () const { return true; }
+  db::Shape do_apply (db::Shapes &shapes, const db::Shape &shape, double dbu, bool relative) const;
+
+private:
+  db::Point m_point, m_org_point;
+};
+
 /**
  *  @brief A polygon change applicator
  */
