@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include "edtService.h"
 #include "ui_PolygonPropertiesPage.h"
 #include "ui_BoxPropertiesPage.h"
+#include "ui_PointPropertiesPage.h"
 #include "ui_PathPropertiesPage.h"
 #include "ui_EditablePathPropertiesPage.h"
 #include "ui_TextPropertiesPage.h"
@@ -152,6 +153,34 @@ private:
   void set_box (const db::Box &box);
 };
 
+class PointPropertiesPage
+  : public ShapePropertiesPage,
+    public Ui::PointPropertiesPage
+{
+Q_OBJECT
+
+public:
+  PointPropertiesPage (edt::Service *service, db::Manager *manager, QWidget *parent);
+
+  virtual std::string description (size_t entry) const;
+  virtual std::string description () const { return ShapePropertiesPage::description (); }
+  virtual void do_update (const db::Shape &shape, double dbu, const std::string &lname);
+  virtual ChangeApplicator *create_applicator (db::Shapes &shapes, const db::Shape &shape, double dbu);
+
+public slots:
+  void changed ();
+
+protected:
+  virtual QCheckBox *dbu_checkbox () const { return dbu_cb; }
+  virtual QCheckBox *abs_checkbox () const { return abs_cb; }
+
+private:
+  double m_dbu;
+
+  db::Point get_point () const;
+  void set_point (const db::Point &point);
+};
+
 class TextPropertiesPage
   : public ShapePropertiesPage,
     public Ui::TextPropertiesPage
