@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -64,6 +64,7 @@
 #include "tlHttpStream.h"
 #include "tlArch.h"
 #include "tlFileUtils.h"
+#include "tlEnv.h"
 
 #include <QIcon>
 #include <QDir>
@@ -124,9 +125,9 @@ static void ui_exception_handler_tl (const tl::Exception &ex, QWidget *parent)
 
     if (gsi_excpt->line () > 0) {
       tl::error << gsi_excpt->sourcefile () << ":" << gsi_excpt->line () << ": " 
-                << gsi_excpt->msg () << tl::to_string (QObject::tr (" (class ")) << gsi_excpt->cls () << ")";
+                << gsi_excpt->msg ();
     } else {
-      tl::error << gsi_excpt->msg () << tl::to_string (QObject::tr (" (class ")) << gsi_excpt->cls () << ")";
+      tl::error << gsi_excpt->msg ();
     }
 
     lay::RuntimeErrorForm error_dialog (parent, "ruby_error_form", gsi_excpt);
@@ -317,6 +318,11 @@ ApplicationBase::parse_cmd (int &argc, char **argv)
       }
       tl::verbosity (v);
 
+    } else if (a == "-k" && (i + 1) < argc) {
+
+      //  ignored (handled earlier)
+      ++i;
+
     } else if (a == "-l" && (i + 1) < argc) {
 
       m_layer_props_file = args [++i];
@@ -563,6 +569,19 @@ ApplicationBase::init_app ()
   }
   std::string short_arch_string = tl::join (as, "-");
 
+#if defined(_WIN32)
+  //  Set SSL_CERT_FILE for Windows installation and libcrypto.
+
+  std::string cert_file = "cert.pem";
+  std::string cert_env_var = "SSL_CERT_FILE";
+  if (! tl::has_env (cert_env_var)) {
+    std::string cert_path = tl::combine_path (m_inst_path, cert_file);
+    if (tl::file_exists (cert_path)) {
+      tl::set_env (cert_env_var, cert_path);
+    }
+  }
+#endif
+
   std::vector<std::string> klp_paths;
 
   for (std::vector <std::string>::const_iterator p = m_klayout_path.begin (); p != m_klayout_path.end (); ++p) {
@@ -686,28 +705,59 @@ ApplicationBase::init_app ()
       tc->add_path (*p);
     }
 
+    tc->load ();
+
+    bool needs_reload = false;
+
+    //  disambiguator for tech name
+    std::map<std::string, int> tech_disambiguator;
+    std::map<std::string, std::string> tech_name_for_file;
+
+    for (auto t = db::Technologies::instance ()->begin (); t != db::Technologies::instance ()->end (); ++t) {
+      tech_disambiguator.insert (std::make_pair (t->name (), 0));
+    }
+
     //  import technologies from the command line
     for (std::vector <std::pair<file_type, std::pair<std::string, std::string> > >::iterator f = m_files.begin (); f != m_files.end (); ++f) {
 
       if (f->first == layout_file_with_tech_file) {
 
+        const std::string &tech_file = f->second.second;
+
         if (tl::verbosity () >= 20) {
           tl::info << "Importing technology from " << f->second.second;
         }
 
         db::Technology t;
-        t.load (f->second.second);
+        t.load (tech_file);
+
+        //  disambiguate the name, so we do not overwrite technologies with the same name from the config
+        if (tech_name_for_file.find (tech_file) != tech_name_for_file.end ()) {
+          t.set_name (tech_name_for_file [tech_file]);
+        } else if (tech_disambiguator.find (t.name ()) != tech_disambiguator.end ()) {
+          int &index = tech_disambiguator [t.name ()];
+          index += 1;
+          t.set_name (t.name () + tl::sprintf ("[%d]", index));
+          tech_name_for_file.insert (std::make_pair (tech_file, t.name ()));
+        } else {
+          tech_disambiguator.insert (std::make_pair (t.name (), 0));
+          tech_name_for_file.insert (std::make_pair (tech_file, t.name ()));
+        }
 
         tc->add_temp_tech (t);
 
         f->first = layout_file_with_tech;
         f->second.second = t.name ();
 
+        needs_reload = true;
+
       }
 
     }
 
-    tc->load ();
+    if (needs_reload) {
+      tc->load ();
+    }
 
   }
 
@@ -739,7 +789,7 @@ ApplicationBase::init_app ()
     size_t local_folders = (lay::get_appdata_path ().empty () ? 0 : 1);
 
     for (std::vector <std::string>::const_iterator p = m_klayout_path.begin (); p != m_klayout_path.end (); ++p) {
-      if (p - m_klayout_path.begin () < local_folders) {
+      if (size_t (p - m_klayout_path.begin ()) < local_folders) {
         mc->add_path (*p, tl::to_string (QObject::tr ("Local")), std::string (), false);
       } else if (m_klayout_path.size () == 1 + local_folders) {
         mc->add_path (*p, tl::to_string (QObject::tr ("Global")), std::string (), true);
@@ -786,10 +836,8 @@ ApplicationBase::init_app ()
     }
   }
 
-  std::set<std::string> already_executed;
-
   //  run all early autorun macros
-  lym::MacroCollection::root ().autorun_early (&already_executed);
+  lym::MacroCollection::root ().autorun_early ();
 
   //  redo gsi::initialize as the macros may have registered new external classes
   //  through the "doc to external class" mechanism.
@@ -801,8 +849,8 @@ ApplicationBase::init_app ()
     mc->finish ();
 
     //  as this regenerates the macro collection, autorun_early is required again
-    //  note: this does no re-execute macros that have been executed already
-    lym::MacroCollection::root ().autorun_early (&already_executed);
+    //  note: this does not re-execute macros that have been executed already
+    lym::MacroCollection::root ().autorun_early ();
 
   }
 
@@ -811,7 +859,7 @@ ApplicationBase::init_app ()
   lym::MacroCollection::root ().rescan ();
 
   //  and yet another autorun_early pass ..
-  lym::MacroCollection::root ().autorun_early (&already_executed);
+  lym::MacroCollection::root ().autorun_early ();
 
   //  creates the main window or plugin root as required
   setup ();
@@ -1026,6 +1074,7 @@ ApplicationBase::usage ()
   r += tl::to_string (QObject::tr ("  -i                  Disable undo buffering (less memory requirements)")) + "\n";
   r += tl::to_string (QObject::tr ("  -ni                 Enable undo buffering (default, overrides previous -i option)")) + "\n";
   r += tl::to_string (QObject::tr ("  -j <path>           Add the given path to the macro project paths")) + "\n"; 
+  r += tl::to_string (QObject::tr ("  -k <log file>       Write log to the given file plus stdout/stderr")) + "\n";
   r += tl::to_string (QObject::tr ("  -l <lyp file>       Use layer properties file")) + "\n";
   r += tl::to_string (QObject::tr ("  -lx                 With -l: add other layers as well")) + "\n";
   r += tl::to_string (QObject::tr ("  -lf                 With -l: use the lyp file as it is (no expansion to multiple layouts)")) + "\n";
@@ -1480,6 +1529,18 @@ GuiApplication::initialize ()
 bool
 GuiApplication::notify (QObject *receiver, QEvent *e)
 {
+  QWheelEvent *wheel_event = dynamic_cast<QWheelEvent *>(e);
+  if (wheel_event) {
+    //  intercept wheel events targeting QComboBox objects to avoid
+    //  changing them through wheel actions.
+    for (auto r = receiver; r != 0; r = r->parent ()) {
+      if (dynamic_cast<QComboBox *>(r)) {
+        //  stop further processing
+        return true;
+      }
+    }
+  }
+
   if (dynamic_cast<QPaintEvent *> (e)) {
     //  NOTE: we don't want recursive paint events - the painters are not reentrant.
     //  Hence we disable process_events_impl (specifically for progress reporters).
@@ -1543,18 +1604,18 @@ GuiApplication::force_update_app_menu ()
 #endif
 }
 
-#if defined(__APPLE__)
-// By Thomas Lima (March 7, 2018)
-// 
-// This event interceptor catches MacOS "Open With" event, and KLayout should respond 
-// similarly to the Drop event in MainWindow::dropEvent.
-// 
-// This particular implementation always creates a new window.
-//
-// This was implemented with the inspiration of http://doc.qt.io/qt-5/qfileopenevent.html
 bool
 GuiApplication::event (QEvent *event)
 {
+#if defined(__APPLE__)
+  // By Thomas Lima (March 7, 2018)
+  //
+  // This event interceptor catches MacOS "Open With" event, and KLayout should respond
+  // similarly to the Drop event in MainWindow::dropEvent.
+  //
+  // This particular implementation always creates a new window.
+  //
+  // This was implemented with the inspiration of http://doc.qt.io/qt-5/qfileopenevent.html
   if (event->type() == QEvent::FileOpen) {
       QFileOpenEvent *openEvent = static_cast<QFileOpenEvent *>(event);
       if (mp_mw)
@@ -1566,10 +1627,10 @@ GuiApplication::event (QEvent *event)
         mp_mw->add_mru (file, tech);
       }
   }
+#endif
 
   return QApplication::event(event);
 }
-#endif
 
 
 int
