@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -152,7 +152,7 @@ void init (const std::vector<std::string> &_paths)
         try {
           s_plugins.push_back (do_load_plugin (imp));
           modules.insert (*im);
-        } catch (tl::Exception (&ex)) {
+        } catch (tl::Exception &ex) {
           tl::error << ex.msg ();
         }
       }
