@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -56,19 +56,30 @@ MacroController::add_macro_category (const std::string &name, const std::string
   cat.name = name;
   cat.description = description;
   cat.folders = folders;
+
+  //  replace an existing category or add to back
+  for (auto c = m_macro_categories.begin (); c != m_macro_categories.end (); ++c) {
+    if (c->name == name) {
+      *c = cat;
+      return;
+    }
+  }
+
   m_macro_categories.push_back (cat);
 }
 
 void
 MacroController::finish ()
 {
-  lym::MacroCollection::root ().clear ();
+  lym::MacroCollection &root = lym::MacroCollection::root ();
+
+  root.clear ();
 
   //  Scan built-in macros
   //  These macros are always taken, even if there are no macros requested (they are required to
   //  fully form the API).
-  lym::MacroCollection::root ().add_folder (tl::to_string (QObject::tr ("Built-In")), ":/built-in-macros", "macros", true);
-  lym::MacroCollection::root ().add_folder (tl::to_string (QObject::tr ("Built-In")), ":/built-in-pymacros", "pymacros", true);
+  root.add_folder (tl::to_string (QObject::tr ("Built-In")), ":/built-in-macros", "macros", true);
+  root.add_folder (tl::to_string (QObject::tr ("Built-In")), ":/built-in-pymacros", "pymacros", true);
 
   //  scans the macros from techs and packages (this will allow autorun-early on them)
   //  and updates m_external_paths
@@ -92,13 +103,13 @@ MacroController::finish ()
               description += " - " + tl::to_string (tr ("%1 branch").arg (tl::to_qstring (*f)));
             }
 
-            lym::MacroCollection::root ().add_folder (description, mp, m_macro_categories[c].name, p->readonly);
+            root.add_folder (description, mp, m_macro_categories[c].name, p->readonly);
 
           }
 
         } else if (p->cat == m_macro_categories[c].name) {
 
-          lym::MacroCollection::root ().add_folder (p->description, p->path, m_macro_categories[c].name, p->readonly);
+          root.add_folder (p->description, p->path, m_macro_categories[c].name, p->readonly);
 
         }
 
@@ -121,7 +132,7 @@ MacroController::finish ()
 
   if (! m_no_implicit_macros) {
     for (std::vector <ExternalPathDescriptor>::const_iterator p = m_external_paths.begin (); p != m_external_paths.end (); ++p) {
-      lym::MacroCollection *mc = lym::MacroCollection::root ().add_folder (p->description, p->path, p->cat, p->readonly);
+      lym::MacroCollection *mc = root.add_folder (p->description, p->path, p->cat, p->readonly);
       if (mc) {
         mc->set_virtual_mode (p->type);
       }
@@ -377,22 +388,21 @@ MacroController::sync_package_paths ()
 void
 MacroController::sync_implicit_macros (bool ask_before_autorun)
 {
+  //  gets the external paths (tech, packages) into m_external_paths
+  sync_macro_sources ();
+
   if (m_no_implicit_macros) {
 
-    sync_macro_sources ();
     sync_package_paths ();
 
   } else {
 
     //  determine the paths currently in use
-    std::map<std::string, const ExternalPathDescriptor *> prev_folders_by_path;
+    std::map<std::string, ExternalPathDescriptor> prev_folders_by_path;
     for (std::vector<ExternalPathDescriptor>::const_iterator p = m_external_paths.begin (); p != m_external_paths.end (); ++p) {
-      prev_folders_by_path.insert (std::make_pair (p->path, p.operator-> ()));
+      prev_folders_by_path.insert (std::make_pair (p->path, *p));
     }
 
-    //  gets the external paths (tech, packages) into m_external_paths
-    sync_macro_sources ();
-
     //  delete macro collections which are no longer required or update description
 
     std::vector<lym::MacroCollection *> folders_to_delete;
@@ -433,23 +443,39 @@ MacroController::sync_implicit_macros (bool ask_before_autorun)
 
     for (std::vector<ExternalPathDescriptor>::const_iterator p = m_external_paths.begin (); p != m_external_paths.end (); ++p) {
 
-      if (prev_folders_by_path.find (p->path) != prev_folders_by_path.end ()) {
-        continue;
-      }
+      auto pf = prev_folders_by_path.find (p->path);
+      if (pf != prev_folders_by_path.end ()) {
 
-      if (tl::verbosity () >= 20) {
-        tl::info << "Adding macro folder " << p->path << ", category '" << p->cat << "' for '" << p->description << "'";
-      }
+        if (pf->second.version != p->version) {
 
-      //  Add the folder. Note: it may happen that a macro folder for the tech specific macros already exists in
-      //  a non-tech context.
-      //  In that case, the add_folder method will return 0.
+          if (tl::verbosity () >= 20) {
+            tl::info << "New version (" << p->version << " vs. " << pf->second.version << ") of macro folder " << p->path << ", category '" << p->cat << "' for '" << p->description << "'";
+          }
+
+          lym::MacroCollection *mc = lym::MacroCollection::root ().folder_by_name (p->path);
+          if (mc) {
+            new_folders.push_back (mc);
+          }
+
+        }
+
+      } else {
+
+        if (tl::verbosity () >= 20) {
+          tl::info << "Adding macro folder " << p->path << ", category '" << p->cat << "' for '" << p->description << "'";
+        }
+
+        //  Add the folder. Note: it may happen that a macro folder for the tech specific macros already exists in
+        //  a non-tech context.
+        //  In that case, the add_folder method will return 0.
+
+        //  TODO: is it wise to make this writeable?
+        lym::MacroCollection *mc = lym::MacroCollection::root ().add_folder (p->description, p->path, p->cat, p->readonly);
+        if (mc) {
+          mc->set_virtual_mode (p->type);
+          new_folders.push_back (mc);
+        }
 
-      //  TODO: is it wise to make this writeable?
-      lym::MacroCollection *mc = lym::MacroCollection::root ().add_folder (p->description, p->path, p->cat, p->readonly);
-      if (mc) {
-        mc->set_virtual_mode (p->type);
-        new_folders.push_back (mc);
       }
 
     }
@@ -565,7 +591,7 @@ MacroController::sync_macro_sources ()
             if (*f != macro_categories () [c].name) {
               description += " - " + tl::to_string (tr ("%1 branch").arg (tl::to_qstring (*f)));
             }
-            external_paths.push_back (ExternalPathDescriptor (tl::to_string (macro_dir.path ()), description, macro_categories () [c].name, lym::MacroCollection::SaltFolder, g->is_readonly ()));
+            external_paths.push_back (ExternalPathDescriptor (tl::to_string (macro_dir.path ()), description, macro_categories () [c].name, lym::MacroCollection::SaltFolder, g->is_readonly (), g->version ()));
 
           }
 
