@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,9 +23,11 @@
 #include "laySaltController.h"
 #include "laySaltManagerDialog.h"
 #include "laySaltDownloadManager.h"
+#include "laySaltParsedURL.h"
 #include "layConfig.h"
 #include "layMainWindow.h"
 #include "layQtTools.h"
+#include "layBusy.h"
 #include "tlLog.h"
 
 #include <QDir>
@@ -138,11 +140,12 @@ SaltController::show_editor ()
       lay::restore_dialog_state (mp_salt_dialog, s);
     }
 
-    //  while running the dialog, don't watch file events - that would interfere with
-    //  the changes applied by the dialog itself.
-    m_file_watcher->enable (false);
-    mp_salt_dialog->exec ();
-    m_file_watcher->enable (true);
+    {
+      //  while running the dialog, don't watch file events - that would interfere with
+      //  the changes applied by the dialog itself.
+      tl::FileSystemWatcherDisabled disable_file_watcher;  //  disable file watcher
+      mp_salt_dialog->exec ();
+    }
 
     mp_plugin_root->config_set (cfg_salt_manager_window_state, lay::save_dialog_state (mp_salt_dialog));
 
@@ -154,13 +157,13 @@ SaltController::show_editor ()
 void
 SaltController::sync_file_watcher ()
 {
+  tl::FileSystemWatcherDisabled disable_file_watcher;  //  disable file watcher
+
   if (m_file_watcher) {
     m_file_watcher->clear ();
-    m_file_watcher->enable (false);
     for (lay::Salt::flat_iterator g = m_salt.begin_flat (); g != m_salt.end_flat (); ++g) {
       m_file_watcher->add_file ((*g)->path ());
     }
-    m_file_watcher->enable (true);
   }
 }
 
@@ -200,12 +203,19 @@ SaltController::install_packages (const std::vector<std::string> &packages, bool
       }
     }
 
-    if (n.find ("http:") == 0 || n.find ("https:") == 0 || n.find ("file:") == 0 || n[0] == '/' || n[0] == '\\') {
-      //  it's a URL
+    lay::SaltParsedURL purl (n);
+    const std::string &url = purl.url ();
+
+    if (url.find ("http:") == 0 || url.find ("https:") == 0 || url.find ("file:") == 0 || url[0] == '/' || url[0] == '\\') {
+
+      //  its a URL
       manager.register_download (std::string (), std::string (), n, v);
+
     } else {
-      //  it's a plain name
+
+      //  its a plain name
       manager.register_download (n, std::string (), std::string (), v);
+
     }
 
   }
@@ -216,7 +226,18 @@ SaltController::install_packages (const std::vector<std::string> &packages, bool
     manager.compute_packages (m_salt, salt_mine);
   }
 
-  return manager.execute (0, m_salt);
+  bool result = false;
+
+  {
+    //  while running the dialog, don't watch file events - that would interfere with
+    //  the changes applied by the dialog itself.
+    tl::FileSystemWatcherDisabled disable_file_watcher;  //  disable file watcher
+    result = manager.execute (0, m_salt);
+  }
+
+  sync_file_watcher ();
+
+  return result;
 }
 
 void
