@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -1189,6 +1189,16 @@ Class<lay::LayerPropertiesNode> decl_LayerPropertiesNode (
     "Unlike the name suggests, this node will still contain a hierarchy of nodes below if the original "
     "node did so."
   ) +
+  method ("is_expanded?", &lay::LayerPropertiesNode::expanded,
+    "@brief Gets a value indicating whether the layer tree node is expanded.\n"
+    "This predicate has been introduced in version 0.28.6."
+  ) +
+  method ("expanded=", &lay::LayerPropertiesNode::set_expanded, gsi::arg ("ex"),
+    "@brief Set a value indicating whether the layer tree node is expanded.\n"
+    "Setting this value to 'true' will expand (open) the tree node. Setting it to 'false' will collapse the node.\n"
+    "\n"
+    "This predicate has been introduced in version 0.28.6."
+  ) +
   method_ext ("add_child", &add_child0,
     "@brief Add a child entry\n"
     "@return A reference to the node created\n"
@@ -1231,7 +1241,7 @@ Class<lay::LayerPropertiesNode> decl_LayerPropertiesNode (
     "Each layer properties node object has a unique ID that is created \n"
     "when a new LayerPropertiesNode object is instantiated. The ID is\n"
     "copied when the object is copied. The ID can be used to identify the\n"
-    "object irregardless of it's content.\n"
+    "object irregardless of its content.\n"
   ),
   "@brief A layer properties node structure\n"
   "\n"
@@ -1431,7 +1441,7 @@ Class<lay::LayerPropertiesConstIterator> decl_LayerPropertiesIterator (
     "@brief Returns the index of the child within the parent\n"
     "\n"
     "This method returns the index of that the properties node the iterator points to in the list\n"
-    "of children of it's parent. If the element does not have a parent, the \n"
+    "of children of its parent. If the element does not have a parent, the \n"
     "index of the element in the global list is returned.\n"
   ),  
   "@brief Layer properties iterator\n"
