@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -144,6 +144,14 @@ public:
    */
   static LayoutHandle *find (const std::string &name);
 
+  /**
+   *  @brief Finds a handle by layout object
+   *
+   *  @param layout The Layout object bound to the handle
+   *  @return 0, if there is no layout object with this name. Otherwise a pointer to its handle
+   */
+  static LayoutHandle *find_layout (const db::Layout *layout);
+
   /**
    *  @brief Gets the names of all registered layout objects
    */
