@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -919,6 +919,11 @@ protected:
    */
   virtual void need_realize (unsigned int flags, bool force = false);
 
+  /**
+   *  @brief indicates a change of the collapsed/expanded state
+   */
+  virtual void expanded_state_changed ();
+
   /**
    *  @brief Fetches the current status from the original properties for the LayerPropertiesNodeRef implementation
    */
@@ -1069,6 +1074,20 @@ public:
     return r;
   }
 
+  /**
+   *  @brief Sets the expanded state of the layer properties tree node
+   */
+  void set_expanded (bool ex);
+
+  /**
+   *  @brief Gets the expanded state of the layer properties node
+   */
+  bool expanded () const
+  {
+    refresh ();
+    return m_expanded;
+  }
+
   /**
    *  @brief Child layers: begin iterator
    */
@@ -1207,14 +1226,21 @@ public:
   virtual void realize_source () const;
   virtual void realize_visual () const;
 
+  void set_expanded_silent (bool ex)
+  {
+    m_expanded = ex;
+  }
+
 protected: 
   virtual void need_realize (unsigned int flags, bool force);
+  virtual void expanded_state_changed ();
   void set_parent (const LayerPropertiesNode *);
 
 private:
   //  A reference to the view 
   tl::weak_ptr<lay::LayoutViewBase> mp_view;
   unsigned int m_list_index;
+  bool m_expanded;
   //  the parent node
   tl::weak_ptr<LayerPropertiesNode> mp_parent;
   //  the list of children
@@ -2009,8 +2035,9 @@ private:
   tl::weak_ptr<LayerPropertiesNode> mp_node;
   size_t m_synched_gen_id;
 
-  void need_realize (unsigned int flags, bool force);
-  void refresh () const;
+  virtual void need_realize (unsigned int flags, bool force);
+  virtual void expanded_state_changed ();
+  virtual void refresh () const;
 };
 
 }
