@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -226,6 +226,12 @@ public:
 protected:
   void get_bitmaps (const Viewport &vp, ViewObjectCanvas &canvas, lay::CanvasPlane *&fill, lay::CanvasPlane *&frame, lay::CanvasPlane *&vertex, lay::CanvasPlane *&text);
 
+  lay::LayoutViewBase *view ()
+  {
+    return mp_view;
+  }
+
+private:
   tl::Color m_color;
   tl::Color m_frame_color;
   char m_line_width, m_vertex_size, m_halo;
@@ -390,6 +396,14 @@ public:
    */
   void set (const db::Shape &shape, const db::ICplxTrans &t1, const std::vector<db::DCplxTrans> &trans);
 
+  /**
+   *  @brief Gets the shape
+   */
+  const db::Shape &shape () const
+  {
+    return m_shape;
+  }
+
 private:
   virtual void render (const Viewport &vp, ViewObjectCanvas &canvas);
 
@@ -425,6 +439,14 @@ public:
    */
   ~InstanceMarker ();
 
+  /**
+   *  @brief Gets the instance
+   */
+  const db::Instance &instance () const
+  {
+    return m_inst;
+  }
+
   /**
    *  @brief Set the instance the marker is to display
    *
