@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -327,6 +327,10 @@ Plugin::~Plugin ()
   if (mp_parent) {
     mp_parent->unregister_plugin (this);
   }
+  //  remove us from the children's parent
+  for (auto c = m_children.begin (); c != m_children.end (); ++c) {
+    c->mp_parent = 0;
+  }
 }
 
 void 
