@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -26,6 +26,7 @@
 #include "gsiDeclBasic.h"
 #include "layBrowserDialog.h"
 #include "layBrowserPanel.h"
+#include "layFileDialog.h"
 
 #include <QMessageBox>
 #include <QInputDialog>
@@ -343,7 +344,7 @@ Class<BrowserDialog_Stub> decl_BrowserDialog (QT_EXTERNAL_BASE (QDialog) "lay",
   "The browser dialog displays HTML code in a browser panel. The HTML code is delivered through a separate "
   "object of class \\BrowserSource which acts as a \"server\" for a specific kind of URL scheme. Whenever the "
   "browser sees a URL starting with \"int:\" it will ask the connected BrowserSource object for the HTML code "
-  "of that page using it's 'get' method. The task of the BrowserSource object is to format the data requested "
+  "of that page using its 'get' method. The task of the BrowserSource object is to format the data requested "
   "in HTML and deliver it.\n"
   "\n"
   "One use case for that class is the implementation of rich data browsers for structured information. In a "
@@ -698,7 +699,7 @@ Class<InputDialog> decl_InputDialog ("lay", "InputDialog",
     "@param title The title to display for the dialog\n"
     "@param label The label text to display for the dialog\n"
     "@param value The initial value for the input field\n"
-    "@return A \\StringValue object with has_value? set to true, if \"Ok\" was pressed and the value given in it's value attribute\n"
+    "@return A \\StringValue object with has_value? set to true, if \"Ok\" was pressed and the value given in its value attribute\n"
     "Starting from 0.22, this method is deprecated and it is recommended to use the ask_... equivalent."
   ) +
   gsi::method ("#get_item", &get_item, gsi::arg ("title"), gsi::arg ("label"), gsi::arg ("items"), gsi::arg ("value"),
@@ -707,7 +708,7 @@ Class<InputDialog> decl_InputDialog ("lay", "InputDialog",
     "@param label The label text to display for the dialog\n"
     "@param items The list of items to show in the selection element\n"
     "@param selection The initial selection (index of the element selected initially)\n"
-    "@return A \\StringValue object with has_value? set to true, if \"Ok\" was pressed and the value given in it's value attribute\n"
+    "@return A \\StringValue object with has_value? set to true, if \"Ok\" was pressed and the value given in its value attribute\n"
     "Starting from 0.22, this method is deprecated and it is recommended to use the ask_... equivalent."
   ) +
   gsi::method ("#get_string_password", &get_string_password, gsi::arg ("title"), gsi::arg ("label"), gsi::arg ("value"),
@@ -715,7 +716,7 @@ Class<InputDialog> decl_InputDialog ("lay", "InputDialog",
     "@param title The title to display for the dialog\n"
     "@param label The label text to display for the dialog\n"
     "@param value The initial value for the input field\n"
-    "@return A \\StringValue object with has_value? set to true, if \"Ok\" was pressed and the value given in it's value attribute\n"
+    "@return A \\StringValue object with has_value? set to true, if \"Ok\" was pressed and the value given in its value attribute\n"
     "Starting from 0.22, this method is deprecated and it is recommended to use the ask_... equivalent."
   ) +
   gsi::method ("#get_double", &get_double, gsi::arg ("title"), gsi::arg ("label"), gsi::arg ("value"), gsi::arg ("digits"),
@@ -724,7 +725,7 @@ Class<InputDialog> decl_InputDialog ("lay", "InputDialog",
     "@param label The label text to display for the dialog\n"
     "@param value The initial value for the input field\n"
     "@param digits The number of digits allowed\n"
-    "@return A \\DoubleValue object with has_value? set to true, if \"Ok\" was pressed and the value given in it's value attribute\n"
+    "@return A \\DoubleValue object with has_value? set to true, if \"Ok\" was pressed and the value given in its value attribute\n"
     "Starting from 0.22, this method is deprecated and it is recommended to use the ask_... equivalent."
   ) +
   gsi::method ("#get_double_ex", &get_double_ex, gsi::arg ("title"), gsi::arg ("label"), gsi::arg ("value"), gsi::arg ("min"), gsi::arg ("max"), gsi::arg ("digits"),
@@ -735,7 +736,7 @@ Class<InputDialog> decl_InputDialog ("lay", "InputDialog",
     "@param min The minimum value allowed\n"
     "@param max The maximum value allowed\n"
     "@param digits The number of digits allowed\n"
-    "@return A \\DoubleValue object with has_value? set to true, if \"Ok\" was pressed and the value given in it's value attribute\n"
+    "@return A \\DoubleValue object with has_value? set to true, if \"Ok\" was pressed and the value given in its value attribute\n"
     "Starting from 0.22, this method is deprecated and it is recommended to use the ask_... equivalent."
   ) +
   gsi::method ("#get_int", &get_int, gsi::arg ("title"), gsi::arg ("label"), gsi::arg ("value"),
@@ -743,7 +744,7 @@ Class<InputDialog> decl_InputDialog ("lay", "InputDialog",
     "@param title The title to display for the dialog\n"
     "@param label The label text to display for the dialog\n"
     "@param value The initial value for the input field\n"
-    "@return A \\IntValue object with has_value? set to true, if \"Ok\" was pressed and the value given in it's value attribute\n"
+    "@return A \\IntValue object with has_value? set to true, if \"Ok\" was pressed and the value given in its value attribute\n"
     "Starting from 0.22, this method is deprecated and it is recommended to use the ask_... equivalent."
   ) +
   gsi::method ("#get_int_ex", &get_int_ex, gsi::arg ("title"), gsi::arg ("label"), gsi::arg ("value"), gsi::arg ("min"), gsi::arg ("max"), gsi::arg ("step"),
@@ -754,7 +755,7 @@ Class<InputDialog> decl_InputDialog ("lay", "InputDialog",
     "@param min The minimum value allowed\n"
     "@param max The maximum value allowed\n"
     "@param step The step size for the spin buttons\n"
-    "@return A \\IntValue object with has_value? set to true, if \"Ok\" was pressed and the value given in it's value attribute\n"
+    "@return A \\IntValue object with has_value? set to true, if \"Ok\" was pressed and the value given in its value attribute\n"
     "Starting from 0.22, this method is deprecated and it is recommended to use the ask_... equivalent."
   ) +
   gsi::method ("ask_string", &ask_string, gsi::arg ("title"), gsi::arg ("label"), gsi::arg ("value"),
@@ -926,11 +927,30 @@ static tl::Variant ask_open_file_name (const std::string &title, const std::stri
 
 static tl::Variant ask_save_file_name (const std::string &title, const std::string &dir, const std::string &filter)
 {
-  QString f = QFileDialog::getSaveFileName (QApplication::activeWindow (), tl::to_qstring (title), tl::to_qstring (dir), tl::to_qstring (filter));
+  QString selected_filter;
+
+  QString f = QFileDialog::getSaveFileName (QApplication::activeWindow (), tl::to_qstring (title), tl::to_qstring (dir), tl::to_qstring (filter), &selected_filter);
   if (f.isEmpty ()) {
     return tl::Variant ();
   } else {
-    return tl::Variant (tl::to_string (f));
+    return tl::Variant (lay::FileDialog::add_default_extension (tl::to_string (f), selected_filter));
+  }
+}
+
+static tl::Variant ask_save_file_name2 (const std::string &title, const std::string &dir, const std::string &filter)
+{
+  QString selected_filter;
+  QString qfilter = tl::to_qstring (filter);
+
+  QString f = QFileDialog::getSaveFileName (QApplication::activeWindow (), tl::to_qstring (title), tl::to_qstring (dir), qfilter, &selected_filter);
+  if (f.isEmpty ()) {
+    return tl::Variant ();
+  } else {
+    tl::Variant v;
+    v.set_list ();
+    v.push (lay::FileDialog::add_default_extension (tl::to_string (f), selected_filter));
+    v.push (lay::FileDialog::find_selected_filter (qfilter, selected_filter));
+    return v;
   }
 }
 
@@ -1012,6 +1032,16 @@ Class<FileDialog> decl_FileDialog ("lay", "FileDialog",
     "@return The path of the file chosen or \"nil\" if \"Cancel\" was pressed\n"
     "\n"
     "This method has been introduced in version 0.23. It is somewhat easier to use than the get_... equivalent.\n"
+  ) +
+  gsi::method ("ask_save_file_name_with_filter", &ask_save_file_name2, gsi::arg ("title"), gsi::arg ("dir"), gsi::arg ("filter"),
+    "@brief Select one file for writing\n"
+    "\n"
+    "@param title The title of the dialog\n"
+    "@param dir The directory selected initially\n"
+    "@param filter The filters available, for example \"Images (*.png *.xpm *.jpg);;Text files (*.txt);;XML files (*.xml)\"\n"
+    "@return \"nil\" if \"Cancel\" was pressed, otherwise a pair: The path of the file chosen and the index selected file type (-1 if no specific type was selected)\n"
+    "\n"
+    "This method has been introduced in version 0.28.11.\n"
   ),
   "@brief Various methods to request a file name\n"
   "\n"
