@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -607,6 +607,11 @@ public:
     return m_ids.end ();
   }
 
+  /**
+   *  @brief Gets a value indicating whether the given name is present already
+   */
+  bool has_attribute (const QString &name) const;
+
   /**
    *  @brief Get the attribute ID for a given name
    *
@@ -716,7 +721,14 @@ class LAYUI_PUBLIC GenericSyntaxHighlighter
   : public QSyntaxHighlighter
 {
 public: 
-  GenericSyntaxHighlighter (QObject *parent, QIODevice &input, GenericSyntaxHighlighterAttributes *attributes);
+  /**
+   *  @brief Creates a GenericSyntaxHighlighter
+   *  @param parent The owner of the highlighter
+   *  @param input The stream from which to pull
+   *  @param attributes The attributes
+   *  @param initialize_attributes If true, the attributes are initialized from the itemData lines
+   */
+  GenericSyntaxHighlighter (QObject *parent, QIODevice &input, GenericSyntaxHighlighterAttributes *attributes, bool initialize_attributes);
   
   /**
    *  @brief Implementation of the highlighter
