@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -372,6 +372,10 @@ LayoutViewConfigPage2c::setup (lay::Dispatcher *root)
   root->config_get (cfg_sel_inside_pcells_mode, ipm);
   mp_ui->sel_inside_pcells_cb->setChecked (ipm);
 
+  bool tpm = 0;
+  root->config_get (cfg_text_point_mode, tpm);
+  mp_ui->text_point_mode_cb->setChecked (tpm);
+
   unsigned int sr = 0;
   root->config_get (cfg_search_range, sr);
   mp_ui->search_range_spinbx->setValue (sr);
@@ -392,6 +396,7 @@ LayoutViewConfigPage2c::commit (lay::Dispatcher *root)
   root->config_set (cfg_sel_halo, mp_ui->halo_cb->isChecked ());
   root->config_set (cfg_sel_transient_mode, mp_ui->transient_mode_cb->isChecked ());
   root->config_set (cfg_sel_inside_pcells_mode, mp_ui->sel_inside_pcells_cb->isChecked ());
+  root->config_set (cfg_text_point_mode, mp_ui->text_point_mode_cb->isChecked ());
   root->config_set (cfg_search_range, (unsigned int) mp_ui->search_range_spinbx->value ());
   root->config_set (cfg_search_range_box, (unsigned int) mp_ui->search_range_box_spinbx->value ());
 }
@@ -419,9 +424,21 @@ LayoutViewConfigPage2d::setup (lay::Dispatcher *root)
   root->config_get (cfg_tracking_cursor_color, color, lay::ColorConverter ());
   mp_ui->color_pb->set_color (color);
 
-  bool enabled = 0;
+  bool enabled = false;
   root->config_get (cfg_tracking_cursor_enabled, enabled);
   mp_ui->tracking_cb->setChecked (enabled);
+
+  color = QColor ();
+  root->config_get (cfg_crosshair_cursor_color, color, lay::ColorConverter ());
+  mp_ui->color_chc->set_color (color);
+
+  int line_style = 0;
+  root->config_get (cfg_crosshair_cursor_line_style, line_style);
+  mp_ui->line_style_chc->set_line_style (line_style);
+
+  enabled = false;
+  root->config_get (cfg_crosshair_cursor_enabled, enabled);
+  mp_ui->crosshair_cursor_cb->setChecked (enabled);
 }
 
 void
@@ -430,6 +447,9 @@ LayoutViewConfigPage2d::commit (lay::Dispatcher *root)
   lay::ColorConverter cc;
   root->config_set (cfg_tracking_cursor_color, mp_ui->color_pb->get_color (), cc);
   root->config_set (cfg_tracking_cursor_enabled, mp_ui->tracking_cb->isChecked ());
+  root->config_set (cfg_crosshair_cursor_color, mp_ui->color_chc->get_color (), cc);
+  root->config_set (cfg_crosshair_cursor_line_style, mp_ui->line_style_chc->line_style ());
+  root->config_set (cfg_crosshair_cursor_enabled, mp_ui->crosshair_cursor_cb->isChecked ());
 }
 
 // ------------------------------------------------------------
