@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -367,6 +367,8 @@ LayoutViewFunctions::menu_activated (const std::string &symbol)
     }
   } else if (symbol == "cm_lv_select_all") {
     cm_select_all ();
+  } else if (symbol == "cm_lv_invert_selection") {
+    cm_invert_selection ();
   } else if (symbol == "cm_lv_new_tab") {
     cm_new_tab ();
   } else if (symbol == "cm_lv_rename_tab") {
@@ -387,6 +389,8 @@ LayoutViewFunctions::menu_activated (const std::string &symbol)
     cm_show_all ();
   } else if (symbol == "cm_lv_show") {
     cm_show ();
+  } else if (symbol == "cm_lv_toggle_visibility") {
+    cm_toggle_visibility ();
   } else if (symbol == "cm_lv_rename") {
     cm_rename ();
   } else if (symbol == "cm_lv_delete") {
@@ -440,7 +444,7 @@ LayoutViewFunctions::cm_cell_user_properties ()
     db::properties_id_type prop_id = cell.prop_id ();
 
     lay::UserPropertiesForm props_form (parent_widget ());
-    if (props_form.show (view (), cv_index, prop_id)) {
+    if (props_form.show (view (), cv_index, prop_id, layout.begin_meta (cell.cell_index ()), layout.end_meta (cell.cell_index ()))) {
 
       view ()->transaction (tl::to_string (tr ("Edit cell's user properties")));
       cell.prop_id (prop_id);
@@ -735,7 +739,7 @@ void
 LayoutViewFunctions::cm_cell_cut ()
 {
   if (view ()->hierarchy_panel ()) {
-    //  TODO: currently the hierarchy panel's cut function does it's own transaction handling.
+    //  TODO: currently the hierarchy panel's cut function does its own transaction handling.
     //  Otherwise the cut function is not working propertly.
     view ()->hierarchy_panel ()->cut ();
   }
@@ -946,6 +950,14 @@ LayoutViewFunctions::cm_select_all ()
   }
 }
 
+void
+LayoutViewFunctions::cm_invert_selection ()
+{
+  if (view ()->control_panel ()) {
+    view ()->control_panel ()->cm_invert_selection ();
+  }
+}
+
 void
 LayoutViewFunctions::cm_new_tab ()
 {
@@ -1026,6 +1038,14 @@ LayoutViewFunctions::cm_show ()
   }
 }
 
+void
+LayoutViewFunctions::cm_toggle_visibility ()
+{
+  if (view ()->control_panel ()) {
+    view ()->control_panel ()->cm_toggle_visibility ();
+  }
+}
+
 void
 LayoutViewFunctions::cm_rename ()
 {
@@ -1177,11 +1197,12 @@ LayoutViewFunctions::do_cm_duplicate (bool interactive)
   db::Clipboard::instance ().swap (saved_clipboard);
 
   try {
-    view ()->copy ();
+    bool transient_mode = ! view ()->has_selection ();
+    view ()->copy_view_objects ();
     view ()->clear_selection ();
     view ()->cancel ();
     if (interactive) {
-      view ()->paste_interactive ();
+      view ()->paste_interactive (transient_mode);
     } else {
       view ()->paste ();
     }
@@ -1873,12 +1894,12 @@ LayoutViewFunctions::cm_edit_layer ()
 {
   lay::LayerPropertiesConstIterator sel = view ()->current_layer ();
   if (sel.is_null ()) {
-    throw tl::Exception (tl::to_string (tr ("No layer selected for editing it's properties")));
+    throw tl::Exception (tl::to_string (tr ("No layer selected for editing its properties")));
   }
 
   int index = sel->cellview_index ();
   if (sel->has_children () || index < 0 || int (view ()->cellviews ()) <= index || sel->layer_index () < 0) {
-    throw tl::Exception (tl::to_string (tr ("No valid layer selected for editing it's properties")));
+    throw tl::Exception (tl::to_string (tr ("No valid layer selected for editing its properties")));
   }
 
   const lay::CellView &cv = view ()->cellview (index);
