@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -46,6 +46,7 @@
 #include "tlInternational.h"
 
 #include "laySelectStippleForm.h"
+#include "laySelectLineStyleForm.h"
 
 #include <vector>
 
@@ -246,6 +247,199 @@ DitherPatternSelectionButton::update_menu ()
   } catch (...) { }
 }
 
+// -------------------------------------------------------------
+//  LineStyleSelectionButton implementation
+
+LineStyleSelectionButton::LineStyleSelectionButton (QWidget *parent)
+  : QPushButton (parent), mp_view (0), m_line_style (-1)
+{
+  setMenu (new QMenu (this));
+  update_pattern ();
+  connect (menu (), SIGNAL (aboutToShow ()), this, SLOT (menu_about_to_show ()));
+}
+
+LineStyleSelectionButton::~LineStyleSelectionButton ()
+{
+  // .. nothing yet ..
+}
+
+void
+LineStyleSelectionButton::set_view (lay::LayoutViewBase *view)
+{
+  if (view != mp_view) {
+    mp_view = view;
+    update_menu ();
+  }
+}
+
+void
+LineStyleSelectionButton::set_line_style (int ls)
+{
+  if (ls != m_line_style) {
+    m_line_style = ls;
+    update_pattern ();
+  }
+}
+
+int
+LineStyleSelectionButton::line_style () const
+{
+  return m_line_style;
+}
+
+void
+LineStyleSelectionButton::menu_selected ()
+{
+  QAction *action = dynamic_cast <QAction *> (sender ());
+  if (action) {
+
+    m_line_style = action->data ().toInt ();
+    update_pattern ();
+    emit (line_style_changed (m_line_style));
+
+  }
+}
+
+void
+LineStyleSelectionButton::browse_selected ()
+{
+  if (mp_view) {
+
+    SelectLineStyleForm styles_form (0, mp_view->line_styles (), true);
+    styles_form.set_selected (m_line_style);
+
+    if (styles_form.exec ()) {
+
+      m_line_style = styles_form.selected ();
+      update_pattern ();
+      emit (line_style_changed (m_line_style));
+
+    }
+
+  } else {
+
+    //  Use the default (non-custom) pattern if no view is set.
+    lay::LineStyles default_pattern;
+
+    SelectLineStyleForm styles_form (0, default_pattern, true);
+    styles_form.set_selected (m_line_style);
+
+    if (styles_form.exec ()) {
+
+      m_line_style = styles_form.selected ();
+      update_pattern ();
+      emit (line_style_changed (m_line_style));
+
+    }
+
+  }
+}
+
+void
+LineStyleSelectionButton::update_pattern ()
+{
+  QPushButton::setText (QString::fromUtf8 (" "));
+
+  QString text = QString::fromUtf8 ("XXXXXXX");
+  QFontMetrics fm (font (), this);
+  QRect rt (fm.boundingRect (text)); // dummy text to be compliant with the other color button
+
+  QPushButton::setIconSize (QSize (rt.width (), rt.height ()));
+
+#if QT_VERSION >= 0x050000
+  double dpr = devicePixelRatio ();
+#else
+  double dpr = 1.0;
+#endif
+
+  if (m_line_style < 0) {
+
+    QPixmap pixmap (rt.width () * dpr, rt.height () * dpr);
+#if QT_VERSION >= 0x050000
+    pixmap.setDevicePixelRatio (dpr);
+#endif
+    pixmap.fill (QColor (0, 0, 0, 0));
+
+    QPainter pxpainter (&pixmap);
+    pxpainter.setFont (font ());
+    QColor text_color = palette ().color (QPalette::Active, QPalette::Text);
+    pxpainter.setPen (QPen (text_color));
+
+    QRectF r (0, 0, rt.width () - pxpainter.pen ().widthF (), rt.height () - pxpainter.pen ().widthF ());
+    pxpainter.drawText (r, Qt::AlignHCenter | Qt::AlignVCenter | Qt::TextSingleLine, QObject::tr ("None"));
+
+    QPushButton::setIcon (QIcon (pixmap));
+
+  } else {
+
+    const lay::LineStyleInfo *dp_info;
+    if (mp_view) {
+      dp_info = & mp_view->line_styles ().style ((unsigned int) m_line_style);
+    } else {
+      static lay::LineStyles default_pattern;
+      dp_info = & default_pattern.style ((unsigned int) m_line_style);
+    }
+
+    QPushButton::setIcon (dp_info->get_bitmap (rt.width () * dpr, rt.height () * dpr, dpr));
+
+  }
+}
+
+void
+LineStyleSelectionButton::menu_about_to_show ()
+{
+  update_menu ();
+}
+
+void
+LineStyleSelectionButton::update_menu ()
+{
+  menu ()->clear ();
+  menu ()->addAction (QObject::tr ("None"), this, SLOT (menu_selected ()))->setData (-1);
+  menu ()->addAction (QObject::tr ("Choose ..."), this, SLOT (browse_selected ()));
+  menu ()->addSeparator ();
+
+  //  from_string might throw an exception ...
+  try {
+
+    lay::LineStyles patterns;
+
+    std::string s;
+    if (lay::Dispatcher::instance ()) {
+      lay::Dispatcher::instance ()->config_get (cfg_line_style_palette, s);
+    }
+    lay::LineStylePalette palette = lay::LineStylePalette::default_palette ();
+    if (! s.empty ()) {
+      palette.from_string (s);
+    }
+
+    //  fill the list of stipple palette items
+    for (unsigned int i = 0; i < palette.styles (); ++i) {
+
+      unsigned int n = palette.style_by_index (i);
+      if (int (n) < std::distance (patterns.begin (), patterns.end ())) {
+
+#if QT_VERSION > 0x050000
+        double dpr = devicePixelRatio ();
+#else
+        double dpr = 1.0;
+#endif
+
+        lay::LineStyleInfo info = patterns.begin () [n];
+
+        std::string name (info.name ());
+        if (name.empty ()) {
+          name = tl::sprintf ("#%d", n);
+        }
+
+        menu ()->addAction (QIcon (info.get_bitmap (16, 8)), tl::to_qstring (name), this, SLOT (menu_selected ()))->setData (n);
+
+      }
+    }
+
+  } catch (...) { }
+}
+
 // -------------------------------------------------------------
 //  CellViewSelectionComboBox implementation
 
@@ -403,7 +597,9 @@ BEGIN_PROTECTED
         }
       }
 
-      mp_private->view->manager ()->transaction (tl::to_string (QObject::tr ("New layer"))); 
+      if (mp_private->view->manager ()) {
+        mp_private->view->manager ()->transaction (tl::to_string (QObject::tr ("New layer")));
+      }
 
       unsigned int l = cv->layout ().insert_layer (lp);
       std::vector <unsigned int> nl;
@@ -411,7 +607,9 @@ BEGIN_PROTECTED
       mp_private->view->add_new_layers (nl, mp_private->cv_index);
       mp_private->view->update_content ();
 
-      mp_private->view->manager ()->commit ();
+      if (mp_private->view->manager ()) {
+        mp_private->view->manager ()->commit ();
+      }
 
       //  NOTE: add_new_layers has triggered update_layer_list which already added the new layer
       set_current_layer (lp);
@@ -1073,6 +1271,85 @@ ColorButton::browse_selected ()
   }
 }
 
+// -------------------------------------------------------------
+//  MarginWidget implementation
+
+MarginWidget::MarginWidget (QWidget *parent, const char *name)
+  : QFrame (parent), m_margin ()
+{
+  if (name) {
+    setObjectName (name);
+  }
+
+  setFrameStyle (QFrame::NoFrame);
+
+  QHBoxLayout *layout = new QHBoxLayout (this);
+  layout->setContentsMargins (0, 0, 0, 0);
+
+  QLineEdit *le = new QLineEdit (this);
+  mp_abs_edit = le;
+  le->setSizePolicy (QSizePolicy (QSizePolicy::Fixed, QSizePolicy::Preferred));
+  layout->addWidget (le);
+
+  le = new QLineEdit (this);
+  mp_rel_edit = le;
+  le->setSizePolicy (QSizePolicy (QSizePolicy::Fixed, QSizePolicy::Preferred));
+  layout->addWidget (le);
+
+  QComboBox *mode = new QComboBox (this);
+  mode->addItem (tl::to_qstring ("µm"));
+  mode->addItem (tl::to_qstring ("%"));
+  mp_mode_cb = mode;
+  layout->addWidget (mode);
+
+  connect (mode, SIGNAL (currentIndexChanged (int)), this, SLOT (mode_selection_changed ()));
+
+  set_margin (lay::Margin ());
+}
+
+lay::Margin
+MarginWidget::get_margin () const
+{
+  bool rel_mode = mp_mode_cb->currentIndex () == 1;
+  double rel = 0.0, abs = 0.0;
+  tl::from_string (tl::to_string (mp_rel_edit->text ()), rel);
+  tl::from_string (tl::to_string (mp_abs_edit->text ()), abs);
+
+  lay::Margin m = m_margin;
+  m.set_relative_mode (rel_mode);
+  if (rel_mode) {
+    m.set_relative_value (rel * 0.01);
+  } else {
+    m.set_absolute_value (abs);
+  }
+  return m;
+}
+
+void
+MarginWidget::set_margin (const lay::Margin &margin)
+{
+  m_margin = margin;
+
+  mp_abs_edit->setText (tl::to_qstring (tl::to_string (margin.absolute_value ())));
+  mp_rel_edit->setText (tl::to_qstring (tl::to_string (margin.relative_value () * 100.0)));
+  mp_mode_cb->setCurrentIndex (margin.relative_mode () ? 1 : 0);
+  mode_selection_changed ();
+}
+
+void
+MarginWidget::mode_selection_changed ()
+{
+  bool rel_mode = mp_mode_cb->currentIndex () == 1;
+  //  NOTE: first hiding and then showing avoids layout flicker ..
+  mp_rel_edit->hide ();
+  mp_abs_edit->hide ();
+  if (rel_mode) {
+    mp_rel_edit->show ();
+  } else {
+    mp_abs_edit->show ();
+  }
+}
+
 // -------------------------------------------------------------
 //  DecoratedLineEdit implementation
 
@@ -1216,6 +1493,8 @@ void DecoratedLineEdit::mouseReleaseEvent (QMouseEvent *event)
     }
 
   }
+
+  QLineEdit::mouseReleaseEvent (event);
 }
 
 void DecoratedLineEdit::mousePressEvent (QMouseEvent *event)
@@ -1234,9 +1513,11 @@ void DecoratedLineEdit::mousePressEvent (QMouseEvent *event)
     }
 
   }
+
+  QLineEdit::mousePressEvent (event);
 }
 
-void DecoratedLineEdit::resizeEvent (QResizeEvent * /*event*/)
+void DecoratedLineEdit::resizeEvent (QResizeEvent *event)
 {
   int fw = hasFrame () ? le_frame_width : 0;
 
@@ -1251,6 +1532,8 @@ void DecoratedLineEdit::resizeEvent (QResizeEvent * /*event*/)
     QRect r = geometry ();
     mp_options_label->setGeometry (fw, 0, label_size.width (), r.height ());
   }
+
+  QLineEdit::resizeEvent (event);
 }
 
 // -------------------------------------------------------------
