@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -159,6 +159,7 @@ TEST(11_device_scaling)
 TEST(12_simple_dmos)
 {
   run_test (_this, "ringo_simple_dmos", "ringo.gds");
+  run_test (_this, "ringo_simple_dmos_fixed", "ringo_fixed_sources.gds");
 }
 
 TEST(13_simple_ringo_device_subcircuits)
@@ -274,3 +275,15 @@ TEST(29_DeviceCombineAndTolerances)
   run_test (_this, "res_combine2", "res_combine.gds");
   run_test (_this, "res_combine3", "res_combine.gds");
 }
+
+TEST(30_MustConnect1)
+{
+  run_test (_this, "must_connect1", "must_connect1.gds");
+  run_test (_this, "must_connect1_tl", "must_connect1.gds");
+}
+
+TEST(31_MustConnect2)
+{
+  run_test (_this, "must_connect2", "must_connect2.gds");
+}
+
