@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -51,11 +51,35 @@ MacroInterpreter::can_run (const lym::Macro *macro)
   return false;
 }
 
+namespace
+{
+
+class MacroIncludeFileResolver
+  : public tl::IncludeFileResolver
+{
+public:
+  MacroIncludeFileResolver () { }
+
+  std::string get_text (const std::string &path) const
+  {
+    //  Use lym::Macro to resolve texts - this strips the XML envelope.
+    //  Intentionally not compatibility check is made to allow using any
+    //  type of input and specifically any extension.
+    lym::Macro macro;
+    macro.load_from (path);
+    return macro.text ();
+  }
+};
+
+}
+
 std::pair<std::string, std::string>
 MacroInterpreter::include_expansion (const lym::Macro *macro)
 {
+  MacroIncludeFileResolver include_file_resolver;
+
   std::pair<std::string, std::string> res;
-  res.first = tl::IncludeExpander::expand (macro->path (), macro->text (), res.second).to_string ();
+  res.first = tl::IncludeExpander::expand (macro->path (), macro->text (), res.second, &include_file_resolver).to_string ();
 
   if (res.first != macro->path ()) {
 
@@ -106,31 +130,15 @@ MacroInterpreter::execute_macro (const lym::Macro *macro)
 
     if (cls.current_name () == macro->dsl_interpreter ()) {
 
-      std::pair<std::string, std::string> et = cls->include_expansion (macro);
-      if (et.first.empty () || et.first == macro->path ()) {
-
-        std::unique_ptr<tl::Executable> eo (cls->executable (macro));
-        if (eo.get ()) {
-          eo->do_execute ();
-        }
-
-      } else {
-
-        //  provide a copy which takes the include-expanded version
-        lym::Macro tmp_macro;
-        tmp_macro.assign (*macro);
-        tmp_macro.set_text (et.second);
-        tmp_macro.set_file_path (et.first);
-        std::unique_ptr<tl::Executable> eo (cls->executable (&tmp_macro));
-        if (eo.get ()) {
-          eo->do_execute ();
-        }
-
+      std::unique_ptr<tl::Executable> eo (cls->executable (macro));
+      if (eo.get ()) {
+        eo->do_execute ();
       }
 
       return;
 
     }
+
   }
 
   throw tl::Exception (tl::to_string (tr ("No interpreter registered for DSL type '")) + macro->dsl_interpreter () + "'");
