@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "dbLEFDEFImporter.h"
 #include "dbLayoutUtils.h"
 #include "dbTechnology.h"
+#include "dbShapeProcessor.h"
 
 #include "tlStream.h"
 #include "tlProgress.h"
@@ -37,14 +38,29 @@ namespace db
 // -----------------------------------------------------------------------------------
 //  Path resolution utility
 
-std::string correct_path (const std::string &fn, const db::Layout &layout, const std::string &base_path)
+std::string correct_path (const std::string &fn_in, const db::Layout &layout, const std::string &base_path)
 {
+  const db::Technology *tech = layout.technology ();
+
+  //  Allow LEF reference through expressions, i.e.
+  //    $(base_path) - path of the main file
+  //    $(tech_dir)  - the location of the .lyt file if a technology is specified
+  //    $(tech_name) - the name of the technology if one is specified
+  //  In addition expressions are interpolated, e.g. "$(env('HOME'))".
+
+  tl::Eval expr;
+  expr.set_var ("base_path", base_path);
+  if (tech) {
+    expr.set_var ("tech_dir", tech->base_path ());
+    expr.set_var ("tech_name", tech->name ());
+  }
+
+  std::string fn = expr.interpolate (fn_in);
+
   if (! tl::is_absolute (fn)) {
 
     //  if a technology is given and the file can be found in the technology's base path, take it
     //  from there.
-    const db::Technology *tech = layout.technology ();
-
     if (tech && ! tech->base_path ().empty ()) {
       std::string new_fn = tl::combine_path (tech->base_path (), fn);
       if (tl::file_exists (new_fn)) {
@@ -212,12 +228,12 @@ RuleBasedViaGenerator::create_cell (LEFDEFReaderState &reader, Layout &layout, d
 
   std::set <unsigned int> dl;
 
-  dl = reader.open_layer (layout, m_bottom_layer, ViaGeometry, mask_bottom, via_box.enlarged (m_be));
+  dl = reader.open_layer (layout, m_bottom_layer, ViaGeometry, mask_bottom);
   for (std::set<unsigned int>::const_iterator l = dl.begin (); l != dl.end (); ++l) {
     cell.shapes (*l).insert (db::Polygon (via_box.enlarged (m_be).moved (m_bo)));
   }
 
-  dl = reader.open_layer (layout, m_top_layer, ViaGeometry, mask_top, via_box.enlarged (m_te));
+  dl = reader.open_layer (layout, m_top_layer, ViaGeometry, mask_top);
   for (std::set<unsigned int>::const_iterator l = dl.begin (); l != dl.end (); ++l) {
     cell.shapes (*l).insert (db::Polygon (via_box.enlarged (m_te).moved (m_to)));
   }
@@ -465,6 +481,55 @@ GeometryBasedLayoutGenerator::add_via (const std::string &vn, const db::Trans &t
   m_vias.back ().top_mask = top_mask;
 }
 
+void
+GeometryBasedLayoutGenerator::subtract_overlap_from_outline (const std::set<std::string> &overlap_layers)
+{
+  db::Shapes all_overlaps;
+
+  std::vector<std::map <std::pair<std::string, LayerDetailsKey>, db::Shapes>::iterator> to_remove;
+  for (auto s = m_shapes.begin (); s != m_shapes.end (); ++s) {
+    if (overlap_layers.find (s->first.first) != overlap_layers.end ()) {
+      all_overlaps.insert (s->second);
+      to_remove.push_back (s);
+    }
+  }
+
+  for (auto i = to_remove.begin (); i != to_remove.end (); ++i) {
+    m_shapes.erase (*i);
+  }
+
+  if (all_overlaps.empty ()) {
+    return;
+  }
+
+  for (auto s = m_shapes.begin (); s != m_shapes.end (); ++s) {
+
+    if (s->first.second.purpose != Outline) {
+      continue;
+    }
+
+    db::ShapeProcessor proc;
+
+    size_t pn = 0;
+    for (auto sh = s->second.begin (db::ShapeIterator::All); ! sh.at_end (); ++sh) {
+      proc.insert (*sh, pn);
+      pn += 2;
+    }
+
+    pn = 1;
+    for (auto sh = all_overlaps.begin (db::ShapeIterator::All); ! sh.at_end (); ++sh) {
+      proc.insert (*sh, pn);
+      pn += 2;
+    }
+
+    db::BooleanOp op (db::BooleanOp::And);
+    db::ShapeGenerator sg (s->second, true /*clear shapes*/);
+    db::PolygonGenerator out (sg, true, true);
+    proc.process (out, op);
+
+  }
+}
+
 // -----------------------------------------------------------------------------------
 //  LEFDEFTechnologyComponent implementation
 
@@ -515,6 +580,7 @@ LEFDEFReaderOptions::LEFDEFReaderOptions ()
     m_special_routing_suffix (""),
     m_special_routing_datatype (0),
     m_separate_groups (false),
+    m_joined_paths (false),
     m_map_file (),
     m_macro_resolution_mode (0),
     m_read_lef_with_def (true),
@@ -592,6 +658,7 @@ LEFDEFReaderOptions &LEFDEFReaderOptions::operator= (const LEFDEFReaderOptions &
     m_special_routing_datatype = d.m_special_routing_datatype;
     m_special_routing_datatypes = d.m_special_routing_datatypes;
     m_separate_groups = d.m_separate_groups;
+    m_joined_paths = d.m_joined_paths;
     m_map_file = d.m_map_file;
     m_macro_resolution_mode = d.m_macro_resolution_mode;
     m_lef_files = d.m_lef_files;
@@ -876,16 +943,18 @@ LEFDEFReaderOptions::special_routing_datatype_str () const
 LEFDEFReaderState::LEFDEFReaderState (const LEFDEFReaderOptions *tc, db::Layout &layout, const std::string &base_path)
   : mp_importer (0), m_create_layers (true), m_has_explicit_layer_mapping (false), m_laynum (1), mp_tech_comp (tc)
 {
-  if (! tc->map_file ().empty ()) {
+  if (! tc) {
+
+    //  use default options
+
+  } else if (! tc->map_file ().empty ()) {
 
     read_map_file (tc->map_file (), layout, base_path);
 
   } else {
 
-    if (tc) {
-      m_layer_map = tc->layer_map ();
-      m_create_layers = tc->read_all_layers ();
-    }
+    m_layer_map = tc->layer_map ();
+    m_create_layers = tc->read_all_layers ();
 
   }
 }
@@ -990,7 +1059,6 @@ LEFDEFReaderState::read_map_file (const std::string &filename, db::Layout &layou
 
   //  build an explicit layer mapping now.
 
-  tl_assert (m_has_explicit_layer_mapping);
   m_layers.clear ();
   m_layer_map.clear ();
 
@@ -1013,9 +1081,12 @@ LEFDEFReaderState::read_single_map_file (const std::string &path, std::map<std::
 
   tl::log << tl::to_string (tr ("Reading LEF/DEF map file")) << " " << file_stream.absolute_path ();
 
+  //  Purpose name to purpose code
   std::map<std::string, LayerPurpose> purpose_translation;
   purpose_translation ["LEFPIN"] = LEFPins;
   purpose_translation ["PIN"] = Pins;
+  purpose_translation ["LEFPINNAME"] = LEFLabel;
+  purpose_translation ["PINNAME"] = Label;
   purpose_translation ["FILL"] = Fills;
   purpose_translation ["FILLOPC"] = FillsOPC;
   purpose_translation ["LEFOBS"] = Obstructions;
@@ -1025,6 +1096,11 @@ LEFDEFReaderState::read_single_map_file (const std::string &path, std::map<std::
   purpose_translation ["BLOCKAGE"] = Blockage;
   purpose_translation ["ALL"] = All;
 
+  //  List of purposes corresponding to ALL
+  LayerPurpose all_purposes[] = {
+    LEFPins, Pins, Fills, FillsOPC, Obstructions, SpecialRouting, Routing, ViaGeometry
+  };
+
   while (! ts.at_end ()) {
 
     const std::string &l = ts.get_line ();
@@ -1212,10 +1288,8 @@ LEFDEFReaderState::read_single_map_file (const std::string &path, std::map<std::
 
           } else if (i->second == All) {
 
-            for (std::map<std::string, LayerPurpose>::const_iterator p = purpose_translation.begin (); p != purpose_translation.end (); ++p) {
-              if (p->second != All && p->second != Blockage) {
-                translated_purposes.insert (LayerDetailsKey (p->second, mask, via_size));
-              }
+            for (LayerPurpose *p = all_purposes; p != all_purposes + sizeof (all_purposes) / sizeof (all_purposes[0]); ++p) {
+              translated_purposes.insert (LayerDetailsKey (*p, mask, via_size));
             }
 
           } else {
@@ -1298,9 +1372,12 @@ LEFDEFReaderState::open_layer (db::Layout &layout, const std::string &n, LayerPu
       if (mask > 0) {
         tl::warn << tl::to_string (tr (" Mask ")) << mask << tl::noendl;
       }
+//  not printing via size - too confusing?
+#if 0
       if (via_size != db::DVector ()) {
         tl::warn << tl::to_string (tr (" Via size ")) << via_size.to_string () << tl::noendl;
       }
+#endif
       tl::warn << tl::to_string (tr (" - layer is ignored"));
     }
 
@@ -1990,7 +2067,11 @@ LEFDEFImporter::read (tl::InputStream &stream, db::Layout &layout, LEFDEFReaderS
 void 
 LEFDEFImporter::error (const std::string &msg)
 {
-  throw LEFDEFReaderException (msg, int (mp_stream->line_number ()), m_cellname, m_fn);
+  if (m_sections.empty ()) {
+    throw LEFDEFReaderException (msg, int (mp_stream->line_number ()), m_cellname, m_fn);
+  } else {
+    throw LEFDEFReaderException (msg + tl::sprintf (tl::to_string (tr (" (inside %s)")), tl::join (m_sections, "/")), int (mp_stream->line_number ()), m_cellname, m_fn);
+  }
 }
 
 void 
@@ -2140,6 +2221,18 @@ LEFDEFImporter::get ()
   return r;
 }
 
+void
+LEFDEFImporter::enter_section (const std::string &name)
+{
+  m_sections.push_back (name);
+}
+
+void
+LEFDEFImporter::leave_section ()
+{
+  m_sections.pop_back ();
+}
+
 const std::string &
 LEFDEFImporter::next ()
 {
