@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -874,6 +874,16 @@ public:
     m_separate_groups = f;
   }
 
+  bool joined_paths () const
+  {
+    return m_joined_paths;
+  }
+
+  void set_joined_paths (bool f)
+  {
+    m_joined_paths = f;
+  }
+
   const std::string &map_file () const
   {
     return m_map_file;
@@ -1009,6 +1019,7 @@ private:
   std::map<unsigned int, std::string> m_special_routing_suffixes;
   std::map<unsigned int, int> m_special_routing_datatypes;
   bool m_separate_groups;
+  bool m_joined_paths;
   std::string m_map_file;
   unsigned int m_macro_resolution_mode;
   bool m_read_lef_with_def;
@@ -1202,6 +1213,8 @@ public:
     m_fixedmask = f;
   }
 
+  void subtract_overlap_from_outline (const std::set<std::string> &overlap_layers);
+
 private:
   struct Via {
     Via () : bottom_mask (0), cut_mask (0), top_mask (0) { }
@@ -1706,8 +1719,31 @@ private:
   db::property_names_id_type m_pin_prop_name_id;
   db::LEFDEFReaderOptions m_options;
   int m_warn_level;
+  std::vector<std::string> m_sections;
+
+  friend class LEFDEFSection;
 
   const std::string &next ();
+  void enter_section (const std::string &name);
+  void leave_section ();
+};
+
+class DB_PLUGIN_PUBLIC LEFDEFSection
+{
+public:
+  LEFDEFSection (LEFDEFImporter *importer, const std::string &name)
+    : mp_importer (importer)
+  {
+    mp_importer->enter_section (name);
+  }
+
+  ~LEFDEFSection ()
+  {
+    mp_importer->leave_section ();
+  }
+
+private:
+  LEFDEFImporter *mp_importer;
 };
 
 class DB_PLUGIN_PUBLIC LEFDEFReader
