@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -94,6 +94,14 @@ public:
     return m_cut_layers.find (layer) != m_cut_layers.end ();
   }
 
+  /**
+   *  @brief Returns true if the given layer is an overlap layer
+   */
+  bool is_overlap_layer (const std::string &layer) const
+  {
+    return m_overlap_layers.find (layer) != m_overlap_layers.end ();
+  }
+
   /**
    *  @brief Returns the number of masks for the given layer
    */
@@ -141,7 +149,7 @@ private:
   std::map<std::string, std::pair<double, double> > m_min_widths;
   std::map<std::string, MacroDesc> m_macros;
   std::map<std::string, ViaDesc> m_vias;
-  std::set<std::string> m_routing_layers, m_cut_layers;
+  std::set<std::string> m_routing_layers, m_cut_layers, m_overlap_layers;
   std::map<std::string, unsigned int> m_num_masks;
 
   std::vector <db::Trans> get_iteration (double dbu);
@@ -152,6 +160,7 @@ private:
   void read_viadef_by_geometry (GeometryBasedLayoutGenerator *lg, ViaDesc &desc, const std::string &n, double dbu);
   void read_layer (Layout &layout);
   void read_macro (Layout &layout);
+  void skip_entry ();
 };
 
 }
