@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -140,6 +140,13 @@ public:
         }
 
         db::LEFDEFReader reader (stream);
+
+        //  Add the LEF files specified explicitly
+        db::LEFDEFReaderOptions *lefdef_options = dynamic_cast<db::LEFDEFReaderOptions *> (options.get_options (reader.format ()));
+        auto lef_files = lefdef_options->lef_files ();
+        lef_files.insert (lef_files.end (), data.lef_files.begin (), data.lef_files.end ());
+        lefdef_options->set_lef_files (lef_files);
+
         reader.read_lefdef (*layout, options, import_lef);
 
         lay::LayoutView *view = lay::LayoutView::current ();
