@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -277,145 +277,161 @@ static void run_test2 (tl::TestBase *_this, const char *lef_dir, const char *fil
   }
 }
 
-TEST(1)
+TEST(lef1)
 {
   run_test (_this, "lef1", "lef:in.lef", 0, default_options ());
 }
 
-TEST(2)
+TEST(lef2)
 {
   //  Also tests ability of plugin to properly read LEF
   run_test (_this, "lef2", "read:in.lef", "au.oas.gz", default_options ());
 }
 
-TEST(3)
+TEST(lef3)
 {
   db::LEFDEFReaderOptions options = default_options ();
   options.set_cell_outline_layer ("OUTLINE (2/0)");
   run_test (_this, "lef3", "lef:in.lef", "au.oas.gz", options);
 }
 
-TEST(4)
+TEST(lef4)
 {
   run_test (_this, "lef4", "lef:in.lef", 0, default_options ());
 }
 
-TEST(5)
+TEST(lef5)
 {
   run_test (_this, "lef5", "lef:in.lef", 0, default_options ());
 }
 
-TEST(6)
+TEST(lef6)
 {
   run_test (_this, "lef6", "lef:in.lef", 0, default_options ());
 }
 
-TEST(7)
+TEST(lef7)
 {
   db::LEFDEFReaderOptions options = default_options ();
   options.set_cell_outline_layer ("OUTLINE (8/0)");
   run_test (_this, "lef7", "lef:in_tech.lef+lef:in.lef", "au.oas.gz", options);
 }
 
-TEST(10)
+TEST(lef8)
 {
-  run_test (_this, "def1", "lef:in.lef+def:in.def", "au2.oas.gz", default_options ());
+  //  this is rather a smoke test and throws a number of warnings
+  //  (complete example)
+  run_test (_this, "lef8", "lef:tech.lef+lef:a.lef", "au.oas.gz", default_options ());
 }
 
-TEST(11)
+TEST(def1)
+{
+  run_test (_this, "def1", "lef:in.lef+def:in.def", "au2_2.oas.gz", default_options ());
+}
+
+TEST(def2)
 {
   db::LEFDEFReaderOptions options = default_options ();
   options.set_cell_outline_layer ("OUTLINE (10/0)");
-  run_test (_this, "def2", "lef:0.lef+lef:1.lef+def:in.def.gz", "au.oas.gz", options);
+  run_test (_this, "def2", "lef:0.lef+lef:1.lef+def:in.def.gz", "au_3.oas.gz", options);
 }
 
-TEST(12)
+TEST(def3)
 {
   db::LEFDEFReaderOptions options = default_options ();
   options.set_cell_outline_layer ("OUTLINE (13/0)");
-  run_test (_this, "def3", "lef:in.lef+def:in.def", "au.oas.gz", options);
+  run_test (_this, "def3", "lef:in.lef+def:in.def", "au_2.oas.gz", options);
 }
 
-TEST(13)
+TEST(def4)
 {
-  run_test (_this, "def4", "lef:in.lef+def:in.def", "au2.oas.gz", default_options ());
+  run_test (_this, "def4", "lef:in.lef+def:in.def", "au2_2.oas.gz", default_options ());
 }
 
-TEST(14)
+TEST(def5)
 {
   run_test (_this, "def5", "lef:in.lef+def:in.def", "au.oas.gz", default_options ());
 }
 
-TEST(15)
+TEST(def6)
 {
-  run_test (_this, "def6", "lef:cells.lef+lef:tech.lef+def:in.def.gz", "au-new.oas.gz", default_options ());
+  run_test (_this, "def6", "lef:cells.lef+lef:tech.lef+def:in.def.gz", "au-new_2.oas.gz", default_options ());
 }
 
-TEST(16)
+TEST(def7)
 {
   db::LEFDEFReaderOptions options = default_options ();
   options.set_placement_blockage_layer ("PLACEMENT_BLK (11/0)");
-  run_test (_this, "def7", "lef:cells.lef+lef:tech.lef+def:in.def.gz", "au-new.oas.gz", options);
+  run_test (_this, "def7", "lef:cells.lef+lef:tech.lef+def:in.def.gz", "au-new_2.oas.gz", options);
 
   options.set_placement_blockage_layer ("PLACEMENT_BLK (60/0)");
-  run_test (_this, "def7", "map:in.map+lef:cells.lef+lef:tech.lef+def:in.def.gz", "au2_with_map_file-new.oas.gz", options);
+  run_test (_this, "def7", "map:in.map+lef:cells.lef+lef:tech.lef+def:in.def.gz", "au2_with_map_file-new_2.oas.gz", options);
 }
 
-TEST(17)
+TEST(def8)
 {
   run_test (_this, "def8", "lef:tech.lef+def:in.def", "au.oas.gz", default_options ());
 }
 
-TEST(18)
+TEST(def9)
 {
   db::LEFDEFReaderOptions options = default_options ();
   options.set_separate_groups (true);
-  run_test (_this, "def9", "lef:tech.lef+lef:cells_modified.lef+def:in.def", "au-new.oas.gz", options);
+  run_test (_this, "def9", "lef:tech.lef+lef:cells_modified.lef+def:in.def", "au-new_2.oas.gz", options);
 
-  run_test (_this, "def9", "lef:tech.lef+lef:cells_modified.lef+def:in.def", "au_nogroups-new.oas.gz", default_options ());
+  run_test (_this, "def9", "lef:tech.lef+lef:cells_modified.lef+def:in.def", "au_nogroups-new_2.oas.gz", default_options ());
 }
 
-TEST(19)
+TEST(def10)
 {
   db::LEFDEFReaderOptions opt = default_options ();
   opt.set_cell_outline_layer ("OUTLINE (2/0)");
   run_test (_this, "def10", "def:in.def", "au.oas.gz", opt);
 }
 
-TEST(20)
+TEST(def11)
 {
   db::LEFDEFReaderOptions opt = default_options ();
   opt.set_cell_outline_layer ("OUTLINE (12/0)");
   run_test (_this, "def11", "lef:test.lef+def:test.def", "au.oas.gz", opt);
 }
 
-TEST(21)
+TEST(def12)
 {
   db::LEFDEFReaderOptions opt = default_options ();
   opt.set_cell_outline_layer ("OUTLINE (20/0)");
   run_test (_this, "def12", "lef:test.lef+def:test.def", "au-new.oas.gz", opt);
 }
 
-TEST(22)
+TEST(def13)
 {
   db::LEFDEFReaderOptions opt = default_options ();
   run_test (_this, "def13", "map:test.map+lef:test.lef_5.8+def:top.def.gz", "au2.oas.gz", opt);
 }
 
-TEST(23)
+TEST(def14)
 {
   db::LEFDEFReaderOptions opt = default_options ();
   opt.set_macro_resolution_mode (1);
-  run_test (_this, "def14", "map:test.map+lef:tech.lef+lef:stdlib.lef+def:test.def", "au.oas.gz", opt);
+  run_test (_this, "def14", "map:test.map+lef:tech.lef+lef:stdlib.lef+def:test.def", "au_2.oas.gz", opt);
 }
 
-TEST(24)
+TEST(def15)
 {
   db::LEFDEFReaderOptions opt = default_options ();
   opt.set_macro_resolution_mode (1);
   run_test (_this, "def15", "map:test.map+lef:tech.lef+def:test.def", "au2.oas.gz", opt);
 }
 
+TEST(def16)
+{
+  //  this is rather a smoke test
+  //  (complete example)
+  db::LEFDEFReaderOptions opt = default_options ();
+  opt.set_macro_resolution_mode (1);
+  run_test (_this, "def16", "lef:a.lef+lef:tech.lef+def:a.def", "au_4.oas.gz", opt);
+}
+
 TEST(100)
 {
   run_test (_this, "issue-172", "lef:in.lef+def:in.def", "au.oas.gz", default_options (), false);
@@ -524,6 +540,10 @@ TEST(111_mapfile)
   options.set_map_file ("test.map");
 
   run_test (_this, "mapfile", "read:in.def", "au.oas.gz", options, false);
+
+  options.set_map_file ("test-nonames.map");
+
+  run_test (_this, "mapfile", "read:in.def", "au.oas.gz", options, false);
 }
 
 TEST(112_via_properties)
@@ -866,6 +886,15 @@ TEST(131_patternname)
   run_test (_this, "patternname", "map:v.map+lef:v.lef+def:v.def", "au.oas.gz", default_options (), false);
 }
 
+TEST(132_issue1307_pin_names)
+{
+  db::LEFDEFReaderOptions opt = default_options ();
+  opt.set_produce_pin_names (true);
+  opt.set_pin_property_name (2);
+  opt.set_cell_outline_layer ("OUTLINE (13/0)");
+  run_test (_this, "issue-1307c", "lef:in.lef+def:in.def", "au.oas", opt, false);
+}
+
 TEST(200_lefdef_plugin)
 {
   db::Layout ly;
@@ -954,3 +983,70 @@ TEST(205_lef_resistance)
   run_test (_this, "issue-1214", "read:merged.nom.lef", "au.oas.gz", lefdef_opt, false);
 }
 
+//  issue 1282
+TEST(206_lef_spacing)
+{
+  run_test (_this, "issue-1282", "read:a.lef", 0, default_options (), false);
+}
+
+//  issue-1345
+TEST(207_joined_paths)
+{
+  db::LEFDEFReaderOptions lefdef_opt = default_options ();
+  lefdef_opt.set_joined_paths (true);
+  run_test (_this, "issue-1345", "lef:in.lef+def:in.def", "au.oas.gz", lefdef_opt, false);
+
+  run_test (_this, "issue-1345", "lef:in.lef+def:in.def", "au-nojoin.oas.gz", default_options (), false);
+}
+
+//  issue-1432
+TEST(208_nets_and_rects)
+{
+  run_test (_this, "issue-1432", "map:test.map+lef:test.lef+def:test.def", "au.oas", default_options (), false);
+}
+
+//  issue-1472
+TEST(209_invalid_split_paths)
+{
+  run_test (_this, "issue-1472", "map:tech.map+lef:tech.lef.gz+def:test.def.gz", "au.oas", default_options (), false);
+}
+
+//  issue-1499
+TEST(210_overlaps)
+{
+  run_test (_this, "issue-1499", "map:tech.map+lef:tech.lef+lef:blocks.lef+def:top.def", "au.oas", default_options (), false);
+}
+
+//  issue-1531
+TEST(211_symlinks)
+{
+  db::Layout ly;
+
+  std::string fn_path (tl::testdata ());
+  fn_path += "/lefdef/issue-1531/";
+
+  db::LEFDEFReaderOptions lefdef_opt = default_options ();
+  lefdef_opt.set_map_file ("tech.map");
+  std::vector<std::string> lf;
+  lf.push_back ("tech.lef");
+  lf.push_back ("blocks.lef");
+  lefdef_opt.set_lef_files (lf);
+  lefdef_opt.set_read_lef_with_def (false);
+  db::LoadLayoutOptions opt;
+  opt.set_options (lefdef_opt);
+
+  {
+    tl::InputStream is (fn_path + "top.def");
+    db::Reader reader (is);
+    reader.read (ly, opt);
+  }
+
+  db::compare_layouts (_this, ly, fn_path + "au.oas", db::WriteOAS);
+}
+
+//  issue-1528
+TEST(212_widthtable)
+{
+  run_test (_this, "issue-1528", "map:gds.map+lef:tech.lef+def:routed.def", "au.oas", default_options (), false);
+}
+
