@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -101,7 +101,8 @@ public:
    *  @brief The constructor
    */
   OASISWriterOptions ()
-    : compression_level (2), write_cblocks (true), strict_mode (true), recompress (false), permissive (false), write_std_properties (1), subst_char ("*")
+    : compression_level (2), write_cblocks (true), strict_mode (true), recompress (false), permissive (false),
+      write_std_properties (1), subst_char ("*"), tables_at_end (false)
   {
     //  .. nothing yet ..
   }
@@ -166,6 +167,11 @@ public:
    */
   std::string subst_char;
 
+  /**
+   *  @brief Hidden option, for testing mainly: write tables at end to force forward references
+   */
+  bool tables_at_end;
+
   /** 
    *  @brief Implementation of FormatSpecificWriterOptions
    */
