@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -1420,7 +1420,7 @@ XORToolDialog::run_xor ()
 
     size_t todo_count = 0;
     XORJob::EmptyLayerHandling el_handling = XORJob::EL_optimize;
-    if (summarize) {
+    if (summarize && output_mode == OMMarkerDatabase) {
       el_handling = XORJob::EL_summarize;
     } else if (process_el) {
       el_handling = XORJob::EL_process;
