@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -151,6 +151,13 @@ PythonPtr::~PythonPtr ()
   Py_XDECREF (mp_obj);
 }
 
+PyObject *PythonPtr::release ()
+{
+  PyObject *obj = mp_obj;
+  mp_obj = NULL;
+  return obj;
+}
+
 PythonPtr::operator bool () const
 {
   return mp_obj != NULL;
