@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -32,19 +32,12 @@
 
 #include "tlUnitTest.h"
 #include "tlStream.h"
+#include "tlEnv.h"
 
 int run_pymodtest (tl::TestBase *_this, const std::string &fn)
 {
-  static std::string pypath;
-  if (pypath.empty ()) {
-    pypath = "PYTHONPATH=";
-    pypath += STRINGIFY (PYTHONPATH);
-  }
-#if defined(_WIN32)
-  _putenv (const_cast<char *> (pypath.c_str ()));
-#else
-  putenv (const_cast<char *> (pypath.c_str ()));
-#endif
+  std::string pypath = STRINGIFY (PYTHONPATH);
+  tl::set_env ("PYTHONPATH", pypath);
   tl::info << pypath;
 
   std::string fp (tl::testdata ());
@@ -71,6 +64,9 @@ int run_pymodtest (tl::TestBase *_this, const std::string &fn)
     tl::InputPipe pipe (cmd);
     tl::InputStream is (pipe);
     text = is.read_all ();
+
+    //  subprocess exits without error
+    EXPECT_EQ (pipe.wait(), 0);
   }
 
   tl::info << text;
@@ -89,6 +85,9 @@ PYMODTEST (import_db, "import_db.py")
 PYMODTEST (import_rdb, "import_rdb.py")
 PYMODTEST (import_lay, "import_lay.py")
 
+//  others
+PYMODTEST (issue1327, "issue1327.py")
+
 #if defined(HAVE_QT) && defined(HAVE_QTBINDINGS)
 
 PYMODTEST (import_QtCore, "import_QtCore.py")
