@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -24,8 +24,10 @@
 #include "tlStream.h"
 #include "tlLog.h"
 #include "tlInternational.h"
+#include "tlEnv.h"
 
 #include <cctype>
+#include <fstream>
 
 // Use this define to print debug output
 // #define FILE_UTILS_VERBOSE
@@ -52,6 +54,7 @@
 #  include <dirent.h>
 #  include <libproc.h>
 #  include <dlfcn.h>
+#  include <pwd.h>
 
 #else
 
@@ -59,6 +62,7 @@
 #  include <unistd.h>
 #  include <dirent.h>
 #  include <dlfcn.h>
+#  include <pwd.h>
 
 #endif
 
@@ -377,7 +381,7 @@ std::vector<std::string> dir_entries (const std::string &s, bool with_files, boo
 
   struct _wfinddata_t fileinfo;
 
-  intptr_t h = _wfindfirst (tl::to_wstring (s + "\\*.*").c_str (), &fileinfo);
+  intptr_t h = _wfindfirst (tl::to_wstring (s + "\\*").c_str (), &fileinfo);
   if (h != -1) {
 
     do {
@@ -480,7 +484,9 @@ bool rename_file (const std::string &path, const std::string &new_name)
 bool rm_file (const std::string &path)
 {
 #if defined(_WIN32)
-  return _wunlink (tl::to_wstring (path).c_str ()) == 0;
+  std::wstring wpath = tl::to_wstring (path);
+  _wchmod (wpath.c_str (), _S_IREAD | _S_IWRITE);
+  return _wunlink (wpath.c_str ()) == 0;
 #else
   return unlink (tl::to_local (path).c_str ()) == 0;
 #endif
@@ -581,6 +587,48 @@ cp_dir_recursive (const std::string &source, const std::string &target)
   return true;
 }
 
+bool
+mv_dir_recursive (const std::string &source, const std::string &target)
+{
+  std::vector<std::string> entries;
+  std::string path = tl::absolute_file_path (source);
+  std::string path_to = tl::absolute_file_path (target);
+
+  bool error = false;
+
+  entries = dir_entries (path, false /*without_files*/, true /*with_dirs*/);
+  for (std::vector<std::string>::const_iterator e = entries.begin (); e != entries.end (); ++e) {
+    std::string tc = tl::combine_path (path_to, *e);
+    if (! mkpath (tc)) {
+#if defined(FILE_UTILS_VERBOSE)
+      tl::error << tr ("Unable to create target directory: ") << tc;
+#endif
+      error = true;
+    } else if (! mv_dir_recursive (tl::combine_path (path, *e), tc)) {
+      error = true;
+    }
+  }
+
+  entries = dir_entries (path, true /*with_files*/, false /*without_dirs*/);
+  for (std::vector<std::string>::const_iterator e = entries.begin (); e != entries.end (); ++e) {
+    if (! tl::rename_file (tl::combine_path (path, *e), tl::combine_path (path_to, *e))) {
+#if defined(FILE_UTILS_VERBOSE)
+      tl::error << tr ("Unable to move file from ") << tl::combine_path (path, *e) << tr (" to ") << tl::combine_path (path_to, *e);
+#endif
+      error = true;
+    }
+  }
+
+  if (! tl::rm_dir (path)) {
+#if defined(FILE_UTILS_VERBOSE)
+      tl::error << tr ("Unable to remove folder ") << path;
+#endif
+    error = true;
+  }
+
+  return ! error;
+}
+
 std::string absolute_path (const std::string &s)
 {
   std::vector<std::string> parts = split_path (absolute_file_path (s));
@@ -629,7 +677,7 @@ bool chdir (const std::string &path)
 
 static std::pair<std::string, bool> absolute_path_of_existing (const std::string &s)
 {
-#if defined (_WIN32)
+#if defined(_WIN32)
 
   wchar_t *fp = _wfullpath (NULL, tl::to_wstring (s).c_str (), 0);
   if (fp == NULL) {
@@ -657,6 +705,11 @@ static std::pair<std::string, bool> absolute_path_of_existing (const std::string
 
 bool is_absolute (const std::string &s)
 {
+  //  ~ paths are always absolute, because the home directory is
+  if (s.size () > 0 && s[0] == '~') {
+    return true;
+  }
+
   std::vector<std::string> parts = split_path (s);
   if (parts.size () > 1 && is_drive (parts [0])) {
     return is_part_with_separator (parts [1]);
@@ -669,6 +722,11 @@ bool is_absolute (const std::string &s)
 
 std::string absolute_file_path (const std::string &s)
 {
+  //  ~ paths are always absolute, because the home directory is
+  if (s.size () > 0 && s[0] == '~') {
+    return get_home_path () + std::string (s, 1);
+  }
+
   std::vector<std::string> parts = split_path (s);
   if (parts.empty ()) {
     return current_dir ();
@@ -854,15 +912,41 @@ bool is_same_file (const std::string &a, const std::string &b)
 #endif
 }
 
-static std::string
-get_inst_path_internal ()
+std::string
+get_home_path ()
 {
-#ifdef _WIN32
+#if !defined(_WIN32)
+  if (tl::has_env ("HOME")) {
+    return tl::get_env ("HOME");
+  } else {
+    struct passwd *pwd = getpwuid (getuid ());
+    if (pwd) {
+      return std::string (pwd->pw_dir);
+    }
+  }
+  tl::warn << tl::to_string (tr ("Unable to get home directory (set HOME environment variable)"));
+#else
+  if (tl::has_env ("HOMEDRIVE") && tl::has_env ("HOMEPATH")) {
+    return tl::get_env ("HOMEDRIVE") + tl::get_env ("HOMEPATH");
+  } else if (tl::has_env ("HOMESHARE") && tl::has_env ("HOMEPATH")) {
+    return tl::get_env ("HOMESHARE") + tl::get_env ("HOMEPATH");
+  } else if (tl::has_env ("USERPROFILE")) {
+    return tl::get_env ("USERPROFILE");
+  }
+  tl::warn << tl::to_string (tr ("Unable to get home directory (no HOMEDRIVE/HOMEPATH, HOMESHARE/HOMEPATH or USERPROFILE environment variables)"));
+#endif
+  return std::string (".");
+}
+
+static std::string
+get_app_path_internal ()
+{
+#if defined(_WIN32)
 
   wchar_t buffer[MAX_PATH];
   int len;
   if ((len = GetModuleFileNameW (NULL, buffer, MAX_PATH)) > 0) {
-    return tl::absolute_path (tl::to_string (std::wstring (buffer)));
+    return tl::to_string (std::wstring (buffer));
   }
 
 #elif __APPLE__
@@ -871,7 +955,7 @@ get_inst_path_internal ()
   int ret = proc_pidpath (getpid (), buffer, sizeof (buffer));
   if (ret > 0) {
     //  TODO: does this correctly translate paths? (MacOS uses UTF-8 encoding with D-like normalization)
-    return tl::absolute_path (buffer);
+    return buffer;
   }
 
 #elif defined (__FreeBSD__)
@@ -880,7 +964,7 @@ get_inst_path_internal ()
   size_t len = PATH_MAX;
   const int mib[4] = {CTL_KERN, KERN_PROC, KERN_PROC_PATHNAME, -1};
   if (sysctl(&mib[0], 4, &path, &len, NULL, 0) == 0) {
-    return tl::absolute_path(path);
+    return path;
   }
   return "";
 
@@ -888,7 +972,7 @@ get_inst_path_internal ()
 
   std::string pf = tl::sprintf ("/proc/%d/exe", getpid ());
   if (tl::file_exists (pf)) {
-    return tl::absolute_path (pf);
+    return pf;
   }
 
 #endif
@@ -900,11 +984,21 @@ get_inst_path ()
 {
   static std::string s_inst_path;
   if (s_inst_path.empty ()) {
-    s_inst_path = get_inst_path_internal ();
+    s_inst_path = tl::absolute_path (get_app_path_internal ());
   }
   return s_inst_path;
 }
 
+std::string
+get_app_path ()
+{
+  static std::string s_app_path;
+  if (s_app_path.empty ()) {
+    s_app_path = get_app_path_internal ();
+  }
+  return s_app_path;
+}
+
 std::string
 get_module_path (void *addr)
 {
@@ -937,4 +1031,108 @@ get_module_path (void *addr)
 #endif
 }
 
+std::string
+tmpfile (const std::string &domain)
+{
+  std::string tmp = tl::get_env ("TMPDIR");
+  if (tmp.empty ()) {
+    tmp = tl::get_env ("TMP");
+  }
+  if (tmp.empty ()) {
+#if defined(_WIN32)
+    throw tl::Exception (tl::to_string (tr ("TMP and TMPDIR not set - cannot create temporary file")));
+#else
+    tmp = "/tmp";
+#endif
+  }
+
+  std::string templ = tl::combine_path (tmp, domain + "XXXXXX");
+  char *tmpstr = strdup (templ.c_str ());
+
+#if defined(_WIN32)
+  if (_mktemp_s (tmpstr, templ.size () + 1) != 0) {
+    free (tmpstr);
+    throw tl::Exception (tl::to_string (tr ("Unable to create temporary folder name in %s")), tmp);
+  }
+
+  //  for compatibility with Linux, create the file as an empty one
+  std::ofstream os (tmpstr);
+  if (os.bad ()) {
+    throw tl::Exception (tl::to_string (tr ("Unable to create temporary folder in %s")), tmp);
+  }
+  os.close ();
+#else
+  int fd = mkstemp (tmpstr);
+  if (fd < 0) {
+    free (tmpstr);
+    throw tl::Exception (tl::to_string (tr ("Unable to create temporary folder in %s")), tmp);
+  }
+  close (fd);
+#endif
+
+  std::string res = tmpstr;
+  free (tmpstr);
+  return res;
+}
+
+TemporaryFile::TemporaryFile (const std::string &domain)
+{
+  m_path = tmpfile (domain);
+}
+
+TemporaryFile::~TemporaryFile ()
+{
+  tl::rm_file (m_path);
+}
+
+std::string
+tmpdir (const std::string &domain)
+{
+  std::string tmp = tl::get_env ("TMPDIR");
+  if (tmp.empty ()) {
+    tmp = tl::get_env ("TMP");
+  }
+  if (tmp.empty ()) {
+#if defined(_WIN32)
+    throw tl::Exception (tl::to_string (tr ("TMP and TMPDIR not set - cannot create temporary file")));
+#else
+    tmp = "/tmp";
+#endif
+  }
+
+  std::string templ = tl::combine_path (tmp, domain + "XXXXXX");
+  char *tmpstr = strdup (templ.c_str ());
+
+#if defined(_WIN32)
+  if (_mktemp_s (tmpstr, templ.size () + 1) != 0) {
+    free (tmpstr);
+    throw tl::Exception (tl::to_string (tr ("Unable to create temporary folder name in %s")), tmp);
+  }
+  if (! tl::mkdir (tmpstr)) {
+    free (tmpstr);
+    throw tl::Exception (tl::to_string (tr ("Unable to create temporary folder in %s")), tmp);
+  }
+#else
+  if (mkdtemp (tmpstr) == NULL) {
+    free (tmpstr);
+    throw tl::Exception (tl::to_string (tr ("Unable to create temporary folder in %s")), tmp);
+  }
+#endif
+
+  std::string res = tmpstr;
+  free (tmpstr);
+  return res;
+}
+
+TemporaryDirectory::TemporaryDirectory (const std::string &domain)
+{
+  m_path = tmpdir (domain);
+}
+
+TemporaryDirectory::~TemporaryDirectory ()
+{
+  tl::rm_dir_recursive (m_path);
+}
+
+
 }
