@@ -2,7 +2,7 @@
 /*
 
   KLayout Layout Viewer
-  Copyright (C) 2006-2023 Matthias Koefferlein
+  Copyright (C) 2006-2024 Matthias Koefferlein
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -210,6 +210,22 @@ inline bool safe_isspace (char c)
   return c != 0 && static_cast<unsigned char> (c) < 0x80 && isspace (c);
 }
 
+// -------------------------------------------------------------------------
+//  Utility: skip a newline
+
+bool skip_newline (const char *&cp)
+{
+  if (*cp == '\012' || *cp == '\015') {
+    if (*cp == '\015' && cp[1] == '\012') {
+      ++cp;
+    }
+    ++cp;
+    return true;
+  } else {
+    return false;
+  }
+}
+
 // -------------------------------------------------------------------------
 //  Utility: a strtod version that is independent of the locale
 
