@@ -1,7 +1,7 @@
 # encoding: UTF-8
 
 # KLayout Layout Viewer
-# Copyright (C) 2006-2023 Matthias Koefferlein
+# Copyright (C) 2006-2024 Matthias Koefferlein
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@ class KLayoutMain_TestClass < TestBase
   def klayout_bin
     # special location for MacOS
     file = File.join(RBA::Application::instance.inst_path, "klayout.app", "Contents", "MacOS", "klayout")
-    if !File.exists?(file)
+    if !File.exist?(file)
       file = File.join(RBA::Application::instance.inst_path, "klayout")
     end
     return file
@@ -191,6 +191,26 @@ class KLayoutMain_TestClass < TestBase
 
   end
 
+  def test_12
+
+    # Application.exit(0) - Python
+    out = `#{self.klayout_bin} -z -r #{File.join(File.dirname(__FILE__), "test12.py")} 2>&1`
+    assert_equal(out, "Before exit()\n")
+
+    # Application.exit(0) - Ruby
+    out = `#{self.klayout_bin} -z -r #{File.join(File.dirname(__FILE__), "test12.rb")} 2>&1`
+    assert_equal(out, "Before exit()\n")
+
+    # sys.exit(0) - Python
+    out = `#{self.klayout_bin} -z -r #{File.join(File.dirname(__FILE__), "test12s.py")} 2>&1`
+    assert_equal(out, "Before exit()\n")
+
+    # quit() - Python (issue #1565)
+    out = `#{self.klayout_bin} -z -r #{File.join(File.dirname(__FILE__), "test12q.py")} 2>&1`
+    assert_equal(out, "Before quit()\n")
+
+  end
+
 end
 
 load("test_epilogue.rb")
