@@ -1,5 +1,5 @@
 # KLayout Layout Viewer
-# Copyright (C) 2006-2023 Matthias Koefferlein
+# Copyright (C) 2006-2024 Matthias Koefferlein
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -41,6 +41,12 @@ class BasicTest(unittest.TestCase):
     v.read(os.path.join(os.path.dirname(__file__), "..", "gds", "t10.gds"))
     self.assertEqual(v.top_cell().name, "RINGO")
 
+  def test_4(self):
+    # gds2_text plugin loaded? (issue #1393)
+    v = db.Layout()
+    v.read(os.path.join(os.path.dirname(__file__), "..", "gds2_txt", "read.txt"))
+    self.assertEqual(v.top_cell().name, "RINGO")
+
 # run unit tests
 if __name__ == '__main__':
   suite = unittest.TestSuite()
