@@ -1,7 +1,7 @@
 # encoding: UTF-8
 
 # KLayout Layout Viewer
-# Copyright (C) 2006-2023 Matthias Koefferlein
+# Copyright (C) 2006-2024 Matthias Koefferlein
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -111,6 +111,9 @@ class DBEdge_TestClass < TestBase
     assert_equal( a.intersect?( RBA::DEdge::new( -1, 11, 1, 15 ) ), false )
     assert_equal( a.distance( RBA::DPoint::new( 3, 3 ) ), 1.0 )
     assert_equal( a.distance( RBA::DPoint::new( 3, 1 ) ), -1.0 )
+    assert_equal( a.euclidian_distance( RBA::DPoint::new( 3, 3 ) ), 1.0 )
+    assert_equal( a.euclidian_distance( RBA::DPoint::new( 3, 1 ) ), 1.0 )
+    assert_equal( a.euclidian_distance( RBA::DPoint::new( -3, 2 ) ), 1.0 )
     assert_equal( a.distance_abs( RBA::DPoint::new( 3, 3 ) ), 1.0 )
     assert_equal( a.distance_abs( RBA::DPoint::new( 3, 1 ) ), 1.0 )
     assert_equal( a.side_of( RBA::DPoint::new( 3, 3 ) ), 1 )
@@ -224,10 +227,13 @@ class DBEdge_TestClass < TestBase
     assert_equal( a.intersection_point( RBA::Edge::new( RBA::Point::new( -1, -1 ), RBA::Point::new( 1, 5 ) ) ).to_s, "0,2" )
     assert_equal( a.intersection_point( RBA::Edge::new( RBA::Point::new( -1, 3 ), RBA::Point::new( 1, 5 ) ) ) == nil, true )
     assert_equal( a.intersect?( RBA::Edge::new( RBA::Point::new( -1, 11 ), RBA::Point::new( 1, 15 ) ) ), false )
-    assert_equal( a.distance( RBA::Point::new( 3, 3 ) ), 1.0 )
-    assert_equal( a.distance( RBA::Point::new( 3, 1 ) ), -1.0 )
-    assert_equal( a.distance_abs( RBA::Point::new( 3, 3 ) ), 1.0 )
-    assert_equal( a.distance_abs( RBA::Point::new( 3, 1 ) ), 1.0 )
+    assert_equal( a.distance( RBA::Point::new( 3, 3 ) ), 1 )
+    assert_equal( a.distance( RBA::Point::new( 3, 1 ) ), -1 )
+    assert_equal( a.euclidian_distance( RBA::Point::new( 3, 4 ) ), 2 )
+    assert_equal( a.euclidian_distance( RBA::Point::new( 3, 0 ) ), 2 )
+    assert_equal( a.euclidian_distance( RBA::Point::new( -4, 2 ) ), 2 )
+    assert_equal( a.distance_abs( RBA::Point::new( 3, 3 ) ), 1 )
+    assert_equal( a.distance_abs( RBA::Point::new( 3, 1 ) ), 1 )
     assert_equal( a.side_of( RBA::Point::new( 3, 3 ) ), 1 )
     assert_equal( a.side_of( RBA::Point::new( 3, 1 ) ), -1 )
     
