@@ -1,7 +1,7 @@
 # encoding: UTF-8
 
 # KLayout Layout Viewer
-# Copyright (C) 2006-2023 Matthias Koefferlein
+# Copyright (C) 2006-2024 Matthias Koefferlein
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -315,6 +315,8 @@ class DBEdges_TestClass < TestBase
 
     r = r1 + r2
     assert_equal(csort(r.to_s), csort("(0,0;100,0);(50,0;200,0)"))
+    r = r1.join(r2)
+    assert_equal(csort(r.to_s), csort("(0,0;100,0);(50,0;200,0)"))
     assert_equal(r.merged.to_s, "(0,0;200,0)")
     r.merge
     assert_equal(r.is_merged?, true)
@@ -322,9 +324,14 @@ class DBEdges_TestClass < TestBase
     r = r1.dup
     r += r2
     assert_equal(csort(r.to_s), csort("(0,0;100,0);(50,0;200,0)"))
+    r = r1.dup
+    r.join_with(r2)
+    assert_equal(csort(r.to_s), csort("(0,0;100,0);(50,0;200,0)"))
 
     r = r1 | r2
     assert_equal(r.to_s, "(0,0;200,0)")
+    r = r1.or(r2)
+    assert_equal(r.to_s, "(0,0;200,0)")
     assert_equal(r.merged.to_s, "(0,0;200,0)")
     r.merge
     assert_equal(r.is_merged?, true)
@@ -332,9 +339,14 @@ class DBEdges_TestClass < TestBase
     r = r1.dup
     r |= r2
     assert_equal(r.to_s, "(0,0;200,0)")
+    r = r1.dup
+    r.or_with(r2)
+    assert_equal(r.to_s, "(0,0;200,0)")
 
     r = r1 & r2
     assert_equal(r.to_s, "(50,0;100,0)")
+    r = r1.and(r2)
+    assert_equal(r.to_s, "(50,0;100,0)")
     assert_equal(r.is_merged?, true)
     r = r1.andnot(r2)[0]
     assert_equal(r.to_s, "(50,0;100,0)")
@@ -342,9 +354,14 @@ class DBEdges_TestClass < TestBase
     r = r1.dup
     r &= r2
     assert_equal(r.to_s, "(50,0;100,0)")
+    r = r1.dup
+    r.and_with(r2)
+    assert_equal(r.to_s, "(50,0;100,0)")
 
     r = r1 - r2
     assert_equal(r.to_s, "(0,0;50,0)")
+    r = r1.not(r2)
+    assert_equal(r.to_s, "(0,0;50,0)")
     assert_equal(r.is_merged?, true)
     r = r1.andnot(r2)[1]
     assert_equal(r.to_s, "(0,0;50,0)")
@@ -352,13 +369,21 @@ class DBEdges_TestClass < TestBase
     r = r1.dup
     r -= r2
     assert_equal(r.to_s, "(0,0;50,0)")
+    r = r1.dup
+    r.not_with(r2)
+    assert_equal(r.to_s, "(0,0;50,0)")
 
     r = r1 ^ r2
     assert_equal(csort(r.to_s), csort("(0,0;50,0);(100,0;200,0)"))
+    r = r1.xor(r2)
+    assert_equal(csort(r.to_s), csort("(0,0;50,0);(100,0;200,0)"))
     assert_equal(r.is_merged?, true)
     r = r1.dup
     r ^= r2
     assert_equal(csort(r.to_s), csort("(0,0;50,0);(100,0;200,0)"))
+    r = r1.dup
+    r.xor_with(r2)
+    assert_equal(csort(r.to_s), csort("(0,0;50,0);(100,0;200,0)"))
 
   end
 
