@@ -1,7 +1,7 @@
 # encoding: UTF-8
 
 # KLayout Layout Viewer
-# Copyright (C) 2006-2023 Matthias Koefferlein
+# Copyright (C) 2006-2024 Matthias Koefferlein
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -274,34 +274,54 @@ class DBRegion_TestClass < TestBase
     r2 = RBA::Region::new(RBA::Box::new(-10, -20, 80, 160))
 
     assert_equal((r1 & r2).to_s, "(10,20;10,160;80,160;80,20)")
+    assert_equal((r1.and(r2)).to_s, "(10,20;10,160;80,160;80,20)")
     assert_equal(r1.andnot(r2).count, 2)
     assert_equal(r1.andnot(r2)[0].to_s, "(10,20;10,160;80,160;80,20)")
     rr = r1.dup
     rr &= r2
     assert_equal(rr.to_s, "(10,20;10,160;80,160;80,20)")
+    rr = r1.dup
+    rr.and_with(r2)
+    assert_equal(rr.to_s, "(10,20;10,160;80,160;80,20)")
     
     assert_equal((r1 - r2).to_s, "(80,20;80,160;10,160;10,200;100,200;100,20)")
+    assert_equal((r1.not(r2)).to_s, "(80,20;80,160;10,160;10,200;100,200;100,20)")
     assert_equal(r1.andnot(r2)[1].to_s, "(80,20;80,160;10,160;10,200;100,200;100,20)")
     rr = r1.dup
     rr -= r2
     assert_equal(rr.to_s, "(80,20;80,160;10,160;10,200;100,200;100,20)")
+    rr = r1.dup
+    rr.not_with(r2)
+    assert_equal(rr.to_s, "(80,20;80,160;10,160;10,200;100,200;100,20)")
     
     assert_equal((r1 ^ r2).to_s, "(-10,-20;-10,160;10,160;10,200;100,200;100,20;80,20;80,-20/10,20;80,20;80,160;10,160)")
+    assert_equal((r1.xor(r2)).to_s, "(-10,-20;-10,160;10,160;10,200;100,200;100,20;80,20;80,-20/10,20;80,20;80,160;10,160)")
     r1.min_coherence = true
     assert_equal(csort((r1 ^ r2).to_s), csort("(-10,-20;-10,160;10,160;10,20;80,20;80,-20);(80,20;80,160;10,160;10,200;100,200;100,20)"))
     rr = r1.dup
     rr ^= r2
     assert_equal(csort(rr.to_s), csort("(-10,-20;-10,160;10,160;10,20;80,20;80,-20);(80,20;80,160;10,160;10,200;100,200;100,20)"))
+    rr = r1.dup
+    rr.xor_with(r2)
+    assert_equal(csort(rr.to_s), csort("(-10,-20;-10,160;10,160;10,20;80,20;80,-20);(80,20;80,160;10,160;10,200;100,200;100,20)"))
     
     assert_equal(csort((r1 + r2).to_s), csort("(10,20;10,200;100,200;100,20);(-10,-20;-10,160;80,160;80,-20)"))
+    assert_equal(csort((r1.join(r2)).to_s), csort("(10,20;10,200;100,200;100,20);(-10,-20;-10,160;80,160;80,-20)"))
     rr = r1.dup
     rr += r2
     assert_equal(csort(rr.to_s), csort("(10,20;10,200;100,200;100,20);(-10,-20;-10,160;80,160;80,-20)"))
+    rr = r1.dup
+    rr.join_with(r2)
+    assert_equal(csort(rr.to_s), csort("(10,20;10,200;100,200;100,20);(-10,-20;-10,160;80,160;80,-20)"))
     
     assert_equal((r1 | r2).to_s, "(-10,-20;-10,160;10,160;10,200;100,200;100,20;80,20;80,-20)")
+    assert_equal((r1.or(r2)).to_s, "(-10,-20;-10,160;10,160;10,200;100,200;100,20;80,20;80,-20)")
     rr = r1.dup
     rr |= r2
     assert_equal(rr.to_s, "(-10,-20;-10,160;10,160;10,200;100,200;100,20;80,20;80,-20)")
+    rr = r1.dup
+    rr.or_with(r2)
+    assert_equal(rr.to_s, "(-10,-20;-10,160;10,160;10,200;100,200;100,20;80,20;80,-20)")
     
     assert_equal((r1 + r2).sized(10).to_s, "(-20,-30;-20,170;0,170;0,210;110,210;110,10;90,10;90,-30)")
     rr = (r1 | r2).dup
