@@ -1,7 +1,7 @@
 # encoding: UTF-8
 
 # KLayout Layout Viewer
-# Copyright (C) 2006-2023 Matthias Koefferlein
+# Copyright (C) 2006-2024 Matthias Koefferlein
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -186,7 +186,7 @@ class LAYLayoutView_TestClass < TestBase
     view.set_config("search-range-box", "5")
     view.select_from(RBA::DBox::new(-1.0, -1.0, 1.0, 1.0))
     assert_equal(selection_changed, 1)
-    assert_equal(view.selection_size, 4)
+    assert_equal(view.selection_size, 2)
     assert_equal(view.has_selection?, true)
 
     view.select_from(RBA::DPoint::new(0, 0), RBA::LayoutView::Invert)
@@ -550,6 +550,26 @@ class LAYLayoutView_TestClass < TestBase
 
   end
 
+  # issue-1533
+  def test_8
+
+    if !RBA.constants.member?(:Application)
+      return
+    end
+
+    app = RBA::Application.instance
+    mw = app.main_window
+    mw.close_all
+    mw.create_view
+
+    ly = RBA::Layout::new
+    mw.current_view.show_layout(ly, false)
+
+    # was crashing
+    mw.current_view.show_layout(ly, false)
+
+  end
+
 end
 
 load("test_epilogue.rb")
